/*
================================================================================
 File: converter.c
--------------------------------------------------------------------------------
 Author:        Piotr Wojtowicz
 Purpose:       USBD RS232C converter
 Compiler:      ANSI C
 Hardware:
 Doc:
--------------------------------------------------------------------------------
 Includes:
--------------------------------------------------------------------------------
 Comments:                      UWAGA!!!
                Oprogramowanie to udostpnione jest do celw pogldowych,
                nie jest przetestowane pod ktem bezpieczestwa ani spenienia
                jakichkolwiek norm wymaganych aby urzdzenie uywajce go
                mogo by dopuszczone do uytku, dlatego uywasz go
                na wasn odpowiedzialno a jego autor nie ponosi adnego
                typu odpowiedzialnoci za jakiekolwiek skutki jego uycia.
                Jeeli nie zgadzasz si z powyszym warunkiem prosz
                o nieuywanie tego oprogramowania.
================================================================================
*/

#include "converter.h"
#include "stm32_usart_init.h"


// -----------------------------------------------------------------------------
//  converter objects
// -----------------------------------------------------------------------------

/**
 * USBDC
 * struktura opisujaca konfiguracje
 * musi byc utworzona na stale poniewaz jest uzywana
 * caly czas podczas uzywania USBD.
 */
volatile uint8_t USBD_irq_state;
static USBDC_params usbdc;
static uint8_t usbd_conv_descriptor[256];
uint8_t trash[USBDEP_FS_BULK_MAX_POSSIBLE_PACKET_SIZE];



static converter_params *conv_usart1;
static converter_params *conv_usart2;
static converter_params *conv_usart3;


#if(USE_VCOM1)
static converter_params vcom1_params;
#endif
#if(USE_VCOM2)
static converter_params vcom2_params;
#endif




/**
 * Manufacturer_String_Descriptor
 * String - jest to jeden z elementow USBD stosowany opcjonalnie
 */
static const uint16_t Manufacturer_String_Descriptor[] = {
	0x0328,    			// bLength, bDescriptorType: string descriptor
	'p',
	'i',
	'o',
	't',
	'r',
	'e',
	'k',
	'l',
	'c',
	'6',
	'0',
	' ',
	'K',
	'N',
	'E',
	' ',
	'A',
	'G',
	'H'
}; /* Manufacturer_String_Descriptor */


/**
 * Product_String_Descriptor
 * String - jest to jeden z elementow USBD stosowany opcjonalnie
 */
static const uint16_t Product_String_Descriptor[] = {
	0x0322,    			// bLength, bDescriptorType: string descriptor
	'V',
	'I',
	'R',
	'T',
	'U',
	'A',
	'L',
	' ',
	'C',
	'O',
	'M',
	' ',
	'P',
	'O',
	'R',
	'T'
}; /* Product_String_Descriptor */


/**
 * SerialNumber_String_Descriptor
 * String - jest to jeden z elementow USBD stosowany opcjonalnie
 */
static uint16_t SerialNumber_String_Descriptor[25];


/**
 * hex
 * przelicza liczbe w HEX (mlodsza polowka bajtu) na znak kodu ASCI
 */
static uint8_t hex (uint8_t l) {		//przelicza liczbe w HEX
	l &= 0xF;			// (mlodsza polowka bajtu)
	return("0123456789ABCDEF"[l]);
}


/**
 * SerialNumber_fill
 * wypelnia string dla USBD pobierajac unikalny numer procesora
 */
static void SerialNumber_fill(void) {
	SerialNumber_String_Descriptor[0] = 0x0332;

	uint8_t ID;
	uint8_t *p_ID = (uint8_t*)0x1FFFF7E8;
	uint8_t i;
	uint16_t *p_string = &SerialNumber_String_Descriptor[1];

	// w ponizszych liniach pobierany jest unikalny numer
	// ID procesora nadany przez producenta
	// i nastepnie umieszczany jest w stringu
	// SerialNumber bedacym parametrem USB device
	for(i = 0; i < 12; i++) {
		ID = p_ID[i];
		p_string[i << 1] = (uint16_t)hex(ID & 0xF);
		p_string[(i << 1) + 1] = (uint16_t)hex((ID >> 4) & 0xF);
	}
}









void EXIT_IRQ(void) {
	USBD_ENTER_CRITICAL();

	USBD_irq_state = 0;

#if(USE_VCOM1)
	if(vcom1_params.irq_wfr_state) {
		vcom1_params.irq_wfr_state = 0;
		CDC_VCOM_RXD_DATA_READY(&(vcom1_params.vcom));
	}
#endif

#if(USE_VCOM2)
	if(vcom2_params.irq_wfr_state) {
		vcom2_params.irq_wfr_state = 0;
		CDC_VCOM_RXD_DATA_READY(&(vcom2_params.vcom));
	}
#endif

	USBD_EXIT_CRITICAL();
}




//	-----------------------------------------------------------------------------
/**
 *  vCOM_Rx
 *  Interrupt handler 
 *  cdc_vcom layer generate this function when endpoint receives some Rx data (from Host)
 */
//	-----------------------------------------------------------------------------
static uint32_t vCOM_Rx_handler (CDC_VCOM_params *vcom, uint8_t* buffer, uint32_t received_size) {
	uint32_t ret;
	uint32_t size;
	converter_params *conv = (converter_params*)CDC_VCOM_GET_UPPER_LAYER_PVOID(vcom);

	size = BUF_GetFreeSpace(&(conv->buf.to_usart));
	if(size < received_size) {
		BUF_Read(&(conv->buf.to_usart), trash, received_size - size);
	}
	BUF_Write(&(conv->buf.to_usart), buffer, received_size);

	size = BUF_GetFreeSpace(&(conv->buf.to_usart));
	if(size >= USBDEP_FS_BULK_MAX_POSSIBLE_PACKET_SIZE) {
		ret = 0;
	}
	else {
		ret = 1;
	}
	USART_ITConfig(conv->USARTx, USART_IT_TXE, ENABLE);

//	print_string("\n\rrx");

	return(ret);
} /* vCOM_Rx */


//	-----------------------------------------------------------------------------
/**
 *  vCOM_Tx
 *  Interrupt handler 
 *  cdc_vcom layer generate this function after succesfully closed Tx (to Host) transfer
 */
 // -----------------------------------------------------------------------------
static uint32_t vCOM_Tx_handler (CDC_VCOM_params *vcom, uint8_t* buffer, uint32_t available_size) {
	uint32_t size;
	converter_params *conv = (converter_params*)CDC_VCOM_GET_UPPER_LAYER_PVOID(vcom);

	size = BUF_GetCount(&(conv->buf.to_vcom));
	if(size) {
		if(size > available_size) {
			size = available_size;
		}
		BUF_Read(&(conv->buf.to_vcom), buffer, size);
	}
	return(size);
} /* vCOM_Tx */





static void USART_Proc(converter_params *conv, USART_TypeDef* USARTx) {
	uint8_t data;
	uint32_t size;

	// check status - if there was receive event
	if (USART_GetITStatus(USARTx, USART_IT_RXNE) != RESET) {
		// read data from register
		data = USARTx->DR;

		// if we have free space in buffer
		if(BUF_GetFreeSpace(&(conv->buf.to_vcom))) {
			// write received data to buffer
			BUF_Write(&(conv->buf.to_vcom), &data, 1);
		}
	}

	// check status - if there was transmit event
	if (USART_GetITStatus(USARTx, USART_IT_TXE) != RESET) {
		// if we have some data in the buffer
		if(BUF_GetCount(&(conv->buf.to_usart))) {
			// read one byte
			BUF_Read(&(conv->buf.to_usart), &data, 1);

			// write byte to data register to transmit it
			USARTx->DR = data;
		}
		// we don't have any data in the buffer
		else {
			// disable transmitter
			USART_ITConfig(USARTx, USART_IT_TXE, DISABLE);
		}

		// get size of free space in the buffer
		size = BUF_GetFreeSpace(&(conv->buf.to_usart));
		// if free size is equal or bigger than size
		// of USB OUT Endpoint buffer
		if(size >= USBDEP_FS_BULK_MAX_POSSIBLE_PACKET_SIZE) {
			USBD_ENTER_CRITICAL();
			if(USBD_irq_state == 0) {
				// enable receiving data through USB OUT Pipe
				CDC_VCOM_RXD_DATA_READY(&(conv->vcom));
			}
			else {
				conv->irq_wfr_state = 1;
			}
			USBD_EXIT_CRITICAL();
		}
	}
}





void USART1_IRQHandler(void) {
	USART_Proc(conv_usart1, USART1);
}

void USART2_IRQHandler(void) {
	USART_Proc(conv_usart2, USART2);
}

void USART3_IRQHandler(void) {
	USART_Proc(conv_usart3, USART3);
}





void converter_rts_irq (CDC_VCOM_params *vcom) {
	converter_params *conv = (converter_params*)CDC_VCOM_GET_UPPER_LAYER_PVOID(vcom);
	if(CDC_VCOM_GET_RTS(vcom)) {
		if(conv->usart_num == 1) {
			GPIO_SetBits(USART1_RTS_PORT, USART1_RTS_PIN);
		}
		else if(conv->usart_num == 2) {
			GPIO_SetBits(USART2_RTS_PORT, USART2_RTS_PIN);
		}
		else if(conv->usart_num == 3) {
			GPIO_SetBits(USART3_RTS_PORT, USART3_RTS_PIN);
		}
	}
	else {
		if(conv->usart_num == 1) {
			GPIO_ResetBits(USART1_RTS_PORT, USART1_RTS_PIN);
		}
		else if(conv->usart_num == 2) {
			GPIO_ResetBits(USART2_RTS_PORT, USART2_RTS_PIN);
		}
		else if(conv->usart_num == 3) {
			GPIO_ResetBits(USART3_RTS_PORT, USART3_RTS_PIN);
		}
	}
}


void converter_dtr_irq (CDC_VCOM_params *vcom) {
	converter_params *conv = (converter_params*)CDC_VCOM_GET_UPPER_LAYER_PVOID(vcom);
	if(CDC_VCOM_GET_DTR(vcom)) {
		if(conv->usart_num == 1) {
			GPIO_SetBits(USART1_DTR_PORT, USART1_DTR_PIN);
		}
		else if(conv->usart_num == 2) {
			GPIO_SetBits(USART2_DTR_PORT, USART2_DTR_PIN);
		}
		else if(conv->usart_num == 3) {
			GPIO_SetBits(USART3_DTR_PORT, USART3_DTR_PIN);
		}
	}
	else {
		if(conv->usart_num == 1) {
			GPIO_ResetBits(USART1_DTR_PORT, USART1_DTR_PIN);
		}
		else if(conv->usart_num == 2) {
			GPIO_ResetBits(USART2_DTR_PORT, USART2_DTR_PIN);
		}
		else if(conv->usart_num == 3) {
			GPIO_ResetBits(USART3_DTR_PORT, USART3_DTR_PIN);
		}
	}
}




void converter_usarts_signals_scan(void) {
	uint8_t pin_state;

	if(conv_usart1) {
		if(conv_usart1->usart_activity) {
			pin_state = GPIO_ReadInputDataBit(USART1_RING_PORT, USART1_RING_PIN);
			if(conv_usart1->ring_state != pin_state) {
				conv_usart1->ring_state = pin_state;
				if(pin_state) {
					CDC_VCOM_SET_RING(&(conv_usart1->vcom));
				}
				else {
					CDC_VCOM_CLR_RING(&(conv_usart1->vcom));
				}
			}
			pin_state = GPIO_ReadInputDataBit(USART1_DCD_PORT, USART1_DCD_PIN);
			if(conv_usart1->dcd_state != pin_state) {
				conv_usart1->dcd_state = pin_state;
				if(pin_state) {
					CDC_VCOM_SET_DCD(&(conv_usart1->vcom));
				}
				else {
					CDC_VCOM_CLR_DCD(&(conv_usart1->vcom));
				}
			}
			pin_state = GPIO_ReadInputDataBit(USART1_DSR_PORT, USART1_DSR_PIN);
			if(conv_usart1->dsr_state != pin_state) {
				conv_usart1->dsr_state = pin_state;
				if(pin_state) {
					CDC_VCOM_SET_DSR(&(conv_usart1->vcom));
				}
				else {
					CDC_VCOM_CLR_DSR(&(conv_usart1->vcom));
				}
			}
		}
	}

	if(conv_usart2) {
		if(conv_usart2->usart_activity) {
			pin_state = GPIO_ReadInputDataBit(USART2_RING_PORT, USART2_RING_PIN);
			if(conv_usart2->ring_state != pin_state) {
				conv_usart2->ring_state = pin_state;
				if(pin_state) {
					CDC_VCOM_SET_RING(&(conv_usart2->vcom));
				}
				else {
					CDC_VCOM_CLR_RING(&(conv_usart2->vcom));
				}
			}
			pin_state = GPIO_ReadInputDataBit(USART2_DCD_PORT, USART2_DCD_PIN);
			if(conv_usart2->dcd_state != pin_state) {
				conv_usart2->dcd_state = pin_state;
				if(pin_state) {
					CDC_VCOM_SET_DCD(&(conv_usart2->vcom));
				}
				else {
					CDC_VCOM_CLR_DCD(&(conv_usart2->vcom));
				}
			}
			pin_state = GPIO_ReadInputDataBit(USART2_DSR_PORT, USART2_DSR_PIN);
			if(conv_usart2->dsr_state != pin_state) {
				conv_usart2->dsr_state = pin_state;
				if(pin_state) {
					CDC_VCOM_SET_DSR(&(conv_usart2->vcom));
				}
				else {
					CDC_VCOM_CLR_DSR(&(conv_usart2->vcom));
				}
			}
		}
	}

	if(conv_usart3) {
		if(conv_usart3->usart_activity) {
			pin_state = GPIO_ReadInputDataBit(USART3_RING_PORT, USART3_RING_PIN);
			if(conv_usart3->ring_state != pin_state) {
				conv_usart3->ring_state = pin_state;
				if(pin_state) {
					CDC_VCOM_SET_RING(&(conv_usart3->vcom));
				}
				else {
					CDC_VCOM_CLR_RING(&(conv_usart3->vcom));
				}
			}
			pin_state = GPIO_ReadInputDataBit(USART3_DCD_PORT, USART3_DCD_PIN);
			if(conv_usart3->dcd_state != pin_state) {
				conv_usart3->dcd_state = pin_state;
				if(pin_state) {
					CDC_VCOM_SET_DCD(&(conv_usart3->vcom));
				}
				else {
					CDC_VCOM_CLR_DCD(&(conv_usart3->vcom));
				}
			}
			pin_state = GPIO_ReadInputDataBit(USART3_DSR_PORT, USART3_DSR_PIN);
			if(conv_usart3->dsr_state != pin_state) {
				conv_usart3->dsr_state = pin_state;
				if(pin_state) {
					CDC_VCOM_SET_DSR(&(conv_usart3->vcom));
				}
				else {
					CDC_VCOM_CLR_DSR(&(conv_usart3->vcom));
				}
			}
		}
	}
}



static void converter_usarts_signals_init(void) {
	GPIO_InitTypeDef GPIO_InitStructure;

	GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;

	if(conv_usart1) {
		GPIO_InitStructure.GPIO_Pin = USART1_RTS_PIN;
		GPIO_InitStructure.GPIO_Mode = USART1_RTS_PIN_MODE;
		GPIO_Init(USART1_RTS_PORT, &GPIO_InitStructure);

		GPIO_InitStructure.GPIO_Pin = USART1_DTR_PIN;
		GPIO_InitStructure.GPIO_Mode = USART1_DTR_PIN_MODE;
		GPIO_Init(USART1_DTR_PORT, &GPIO_InitStructure);

		GPIO_InitStructure.GPIO_Pin = USART1_RING_PIN;
		GPIO_InitStructure.GPIO_Mode = USART1_RING_PIN_MODE;
		GPIO_Init(USART1_RING_PORT, &GPIO_InitStructure);

		GPIO_InitStructure.GPIO_Pin = USART1_DCD_PIN;
		GPIO_InitStructure.GPIO_Mode = USART1_DCD_PIN_MODE;
		GPIO_Init(USART1_DCD_PORT, &GPIO_InitStructure);

		GPIO_InitStructure.GPIO_Pin = USART1_DSR_PIN;
		GPIO_InitStructure.GPIO_Mode = USART1_DSR_PIN_MODE;
		GPIO_Init(USART1_DSR_PORT, &GPIO_InitStructure);
	}

	if(conv_usart2) {
		GPIO_InitStructure.GPIO_Pin = USART2_RTS_PIN;
		GPIO_InitStructure.GPIO_Mode = USART2_RTS_PIN_MODE;
		GPIO_Init(USART2_RTS_PORT, &GPIO_InitStructure);

		GPIO_InitStructure.GPIO_Pin = USART2_DTR_PIN;
		GPIO_InitStructure.GPIO_Mode = USART2_DTR_PIN_MODE;
		GPIO_Init(USART2_DTR_PORT, &GPIO_InitStructure);

		GPIO_InitStructure.GPIO_Pin = USART2_RING_PIN;
		GPIO_InitStructure.GPIO_Mode = USART2_RING_PIN_MODE;
		GPIO_Init(USART2_RING_PORT, &GPIO_InitStructure);

		GPIO_InitStructure.GPIO_Pin = USART2_DCD_PIN;
		GPIO_InitStructure.GPIO_Mode = USART2_DCD_PIN_MODE;
		GPIO_Init(USART2_DCD_PORT, &GPIO_InitStructure);

		GPIO_InitStructure.GPIO_Pin = USART2_DSR_PIN;
		GPIO_InitStructure.GPIO_Mode = USART2_DSR_PIN_MODE;
		GPIO_Init(USART2_DSR_PORT, &GPIO_InitStructure);
	}

	if(conv_usart3) {
		GPIO_InitStructure.GPIO_Pin = USART3_RTS_PIN;
		GPIO_InitStructure.GPIO_Mode = USART3_RTS_PIN_MODE;
		GPIO_Init(USART3_RTS_PORT, &GPIO_InitStructure);

		GPIO_InitStructure.GPIO_Pin = USART3_DTR_PIN;
		GPIO_InitStructure.GPIO_Mode = USART3_DTR_PIN_MODE;
		GPIO_Init(USART3_DTR_PORT, &GPIO_InitStructure);

		GPIO_InitStructure.GPIO_Pin = USART3_RING_PIN;
		GPIO_InitStructure.GPIO_Mode = USART3_RING_PIN_MODE;
		GPIO_Init(USART3_RING_PORT, &GPIO_InitStructure);

		GPIO_InitStructure.GPIO_Pin = USART3_DCD_PIN;
		GPIO_InitStructure.GPIO_Mode = USART3_DCD_PIN_MODE;
		GPIO_Init(USART3_DCD_PORT, &GPIO_InitStructure);

		GPIO_InitStructure.GPIO_Pin = USART3_DSR_PIN;
		GPIO_InitStructure.GPIO_Mode = USART3_DSR_PIN_MODE;
		GPIO_Init(USART3_DSR_PORT, &GPIO_InitStructure);
	}

}



static void USART_change_params(CDC_VCOM_params *vcom) {
	converter_params *conv;
	uint16_t stop_bits;
	uint16_t patity;

	if(CDC_VCOM_GET_BAUDRATE(vcom) && CDC_VCOM_GET_DATA_BITS(vcom)) {
		conv = (converter_params*)CDC_VCOM_GET_UPPER_LAYER_PVOID(vcom);

		switch(CDC_VCOM_GET_STOP_BITS(vcom)) {
			// 1 stop bit
			//case 0:  // -> default

			// 1.5 stop bits
			case 1:
				stop_bits = USART_StopBits_1_5;
				break;
			// 2 stop bits
			case 2:
				stop_bits = USART_StopBits_2;
				break;
			default:
				stop_bits = USART_StopBits_1;
				break;
		}

		switch(CDC_VCOM_GET_PARITY(vcom)) {
			// None
			//case 0:  // -> default
			// Odd
			case 1:
				patity = USART_Parity_Odd;
				break;
			// Odd
			case 2:
				patity = USART_Parity_Even;
				break;
			default:
				patity = USART_Parity_No;
				break;
		}

		configure_usart(conv->usart_num, CDC_VCOM_GET_BAUDRATE(vcom), stop_bits, patity, 1);

		conv->usart_activity = 0x55;
	}
	else {
		conv = (converter_params*)CDC_VCOM_GET_UPPER_LAYER_PVOID(vcom);

		deconfigure_usart(conv->usart_num);

		conv->usart_activity = 0;
		conv->ring_state = 0;
		conv->dcd_state = 0;
		conv->dsr_state = 0;
	}
}


static void usbd_init(void) {
	USBD_device_descriptor *usbd_dev_desc;
	CCM_session_params ccm_session;


	// ----------------------------------------------------------------------
	// Inicjalizacja konfiguracji
	// ----------------------------------------------------------------------
	// otwarcie konfiguracji do automatycznego tworzenia
	// za pomoc biblioteki CCM
	CCM_open_config(USBD_STM32, &usbdc, &ccm_session);

	// dodanie wirtualnego portu szeregowego (vCOM)
	// (lub 2 vCOM-w) do tworzonej przez nas konfiguracji
#if(USE_VCOM1)
	CDC_VCOM_add_item(&(vcom1_params.vcom), &ccm_session);
#endif
#if(USE_VCOM2)
	CDC_VCOM_add_item(&(vcom2_params.vcom), &ccm_session);
#endif

	// zamkniecie sesji CCM i zainstalowanie konfiguracji -
	// po wyjsciu z tej funkcji mamy gotowy deskryptor konfiguracji
	// w tablicy usbd_conv_descriptor oraz wypelniona i zainstalowana
	// strukture usbdc.
	CCM_close_and_install_config(usbd_conv_descriptor, &ccm_session);



	// ----------------------------------------------------------------------
	// Zmiana pol deskryptora urzadzenia (Devce Descriptor)
	// ----------------------------------------------------------------------
	usbd_dev_desc = USBD_get_dev_descriptor_pointer(&usbdc);
	// tu wypelniamy sobie parametry vendorID i productID
	usbd_dev_desc->idVendor           = 0x0483;
	usbd_dev_desc->idProduct          = 0x5740;

	// jakiej wersji USB uzywamy
	usbd_dev_desc->bcdUSB            = 0x0110;



	// ----------------------------------------------------------------------
	// Elementy opcjonalne - stringi
	// ----------------------------------------------------------------------
	// zainstalowanie stringow
	USBDS_install((uint8_t*)Manufacturer_String_Descriptor, &(usbd_dev_desc->iManufacturer), &usbdc);
	USBDS_install((uint8_t*)Product_String_Descriptor, &(usbd_dev_desc->iProduct), &usbdc);
	// przygotowujemy SerialNumber_String_Descriptor
	SerialNumber_fill();
	USBDS_install((uint8_t*)SerialNumber_String_Descriptor, &(usbd_dev_desc->iSerialNumber), &usbdc);
}


static void configure_vcom(converter_params *vcom_params, CDC_VCOM_params *vcom, uint8_t usart_num) {
	// utworzenie buforow
	BUF_CreateOverExistingArea(&(vcom_params->buf.to_usart), vcom_params->space.to_usart, TO_USART_BUF_SIZE);
	BUF_CreateOverExistingArea(&(vcom_params->buf.to_vcom), vcom_params->space.to_vcom, TO_VCOM_BUF_SIZE);

	// wyczyszczenie buforow
	BUF_Clear(&(vcom_params->buf.to_usart));
	BUF_Clear(&(vcom_params->buf.to_vcom));

	// wypelnienie pola struktury vcom dedykowanego dla wyzszej klasy
	// wskaznikiem do struktury zawierajacej parametry danego portu
	// szeregowego oraz bufory - przyspiesza to funkcje wywolywane
	// przez biblioteke cdc_vcom
	CDC_VCOM_SET_UPPER_LAYER_PVOID(vcom, vcom_params);

	// zbindowanie funkcji i tablic cache z konkretnym objektem vcom
	CDC_VCOM_CONNECT_CHANGE_PARAMS_HANDLER(vcom, USART_change_params);
	CDC_VCOM_CONNECT_RXD_HANDLER(vcom, vCOM_Rx_handler);
	CDC_VCOM_CONNECT_TXD_HANDLER(vcom, vCOM_Tx_handler);
	CDC_VCOM_CONNECT_RXD_BUFFER(vcom, vcom_params->cache.rx);
	CDC_VCOM_CONNECT_TXD_BUFFER(vcom, vcom_params->cache.tx);
	CDC_VCOM_CONNECT_DTR_HANDLER(vcom, converter_dtr_irq);
	CDC_VCOM_CONNECT_RTS_HANDLER(vcom, converter_rts_irq);

	vcom_params->irq_wfr_state = 0;
	vcom_params->usart_num = usart_num;
	vcom_params->usart_activity = 0;
	if(usart_num == 1) {
		vcom_params->USARTx = USART1;
		conv_usart1 = vcom_params;
	}
	else if(usart_num == 2) {
		vcom_params->USARTx = USART2;
		conv_usart2 = vcom_params;
	}
	else if(usart_num == 3) {
		vcom_params->USARTx = USART3;
		conv_usart3 = vcom_params;
	}
}


// -----------------------------------------------------------------------------
//  usb_rs232_converter_init 
// -----------------------------------------------------------------------------
void converter_init(void) {
/*	volatile uint32_t counter;

	for(counter = 0; counter < 1000000; counter++) {
		asm("nop");
		asm("nop");
		asm("nop");
		asm("nop");
		asm("nop");
		asm("nop");
		asm("nop");
		asm("nop");
	}//*/

	init_oprint(2);

	print_string("\n\rprint test\n\r");//*/

	USBD_irq_state = 0;

	conv_usart1 = 0;
	conv_usart2 = 0;
	conv_usart3 = 0;

	usbd_init();

#if(USE_VCOM1)
	configure_vcom(&vcom1_params, &(vcom1_params.vcom), VCOM1_USART_NUM);
#endif
#if(USE_VCOM2)
	configure_vcom(&vcom2_params, &(vcom2_params.vcom), VCOM2_USART_NUM);
#endif

	converter_usarts_signals_init();

/*
	for(counter = 0; counter < 1000000; counter++) {
		asm("nop");
		asm("nop");
		asm("nop");
		asm("nop");
		asm("nop");
		asm("nop");
		asm("nop");
		asm("nop");
	}
	print_string("\n\rprint test\n\r");

	print_string("\n\rmps:\n\r0: ");
	print_data(ccm_session.endpoint[0].max_packet_size, 4, 10);
	print_string("\n\r1: ");
	print_data(ccm_session.endpoint[1].max_packet_size, 4, 10);
	print_string("\n\r2: ");
	print_data(ccm_session.endpoint[2].max_packet_size, 4, 10);
	print_string("\n\r3: ");
	print_data(ccm_session.endpoint[3].max_packet_size, 4, 10);
	print_string("\n\r4: ");
	print_data(ccm_session.endpoint[4].max_packet_size, 4, 10);
	print_string("\n\r5: ");
	print_data(ccm_session.endpoint[5].max_packet_size, 4, 10);
	print_string("\n\r6: ");
	print_data(ccm_session.endpoint[6].max_packet_size, 4, 10);
	print_string("\n\r7: ");
	print_data(ccm_session.endpoint[7].max_packet_size, 4, 10);
	print_string("\n\r");//*/


	USBD_activate(&usbdc);
}
