/*
================================================================================
 File: oprint.c
--------------------------------------------------------------------------------
 Author:        Piotr Wojtowicz
 Purpose:       simple object print library
 Compiler:      ANSI C
 Hardware:      
 Doc:
--------------------------------------------------------------------------------
 Includes:      
--------------------------------------------------------------------------------
 Comments:                      UWAGA!!!
                Oprogramowanie to udostpnione jest do celw pogldowych,
                nie jest przetestowane pod ktem bezpieczestwa ani spenienia
                jakichkolwiek norm wymaganych aby urzdzenie uywajce go
                mogo by dopuszczone do uytku, dlatego uywasz go
                na wasn odpowiedzialno a jego autor nie ponosi adnego
                typu odpowiedzialnoci za jakiekolwiek skutki jego uycia.
                Jeeli nie zgadzasz si z powyszym warunkiem prosz
                o nieuywanie tego oprogramowania.
================================================================================
*/

#include "oprint.h"


/********************************************************************************/
/*                              Private data                                    */
/********************************************************************************/

/** 
 *  Private pointer to commands executor params
*/
static oprint_exe_params *oprint_exe = 0;

/** 
 *  Private pointer to default commands executor params
*/
static oprint_exe_params *oprint_default_exe = 0;

/** 
 *  Private pointer to output device
*/
static void *oprint_dev = 0;

/** 
 *  Private pointer to default output device
*/
static void *oprint_default_dev = 0;


/********************************************************************************/
/*                              Private functions                               */
/********************************************************************************/

/** 
 *  Gets commands executor params pointer
 *
 *  \return commands executor params pointer
*/
static oprint_exe_params *oprint_get_exe(void) {
	if(oprint_exe) {
		return(oprint_exe);
	}
	else if(oprint_default_exe) {
		return(oprint_default_exe);
	}
	else {
		return(0);
	}
}

/** 
 *  Gets device pointer
 *
 *  \return device pointer
*/
static void *oprint_get_dev(void) {
	if(oprint_dev) {
		return(oprint_dev);
	}
	else if(oprint_default_dev) {
		return(oprint_default_dev);
	}
	else {
		return(0);
	}
}

/** 
 *  Modifies string with data to be displayed
 *  
 *  \param old_buf buffer with string - the highest is zero, 1-byte
 *    lower byte represents the lowest digit, the lowest byte represents
 *    the highest digit
*/
static uint8_t oprint_parametrize_data_string(uint8_t params, uint8_t length, uint8_t *old_buf, uint8_t *new_buf) {
	uint8_t var1 = 0;
	uint8_t dot_pos;
	uint8_t index;

	// replaces '0' (zeroes) with ' ' (spaces)
	if (params & 0xF) {
		while (old_buf[var1] == '0') {
			old_buf[var1] = ' ';
			var1++;
			if(var1 == (length - 1)) {
				break;
			}
		}
	}

	// copies string from old_buf to new_buf and puts dots into string if requied
	if(params & 0xF0) {
		params = params >> 4;
		dot_pos = length % params;
		if(!dot_pos) {
			dot_pos = params;
		}
	}
	else {
		dot_pos = length + 1;
	}

	index = 0;
	while(length) {
		new_buf[index] = *old_buf;
		old_buf++;
		index++;
		length--;
		dot_pos--;
		if(!dot_pos && length) {
			if(new_buf[index-1] != ' ') {
				new_buf[index] = '.';
				index++;
			}
			dot_pos = params;
		}
	}
	new_buf[index] = 0;
	return(var1);
}


/********************************************************************************/
/*                              Public data                                     */
/********************************************************************************/

/** 
 *  Public new line data string
*/
const char endl[2] = {
	'\n',
	'\r'
};


/********************************************************************************/
/*                              Public functions                                */
/********************************************************************************/

/** 
 *  Sets device pointer
 *  
 *  \param dev pointer to output device
*/
void oprint_set_dev(void *dev) {
	oprint_dev = dev;
} /* print_set_dev */

/** 
 *  Sets default device pointer
 *  
 *  \param dev pointer to default output device
*/
void oprint_set_default_dev(void *dev) {
	oprint_default_dev = dev;
} /* print_set_default_dev */

/** 
 *  Sets pointer to commands executor params
 *  
 *  \param exe pointer to commands executor params
*/
void oprint_set_exe(oprint_exe_params *exe) {
	oprint_exe = exe;
} /* print_set_exe */

/** 
 *  Sets pointer to default commands executor params
 *  
 *  \param exe pointer to default commands executor params
*/
void oprint_set_default_exe(oprint_exe_params *exe) {
	oprint_default_exe = exe;
} /* print_set_default_exe */

/** 
 *  Converts lower 4 bits to ascii character
 *  
 *  \param lower_half_data data to be converted - just bits 0 to 3
 *     will be converted
 *  \return ascii character
*/
uint8_t oprint_uint4_to_ascii(uint8_t lower_half_data) {
	uint8_t ret = "0123456789ABCDEF"[lower_half_data & 0x0F];
	return(ret);
} /* print_uint4_to_ascii */


/** 
 *  Prints characters string into a dev device by exe
 *  command executor
 *  
 *  \param exe pointer to commands executor params
 *  \param dev pointer to output device
 *  \param string pointer string to be printed
*/
void oprint_string(oprint_exe_params *exe, void *dev, const char *string) {
	if(!exe) {
		exe = oprint_get_exe();
	}
	if(!dev) {
		dev = oprint_get_dev();
	}

	if(!exe) {
		return;
	}

	if(!exe->print_text) {
		return;
	}

	exe->print_text(dev, string);
} /* oprint_string */

/** 
 *  Prints 32-bit variable in hexadecimal format
 *  
 *  \param exe pointer to commands executor params
 *  \param dev pointer to output device
 *  \param data data to be printed
 *  \param length length of data to be printed
 *  \param params data display format params
 *     - bits 0 to 7 - this 8-bit value gives base of data
 *        possible: 2 (binary), 10 (decimal), 16 (hexadecimal)
 *     - bits 8 to 11 - if this 4-bit value is different that zero
 *        then zeroes on the begining of data will not be displayed
 *     - bits 12 to 15 - if  this 4-bit value is different that zero
 *        then in displayed data will be displayed dots. this 4-bit
 *        value indicates how many digits will be displayed between
 *        two dots
 *     an example: data = 0x00123456, params = 0x4100 | 16,
 *        length = 8; result on the display: 0x12.3456
*/
void oprint_data (oprint_exe_params *exe, void *dev, uint32_t data, uint8_t length, uint16_t params) {
	uint8_t text[33];
	uint8_t text_table[67];
	uint8_t pos_index;
	uint8_t base;

	base = (uint8_t)(params & 0xFF);
	if(!((base == 2) || (base == 10) || (base == 16))) {
		return;
	}

	if(base == 2) {
		if(length > 32) {
			length = 32;
		}
	}
	else if(base == 10) {
		if(length > 10) {
			length = 10;
		}
	}
	else if(base == 16) {
		if(length > 8) {
			length = 8;
		}
	}

	for (pos_index = 0; pos_index < length; pos_index++) {
		text[length - 1 - pos_index] = "0123456789ABCDEF"[data % base];
		data /= base;
	}
	text[length] = 0;

	params >>= 8;
	if(base == 10) {
		oprint_parametrize_data_string((uint8_t)params, length, text, text_table);
	}
	else {
		pos_index = oprint_parametrize_data_string((uint8_t)params, length, text, &text_table[2]);
	}

	if(base == 16) {
		text_table[0] = ' ';
		text_table[1] = ' ';
		text_table[pos_index] = '0';
		text_table[pos_index + 1] = 'x';
	}
	else if(base == 2) {
		text_table[0] = ' ';
		text_table[1] = ' ';
		text_table[pos_index] = 'b';
		text_table[pos_index + 1] = '_';
	}
	
	oprint_string(exe, dev, (const char*)text_table);
} /* oprint_data */

/** 
 *  Sets cursor on the specified position on the display
 *  
 *  \param exe pointer to commands executor params
 *  \param dev pointer to output device
 *  \param x x position
 *  \param y y position
*/
void oprint_pos(oprint_exe_params *exe, void *dev, uint32_t x, uint32_t y) {
	if(!exe) {
		exe = oprint_get_exe();
	}
	if(!dev) {
		dev = oprint_get_dev();
	}

	if(!exe) {
		return;
	}

	if(!exe->set_pos) {
		return;
	}

	exe->set_pos(dev, x, y);
} /* oprint_pos */

/** 
 *  Clears display
 *
 *  \param exe pointer to commands executor params
 *  \param dev pointer to output device
*/
void oprint_clear(oprint_exe_params *exe, void *dev) {
	if(!exe) {
		exe = oprint_get_exe();
	}
	if(!dev) {
		dev = oprint_get_dev();
	}

	if(!exe) {
		return;
	}

	if(!exe->clear_display) {
		return;
	}

	exe->clear_display(dev);
} /* oprint_clear */


/** 
 *  Prints characters string into the print_dev device if setted
 *  else to the print_default_dev if setted
 *  
 *  \param string pointer string to be printed
*/
void print_string(const char *string) {
	oprint_string(0, 0, string);
} /* print_string */

/** 
 *  Prints 32-bit variable in hexadecimal format
 *  
 *  \param data data to be printed
 *  \param length length of data to be printed
 *  \param params data display format params
 *     - bits 0 to 7 - this 8-bit value gives base of data
 *        possible: 2 (binary), 10 (decimal), 16 (hexadecimal)
 *     - bits 8 to 11 - if this 4-bit value is different that zero
 *        then zeroes on the highest positions of the data
 *        will not be displayed
 *     - bits 12 to 15 - if  this 4-bit value is different that zero
 *        then in displayed data will be displayed dots. this 4-bit
 *        value indicates how many digits will be displayed between
 *        two dots
 *     an example: data = 0x00123456, params = 0x4100 | 16,
 *        length = 8; result on the display: 0x12.3456
*/
void print_data (uint32_t data, uint8_t length, uint16_t params) {
	oprint_data(0, 0, data, length, params);
} /* print_data */

/** 
 *  Sets cursor on the specified position on the display
 *  
 *  \param x x position
 *  \param y y position
*/
void print_pos(uint32_t x, uint32_t y) {
	oprint_pos(0, 0, x, y);
} /* print_pos */

/** 
 *  Clears display
*/
void print_clear(void) {
	oprint_clear(0, 0);
} /* print_clear */
