/*
================================================================================
 File: sections.h
--------------------------------------------------------------------------------
 Author:        Piotr Wojtowicz
 Purpose:       critical and irq sections definitions
 Compiler:      ANSI C
 Hardware:
 Doc:
--------------------------------------------------------------------------------
 Includes:
--------------------------------------------------------------------------------
 Comments:                      UWAGA!!!
                Oprogramowanie to udostpnione jest do celw pogldowych,
                nie jest przetestowane pod ktem bezpieczestwa ani spenienia
                jakichkolwiek norm wymaganych aby urzdzenie uywajce go
                mogo by dopuszczone do uytku, dlatego uywasz go
                na wasn odpowiedzialno a jego autor nie ponosi adnego
                typu odpowiedzialnoci za jakiekolwiek skutki jego uycia.
                Jeeli nie zgadzasz si z powyszym warunkiem prosz
                o nieuywanie tego oprogramowania.
================================================================================
*/

#ifndef SECTIONS_H_
#define SECTIONS_H_

#include <stdint.h>

extern unsigned int USBD_critical_state;

#define ENTER_CRITICAL(x) \
	do { \
		__asm volatile ("	CPSID	I"); \
		USBD_critical_state++; \
	} while (0)

#define EXIT_CRITICAL(x) \
	do { \
		if (USBD_critical_state) { \
			USBD_critical_state--; \
		} \
		if (!USBD_critical_state) { \
			__asm volatile ("	CPSIE	I"); \
		} \
	} while (0)



// USBD sections
#define USBD_ENTER_CRITICAL  ENTER_CRITICAL
#define USBD_EXIT_CRITICAL   EXIT_CRITICAL

#define USBD_ENTER_IRQ(x) ENTER_IRQ()
#define USBD_EXIT_IRQ(x) EXIT_IRQ()


extern volatile uint8_t USBD_irq_state;


#define ENTER_IRQ()			USBD_irq_state = 1
void EXIT_IRQ(void);




#endif /*SECTIONS_H_*/
