/*
================================================================================
 File: stm32_usart_init.c
--------------------------------------------------------------------------------
 Author:        Piotr Wojtowicz
 Purpose:       usart initialization procedure for stm32
 Compiler:      ANSI C
 Hardware:
 Doc:
--------------------------------------------------------------------------------
 Includes:
--------------------------------------------------------------------------------
 Comments:                      UWAGA!!!
                Oprogramowanie to udostpnione jest do celw pogldowych,
                nie jest przetestowane pod ktem bezpieczestwa ani spenienia
                jakichkolwiek norm wymaganych aby urzdzenie uywajce go
                mogo by dopuszczone do uytku, dlatego uywasz go
                na wasn odpowiedzialno a jego autor nie ponosi adnego
                typu odpowiedzialnoci za jakiekolwiek skutki jego uycia.
                Jeeli nie zgadzasz si z powyszym warunkiem prosz
                o nieuywanie tego oprogramowania.
================================================================================
*/

#include "stm32_usart_init.h"
#include "stm32f10x_conf.h"


static oprint_exe_params usart_oprint;



/**
 *  Sets cursor to the specified position on the display
 *
 *  \param dev pointer to output device
 *  \param x x position
 *  \param y y position
*/
static void usart_set_position (void *dev, uint32_t x, uint32_t y) {

}

/**
 *  Clears display
 *
 *  \param dev pointer to output device
*/
static void usart_clear_display (void *dev) {

}
/**
 *  Prints string on the display
 *
 *  \param dev pointer to output device
 *  \param string pointer string to be printed
*/
static void usart_print (void *dev, const char *string) {
	uint32_t length = strlen(string);
	uint8_t *data = (uint8_t*)string;
	USART_TypeDef* USARTx = (USART_TypeDef*)dev;

	if(!((USARTx == USART1) || (USARTx == USART2) || (USARTx == USART3))) {
		USARTx = USART2;
	}

	while(length) {
		if (USARTx->SR & 0x80) {
			USARTx->DR = *data;
			data++;
			length--;
		}
	}
}


void init_oprint(uint8_t usart_num) {
	usart_oprint.clear_display = usart_clear_display;
	usart_oprint.set_pos = usart_set_position;
	usart_oprint.print_text = usart_print;

	oprint_set_exe(&usart_oprint);

	if(usart_num == 1) {
		oprint_set_dev(USART1);
	}
	else if(usart_num == 3) {
		oprint_set_dev(USART3);
	}
	else {
		oprint_set_dev(USART2);
		usart_num = 2;
	}

	configure_usart(usart_num, 115200, USART_StopBits_1, USART_Parity_No, 0);
}


void configure_usart(uint8_t usart_num, uint32_t baudrate, uint16_t stop_bits, uint16_t patity, uint8_t enable_irq) {
	GPIO_InitTypeDef GPIO_InitStructure;
	USART_InitTypeDef USART_InitStructure;
	NVIC_InitTypeDef NVIC_InitStructure;
	USART_TypeDef* USARTx;
	GPIO_TypeDef* GPIO_Rx_Port;
	GPIO_TypeDef* GPIO_Tx_Port;
	uint16_t GPIO_Rx_Pin;
	uint16_t GPIO_Tx_Pin;
	uint8_t NVIC_IRQChannel;


	// identify USART number, reset the peripheral and enable its clock
	{
		if(usart_num == 1) {
			USARTx = USART1;
			GPIO_Rx_Port = USART1_RX_PORT;
			GPIO_Tx_Port = USART1_TX_PORT;
			GPIO_Rx_Pin = USART1_RX_PIN;
			GPIO_Tx_Pin = USART1_TX_PIN;
			NVIC_IRQChannel = USART1_IRQn;

			// reset peripheral and enable its clock
			RCC_APB2PeriphResetCmd(RCC_APB2Periph_USART1, ENABLE);
			RCC_APB2PeriphResetCmd(RCC_APB2Periph_USART1, DISABLE);
			RCC_APB2PeriphClockCmd(RCC_APB2Periph_USART1, ENABLE);
		}
		else if(usart_num == 2) {
			USARTx = USART2;
			GPIO_Rx_Port = USART2_RX_PORT;
			GPIO_Tx_Port = USART2_TX_PORT;
			GPIO_Rx_Pin = USART2_RX_PIN;
			GPIO_Tx_Pin = USART2_TX_PIN;
			NVIC_IRQChannel = USART2_IRQn;

			// reset peripheral and enable its clock
			RCC_APB1PeriphResetCmd(RCC_APB1Periph_USART2, ENABLE);
			RCC_APB1PeriphResetCmd(RCC_APB1Periph_USART2, DISABLE);
			RCC_APB1PeriphClockCmd(RCC_APB1Periph_USART2, ENABLE);
		}
		else if(usart_num == 3) {
			USARTx = USART3;
			GPIO_Rx_Port = USART3_RX_PORT;
			GPIO_Tx_Port = USART3_TX_PORT;
			GPIO_Rx_Pin = USART3_RX_PIN;
			GPIO_Tx_Pin = USART3_TX_PIN;
			NVIC_IRQChannel = USART3_IRQn;

			// reset peripheral and enable its clock
			RCC_APB1PeriphResetCmd(RCC_APB1Periph_USART3, ENABLE);
			RCC_APB1PeriphResetCmd(RCC_APB1Periph_USART3, DISABLE);
			RCC_APB1PeriphClockCmd(RCC_APB1Periph_USART3, ENABLE);
		}
		else {
			return;
		}
	}


	// configure GPIO
	{
		GPIO_InitStructure.GPIO_Pin = GPIO_Tx_Pin;
		GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;
		GPIO_InitStructure.GPIO_Mode = GPIO_Mode_AF_PP;
		GPIO_Init(GPIO_Tx_Port, &GPIO_InitStructure);

		GPIO_InitStructure.GPIO_Pin = GPIO_Rx_Pin;
		GPIO_InitStructure.GPIO_Mode = GPIO_Mode_IPU;
		GPIO_Init(GPIO_Rx_Port, &GPIO_InitStructure);
	}


	// configure USART peripheral
	// and enable it
	{
		USART_InitStructure.USART_BaudRate = baudrate;
		USART_InitStructure.USART_WordLength = USART_WordLength_8b;
		USART_InitStructure.USART_StopBits = stop_bits;
		USART_InitStructure.USART_Parity = patity ;
		USART_InitStructure.USART_HardwareFlowControl = USART_HardwareFlowControl_None;
		USART_InitStructure.USART_Mode = USART_Mode_Rx | USART_Mode_Tx;

		// Initialize registers
		USART_Init(USARTx, &USART_InitStructure);
		// Enable the USART
		USART_Cmd(USARTx, ENABLE);
	}


	// Enable the USART Interrupt
	if(enable_irq) {
		NVIC_InitStructure.NVIC_IRQChannel = NVIC_IRQChannel;
		NVIC_InitStructure.NVIC_IRQChannelPreemptionPriority = 0;
		NVIC_InitStructure.NVIC_IRQChannelSubPriority = 0;
		NVIC_InitStructure.NVIC_IRQChannelCmd = ENABLE;
		NVIC_Init(&NVIC_InitStructure);

		// Configure the transmit and receive ISR
		USART_ITConfig(USARTx, USART_IT_TXE, ENABLE);
		USART_ITConfig(USARTx, USART_IT_RXNE, ENABLE);
	}
}

void deconfigure_usart(uint8_t usart_num) {
	GPIO_InitTypeDef GPIO_InitStructure;
	NVIC_InitTypeDef NVIC_InitStructure;
	GPIO_TypeDef* GPIO_Rx_Port;
	GPIO_TypeDef* GPIO_Tx_Port;
	uint16_t GPIO_Rx_Pin;
	uint16_t GPIO_Tx_Pin;

	NVIC_InitStructure.NVIC_IRQChannelPreemptionPriority = 0;
	NVIC_InitStructure.NVIC_IRQChannelSubPriority = 0;
	NVIC_InitStructure.NVIC_IRQChannelCmd = DISABLE;

	// identify USART number, reset the peripheral and enable its clock
	{
		if(usart_num == 1) {
			GPIO_Rx_Port = USART1_RX_PORT;
			GPIO_Tx_Port = USART1_TX_PORT;
			GPIO_Rx_Pin = USART1_RX_PIN;
			GPIO_Tx_Pin = USART1_TX_PIN;

			// Disable the interrupt
			NVIC_InitStructure.NVIC_IRQChannel = USART1_IRQn;
			NVIC_Init(&NVIC_InitStructure);

			// Disable the USART
			USART_Cmd(USART1, DISABLE);

			// reset peripheral and enable its clock
			RCC_APB2PeriphResetCmd(RCC_APB2Periph_USART1, ENABLE);
			RCC_APB2PeriphResetCmd(RCC_APB2Periph_USART1, DISABLE);
			RCC_APB2PeriphClockCmd(RCC_APB2Periph_USART1, DISABLE);
		}
		else if(usart_num == 2) {
			GPIO_Rx_Port = USART2_RX_PORT;
			GPIO_Tx_Port = USART2_TX_PORT;
			GPIO_Rx_Pin = USART2_RX_PIN;
			GPIO_Tx_Pin = USART2_TX_PIN;

			// Disable the interrupt
			NVIC_InitStructure.NVIC_IRQChannel = USART2_IRQn;
			NVIC_Init(&NVIC_InitStructure);

			// Disable the USART
			USART_Cmd(USART2, DISABLE);

			// reset peripheral and enable its clock
			RCC_APB1PeriphResetCmd(RCC_APB1Periph_USART2, ENABLE);
			RCC_APB1PeriphResetCmd(RCC_APB1Periph_USART2, DISABLE);
			RCC_APB1PeriphClockCmd(RCC_APB1Periph_USART2, DISABLE);
		}
		else if(usart_num == 3) {
			GPIO_Rx_Port = USART3_RX_PORT;
			GPIO_Tx_Port = USART3_TX_PORT;
			GPIO_Rx_Pin = USART3_RX_PIN;
			GPIO_Tx_Pin = USART3_TX_PIN;

			// Disable the interrupt
			NVIC_InitStructure.NVIC_IRQChannel = USART3_IRQn;
			NVIC_Init(&NVIC_InitStructure);

			// Disable the USART
			USART_Cmd(USART3, DISABLE);

			// reset peripheral and enable its clock
			RCC_APB1PeriphResetCmd(RCC_APB1Periph_USART3, ENABLE);
			RCC_APB1PeriphResetCmd(RCC_APB1Periph_USART3, DISABLE);
			RCC_APB1PeriphClockCmd(RCC_APB1Periph_USART3, DISABLE);
		}
		else {
			return;
		}
	}


	// deconfigure GPIO
	{
		GPIO_InitStructure.GPIO_Pin = GPIO_Tx_Pin;
		GPIO_InitStructure.GPIO_Speed = GPIO_Speed_2MHz;
		GPIO_InitStructure.GPIO_Mode = GPIO_Mode_IN_FLOATING;
		GPIO_Init(GPIO_Tx_Port, &GPIO_InitStructure);

		GPIO_InitStructure.GPIO_Pin = GPIO_Rx_Pin;
		GPIO_InitStructure.GPIO_Mode = GPIO_Mode_IN_FLOATING;
		GPIO_Init(GPIO_Rx_Port, &GPIO_InitStructure);
	}
}
