/*
================================================================================
 File: dfu.c
--------------------------------------------------------------------------------
 Author:        Piotr Wojtowicz
 Purpose:       USBD Device Firmware Upgrade layer
 Compiler:      ANSI C
 Hardware:      
 Doc:
--------------------------------------------------------------------------------
 Includes:      
--------------------------------------------------------------------------------
 Comments:                      UWAGA!!!
                Oprogramowanie to udostępnione jest do celów poglądowych,
                nie jest przetestowane pod kątem bezpieczeństwa ani spełnienia
                jakichkolwiek norm wymaganych aby urządzenie używające go
                mogło być dopuszczone do użytku, dlatego używasz go
                na własną odpowiedzialność a jego autor nie ponosi żadnego
                typu odpowiedzialności za jakiekolwiek skutki jego użycia.
                Jeżeli nie zgadzasz się z powyższym warunkiem proszę
                o nieużywanie tego oprogramowania.
================================================================================
*/

#ifndef DFU_H_
#include "dfu.h"
#endif

#include <string.h>



// -----------------------------------------------------------------------------
//  8-bit variable in FLASH memory that's address is used
//  as identifier of this layer
// -----------------------------------------------------------------------------
static const uint8_t DFU_USBD_identifier = 0xFE;


static void DFU_state_refresh(USBDC_params *usbdc, uint8_t request_type, USBDR_params *request, DFU_interface *dfu);

#if(USBD_DFU_APP_MODE_SUPPORT == USBD_FEATURE_ON)
static void DFU_RUN_TIME_MODE_Control_handler (USBDC_params *usbdc, USBDEP_params *ep, uint8_t interface, USBDR_params *request);
#endif

#if(USBD_DFU_DFU_MODE_SUPPORT == USBD_FEATURE_ON)
static void DFU_DFU_MODE_Control_handler (USBDC_params *usbdc, USBDEP_params *ep, uint8_t interface, USBDR_params *request);
#endif



static uint32_t DFU_get_timeout(DFU_interface *dfu) {
	DFU_timeout timeout;

	timeout.elements.L = dfu->extended_data.DFU_state.bwPollTimeout_L;
	timeout.elements.M = dfu->extended_data.DFU_state.bwPollTimeout_M;
	timeout.elements.H = dfu->extended_data.DFU_state.bwPollTimeout_H;
	timeout.elements.not_used = 0;
	
	return(timeout.timeout);
}


#if(USBD_DFU_DNLOAD_SUPPORT == USBD_FEATURE_ON)
static void DFU_EP0_DNLOAD_close_control(USBDC_params *usbdc, USBDEP_params *ep) {
	DFU_interface *dfu = (DFU_interface *)(usbdc->interfaces[usbdc->upper_layer_data.uint8].upper_layer_data.pvoid);
	DFU_status_type *dfu_status = &(dfu->extended_data.DFU_state);
	if((ep->transfer_params.transfer_state == USBD_END_OF_TRANSFER)
			|| (ep->transfer_params.transfer_state == USBD_TOO_LESS_SPACE_FOR_NEXT_PACKET)) {
		if(dfu->user_data.DFU_DNLOAD_handler) {
			dfu_status->bStatus = dfu->user_data.DFU_DNLOAD_handler(
					&(dfu->extended_data.DFU_transfer_buffer),
					ep->transfer_params.data_transferred_length,
					dfu->extended_data.DFU_transfer_packet_number);
		}
		if(dfu_status->bStatus == DFU_BSTATUS_OK) {
			USBD_send_status(0, 0, usbdc);
		}
		else {
			dfu_status->bState = DFU_BSTATE_DFU_ERROR;
			USBD_send_stall(0, usbdc);
		}
	}
}



static void DFU_EP0_DNLOAD_manifest(USBDC_params *usbdc, USBDEP_params *ep) {
	DFU_interface *dfu = (DFU_interface *)(usbdc->interfaces[usbdc->upper_layer_data.uint8].upper_layer_data.pvoid);
	DFU_state_refresh(usbdc, USBD_DFU_BWPOLLTIMEOUT_ELAPSED, 0, dfu);
}
#endif


static void DFU_state_refresh(USBDC_params *usbdc, uint8_t request_type, USBDR_params *request, DFU_interface *DFU) {
	DFU_status_type *dfu_status = &(DFU->extended_data.DFU_state);
	if(request_type == USBD_RESET) {
#if(USBD_DFU_DFU_MODE_SUPPORT == USBD_FEATURE_ON)
		if(dfu_status->bState > DFU_BSTATE_APP_DETACH) {
			if(DFU->user_data.DFU_is_app_correct) {
				if(DFU->user_data.DFU_is_app_correct()) {
					dfu_status->bState = DFU_BSTATE_APP_IDLE;
					if(DFU->user_data.DFU_event_handler) {
						(*DFU->user_data.DFU_event_handler)(dfu_status->bState);
					}
					return;
				}
			}
			dfu_status->bState = DFU_BSTATE_DFU_ERROR;
			dfu_status->bStatus = DFU_BSTATUS_ERR_FILE;
		}
#endif

#if((USBD_DFU_DFU_MODE_SUPPORT == USBD_FEATURE_ON) && (USBD_DFU_APP_MODE_SUPPORT == USBD_FEATURE_ON))
		else 
#endif

#if(USBD_DFU_APP_MODE_SUPPORT == USBD_FEATURE_ON)
		if(dfu_status->bState == DFU_BSTATE_APP_DETACH) {
			dfu_status->bState = DFU_BSTATE_DFU_IDLE;
			if(DFU->user_data.DFU_event_handler) {
				(*DFU->user_data.DFU_event_handler)(dfu_status->bState);
			}
		}
#endif
		return;
	}
	switch(dfu_status->bState) {
#if(USBD_DFU_APP_MODE_SUPPORT == USBD_FEATURE_ON)
		case DFU_BSTATE_APP_IDLE:
			switch(request_type) {
				case USBD_DFU_BREQUEST_DETACH:
					dfu_status->bState = DFU_BSTATE_APP_DETACH;
					USBD_send_status(0, 0, usbdc);
#if(USBD_DFU_WILL_DETACH_SUPPORT == USBD_FEATURE_ON)
					if(DFU->user_data.DFU_bmAttributes & DFU_BMATTRIBUTES_BIT_WILL_DETACH) {
						if(DFU->user_data.DFU_event_handler) {
							(*DFU->user_data.DFU_event_handler)(dfu_status->bState);
						}
					}
					else
#endif
						DFU_set_timeout(DFU, DFU->user_data.DFU_detach_timeout & 0xFFFF);
					break;

				case USBD_DFU_BREQUEST_GETSTATUS:
					// send STATUS
					USBD_send_data_in(
							(uint8_t*)dfu_status,
							6,
							0,
							usbdc);
					break;

				case USBD_DFU_BREQUEST_GETSTATE:
					// send STATE
					USBD_send_data_in(
							(uint8_t*)&dfu_status->bState,
							1,
							0,
							usbdc);
					break;

				default:
					USBD_send_stall(0, usbdc);
					break;
			}
			break;

		case DFU_BSTATE_APP_DETACH:
			switch(request_type) {
				case USBD_DFU_BREQUEST_GETSTATUS:
					// send STATUS
					USBD_send_data_in(
							(uint8_t*)dfu_status,
							6,
							0,
							usbdc);
					break;

				case USBD_DFU_BREQUEST_GETSTATE:
					// send STATE
					USBD_send_data_in(
							(uint8_t*)&dfu_status->bState,
							1,
							0,
							usbdc);
					break;

				case USBD_DFU_BWPOLLTIMEOUT_ELAPSED:
					dfu_status->bState = DFU_BSTATE_APP_IDLE;
					break;
					
				default:
					dfu_status->bState = DFU_BSTATE_APP_IDLE;
					USBD_send_stall(0, usbdc);
					break;
			}
			break;
#endif

#if(USBD_DFU_DFU_MODE_SUPPORT == USBD_FEATURE_ON)
		case DFU_BSTATE_DFU_IDLE:
			switch(request_type) {
				case USBD_DFU_BREQUEST_DNLOAD:
#if(USBD_DFU_DNLOAD_SUPPORT == USBD_FEATURE_ON)
					if(request->wLength && (DFU->user_data.DFU_bmAttributes & DFU_BMATTRIBUTES_BIT_CAN_DNLOAD)) {
						dfu_status->bState = DFU_BSTATE_DFU_DNLOAD_SYNC;

						DFU->extended_data.DFU_transfer_packet_number = request->wValue;
						usbdc->ep_irq[0] = DFU_EP0_DNLOAD_close_control;
						USBD_recv_data_out(DFU->extended_data.DFU_transfer_buffer, request->wLength, 0, usbdc);
					}
					else {
						dfu_status->bState = DFU_BSTATE_DFU_ERROR;
						dfu_status->bStatus = DFU_BSTATUS_ERR_NOTDONE;
						USBD_send_stall(0, usbdc);
					}
#else
					dfu_status->bState = DFU_BSTATE_DFU_ERROR;
					dfu_status->bStatus = DFU_BSTATUS_ERR_STALLEDPKT;
					USBD_send_stall(0, usbdc);
#endif
					break;

				case USBD_DFU_BREQUEST_UPLOAD:
#if(USBD_DFU_UPLOAD_SUPPORT == USBD_FEATURE_ON)
					dfu_status->bState = DFU_BSTATE_DFU_UPLOAD_IDLE;
					if(DFU->user_data.DFU_bmAttributes & DFU_BMATTRIBUTES_BIT_CAN_UPLOAD) {
						// TODO: start uploading
					}
#else
					dfu_status->bState = DFU_BSTATE_DFU_ERROR;
					dfu_status->bStatus = DFU_BSTATUS_ERR_STALLEDPKT;
					USBD_send_stall(0, usbdc);
#endif
					break;

				case USBD_DFU_BREQUEST_ABORT:
					USBD_send_status(0, 0, usbdc);
					break;

				case USBD_DFU_BREQUEST_GETSTATUS:
					// send STATUS
					USBD_send_data_in(
							(uint8_t*)dfu_status,
							6,
							0,
							usbdc);
					break;

				case USBD_DFU_BREQUEST_GETSTATE:
					// send STATE
					USBD_send_data_in(
							(uint8_t*)&dfu_status->bState,
							1,
							0,
							usbdc);
					break;
					
				default:
					dfu_status->bState = DFU_BSTATE_DFU_ERROR;
					dfu_status->bStatus = DFU_BSTATUS_ERR_STALLEDPKT;
					USBD_send_stall(0, usbdc);
					break;
			}
			break;

#if(USBD_DFU_DNLOAD_SUPPORT == USBD_FEATURE_ON)
		case DFU_BSTATE_DFU_DNLOAD_SYNC:
			switch(request_type) {
				case USBD_DFU_BREQUEST_GETSTATUS:
					if(DFU_get_timeout(DFU)) {
						dfu_status->bState = DFU_BSTATE_DFU_DNBUSY;
					}
					else {
						dfu_status->bState = DFU_BSTATE_DFU_DNLOAD_IDLE;
					}

					// send STATUS
					USBD_send_data_in(
							(uint8_t*)dfu_status,
							6,
							0,
							usbdc);
					break;

				case USBD_DFU_BREQUEST_GETSTATE:
					// send STATE
					USBD_send_data_in(
							(uint8_t*)&dfu_status->bState,
							1,
							0,
							usbdc);
					break;

				case USBD_DFU_BWPOLLTIMEOUT_ELAPSED:
					break;

				case USBD_DFU_BREQUEST_ABORT:
					dfu_status->bState = DFU_BSTATE_DFU_IDLE;
					USBD_send_status(0, 0, usbdc);
					break;
					
				default:
					dfu_status->bState = DFU_BSTATE_DFU_ERROR;
					dfu_status->bStatus = DFU_BSTATUS_ERR_STALLEDPKT;
					USBD_send_stall(0, usbdc);
					break;
			}
			break;

		case DFU_BSTATE_DFU_DNBUSY:
			switch(request_type) {
				case USBD_DFU_BWPOLLTIMEOUT_ELAPSED:
					dfu_status->bState = DFU_BSTATE_DFU_DNLOAD_SYNC;
					break;
					
				default:
					dfu_status->bState = DFU_BSTATE_DFU_ERROR;
					dfu_status->bStatus = DFU_BSTATUS_ERR_STALLEDPKT;
					USBD_send_stall(0, usbdc);
					break;
			}
			break;

		case DFU_BSTATE_DFU_DNLOAD_IDLE:
			switch(request_type) {
				case USBD_DFU_BREQUEST_DNLOAD:
					if(request->wLength > 0) {
						dfu_status->bState = DFU_BSTATE_DFU_DNLOAD_SYNC;

						usbdc->ep_irq[0] = DFU_EP0_DNLOAD_close_control;
						USBD_recv_data_out(DFU->extended_data.DFU_transfer_buffer, request->wLength, 0, usbdc);
					}
					// wLength == 0
					else {
						if(DFU->user_data.DFU_DNLOAD_handler) {
							dfu_status->bStatus = DFU->user_data.DFU_DNLOAD_handler(&(DFU->extended_data.DFU_transfer_buffer), 0, 0);
						}
						if(dfu_status->bStatus == DFU_BSTATUS_OK) {
							dfu_status->bState = DFU_BSTATE_DFU_MANIFEST_SYNC;
							
#if(USBD_DFU_MANIFESTATION_TOLERANT_SUPPORT == USBD_FEATURE_ON)
							DFU->extended_data.DFU_manifestation_phase = 0;
#endif
							USBD_send_status(0, 0, usbdc);

							if(DFU->user_data.DFU_event_handler) {
								(*DFU->user_data.DFU_event_handler)(dfu_status->bState);
							}
						}
						else {
							dfu_status->bState = DFU_BSTATE_DFU_ERROR;
							USBD_send_stall(0, usbdc);
						}
					}
					break;

				case USBD_DFU_BREQUEST_ABORT:
					dfu_status->bState = DFU_BSTATE_DFU_IDLE;
					USBD_send_status(0, 0, usbdc);
					break;

				case USBD_DFU_BREQUEST_GETSTATUS:
					// send STATUS
					USBD_send_data_in(
							(uint8_t*)dfu_status,
							6,
							0,
							usbdc);
					break;

				case USBD_DFU_BREQUEST_GETSTATE:
					// send STATE
					USBD_send_data_in(
							(uint8_t*)&dfu_status->bState,
							1,
							0,
							usbdc);
					break;
					
				default:
					dfu_status->bState = DFU_BSTATE_DFU_ERROR;
					dfu_status->bStatus = DFU_BSTATUS_ERR_STALLEDPKT;
					USBD_send_stall(0, usbdc);
					break;
			}
			break;

		case DFU_BSTATE_DFU_MANIFEST_SYNC:
			switch(request_type) {
				case USBD_DFU_BREQUEST_GETSTATUS:
#if(USBD_DFU_MANIFESTATION_TOLERANT_SUPPORT == USBD_FEATURE_ON)
					if(!DFU->extended_data.DFU_manifestation_phase) {
						dfu_status->bState = DFU_BSTATE_DFU_MANIFEST;
						DFU->extended_data.DFU_manifestation_phase = 1;
					}
					else {
						dfu_status->bState = DFU_BSTATE_DFU_IDLE;
					}
#else
					dfu_status->bState = DFU_BSTATE_DFU_MANIFEST;
#endif

					if(DFU->user_data.DFU_event_handler) {
						(*DFU->user_data.DFU_event_handler)(dfu_status->bState);
					}

					if(!DFU_get_timeout(DFU)) {
						usbdc->ep_irq[0] = DFU_EP0_DNLOAD_manifest;
					}
					// send STATUS
					USBD_send_data_in(
							(uint8_t*)dfu_status,
							6,
							0,
							usbdc);
					break;

				case USBD_DFU_BREQUEST_GETSTATE:
					// send STATE
					USBD_send_data_in(
							(uint8_t*)&dfu_status->bState,
							1,
							0,
							usbdc);
					break;

				case USBD_DFU_BREQUEST_ABORT:
					dfu_status->bState = DFU_BSTATE_DFU_IDLE;
					USBD_send_status(0, 0, usbdc);
					break;
					
				default:
					dfu_status->bState = DFU_BSTATE_DFU_ERROR;
					dfu_status->bStatus = DFU_BSTATUS_ERR_STALLEDPKT;
					USBD_send_stall(0, usbdc);
					break;
			}
			break;

		case DFU_BSTATE_DFU_MANIFEST:
			switch(request_type) {
				case USBD_DFU_BWPOLLTIMEOUT_ELAPSED:
#if(USBD_DFU_MANIFESTATION_TOLERANT_SUPPORT == USBD_FEATURE_ON)
					if(DFU->user_data.DFU_bmAttributes & DFU_BMATTRIBUTES_BIT_MANIFESTATION_TOLERANT) {
						dfu_status->bState = DFU_BSTATE_DFU_MANIFEST_SYNC;
					}
					else {
						dfu_status->bState = DFU_BSTATE_DFU_MANIFEST_WAIT_RESET;
					}
#else
					dfu_status->bState = DFU_BSTATE_DFU_MANIFEST_WAIT_RESET;
#endif
					if(DFU->user_data.DFU_event_handler) {
						(*DFU->user_data.DFU_event_handler)(dfu_status->bState);
					}
					break;
					
				default:
					dfu_status->bState = DFU_BSTATE_DFU_ERROR;
					dfu_status->bStatus = DFU_BSTATUS_ERR_STALLEDPKT;
					USBD_send_stall(0, usbdc);
					break;
			}
			break;

//		case DFU_BSTATE_DFU_MANIFEST_WAIT_RESET:
#endif

#if(USBD_DFU_UPLOAD_SUPPORT == USBD_FEATURE_ON)
		case DFU_BSTATE_DFU_UPLOAD_IDLE:
			switch(request_type) {
				case USBD_DFU_BREQUEST_UPLOAD:
					if(request->wValue == USBDC_DFU_wTransferSize) {// TODO:USBDC_DFU_wTransferSize
						dfu_status->bState = DFU_BSTATE_DFU_UPLOAD_IDLE;
						// TODO: upload
					}
					else {
						dfu_status->bState = DFU_BSTATE_DFU_IDLE;
					}
					break;

				case USBD_DFU_BREQUEST_ABORT:
					dfu_status->bState = DFU_BSTATE_DFU_IDLE;
					USBD_send_status(0, 0, usbdc);
					break;

				case USBD_DFU_BREQUEST_GETSTATUS:
					// send STATUS
					USBD_send_data_in(
							(uint8_t*)dfu_status,
							6,
							0,
							usbdc);
					break;

				case USBD_DFU_BREQUEST_GETSTATE:
					// send STATE
					USBD_send_data_in(
							(uint8_t*)&dfu_status->bState,
							1,
							0,
							usbdc);
					break;
					
				default:
					dfu_status->bState = DFU_BSTATE_DFU_ERROR;
					dfu_status->bStatus = DFU_BSTATUS_ERR_STALLEDPKT;
					USBD_send_stall(0, usbdc);
					break;
			}
			break;
#endif

		case DFU_BSTATE_DFU_ERROR:
			switch(request_type) {
				case USBD_DFU_BREQUEST_GETSTATUS:
					// send STATUS
					USBD_send_data_in(
							(uint8_t*)dfu_status,
							6,
							0,
							usbdc);
					break;

				case USBD_DFU_BREQUEST_GETSTATE:
					// send STATE
					USBD_send_data_in(
							(uint8_t*)&dfu_status->bState,
							1,
							0,
							usbdc);
					break;

				case USBD_DFU_BREQUEST_CLRSTATUS:
					dfu_status->bState = DFU_BSTATE_DFU_IDLE;
					dfu_status->bStatus = DFU_BSTATUS_OK;
					USBD_send_status(0, 0, usbdc);
					break;
					
				default:
					USBD_send_stall(0, usbdc);
					break;
			}
			break;
#endif

		default:
			dfu_status->bState = DFU_BSTATE_DFU_ERROR;
			dfu_status->bStatus = DFU_BSTATUS_ERR_UNKNOWN;
			break;
	}
}



#if(USBD_DFU_APP_MODE_SUPPORT == USBD_FEATURE_ON)
static void DFU_RUN_TIME_MODE_Control_handler (USBDC_params *usbdc, USBDEP_params *ep, uint8_t interface, USBDR_params *request) {
	DFU_interface *DFU = (DFU_interface *)(usbdc->interfaces[interface].upper_layer_data.pvoid);
	usbdc->upper_layer_data.uint8 = interface;

	switch(request->Request.WholeRequestValue) {
		case DFU_DETACH_Interface:
		case DFU_GETSTATUS_Interface:
		case DFU_GETSTATE_Interface:
			DFU_state_refresh(usbdc, request->Request.RequestFields.bRequest, request, DFU);
			break;

		default:
			USBD_send_stall(0, usbdc);
			break;
	}
}
#endif



#if(USBD_DFU_DFU_MODE_SUPPORT == USBD_FEATURE_ON)
static void DFU_DFU_MODE_Control_handler (USBDC_params *usbdc, USBDEP_params *ep, uint8_t interface, USBDR_params *request) {
	DFU_interface *dfu = (DFU_interface *)(usbdc->interfaces[interface].upper_layer_data.pvoid);
	usbdc->upper_layer_data.uint8 = interface;

	switch(request->Request.WholeRequestValue) {
		case DFU_DNLOAD_Interface:
		case DFU_UPLOAD_Interface:
		case DFU_GETSTATUS_Interface:
		case DFU_CLRSTATUS_Interface:
		case DFU_GETSTATE_Interface:
		case DFU_ABORT_Interface:
			DFU_state_refresh(usbdc, request->Request.RequestFields.bRequest, request, dfu);
			break;

#if(USBD_ALTERNATE_SETTING_SUPPORT == USBD_FEATURE_ON)
		case USBD_SET_INTERFACE_INTERFACE:
			if(dfu->user_data.DFU_set_interface_handler) {
				(*dfu->user_data.DFU_set_interface_handler)((uint8_t)request->wValue);
			}
			break;
#endif

		default:
			USBD_send_stall(0, usbdc);
			break;
	}
}
#endif



static void DFU_event(USBDC_params *usbdc, uint8_t flags, USBDE_params *event) {
	DFU_interface *dfu = (DFU_interface *)(usbdc->interfaces[event->upper_layer_data.uint8].upper_layer_data.pvoid);
	DFU_timeout timeout;

	switch(flags) {
		case USBDEF_CONFIGURED:
			event->active_events |= USBDEM_RESET;
			break;

		case USBDEF_UNCONFIGURED:
			event->active_events = USBDEM_CONFIGURED | USBDEM_UNCONFIGURED;
			break;

		case USBDEF_RESET:
			DFU_state_refresh(usbdc, USBD_RESET, 0, dfu);
			break;

		case USBDEF_SOF:
			timeout.timeout = DFU_get_timeout(dfu);
			if(timeout.timeout) {
				timeout.timeout--;
			}
			dfu->extended_data.DFU_state.bwPollTimeout_L = timeout.elements.L;
			dfu->extended_data.DFU_state.bwPollTimeout_M = timeout.elements.M;
			dfu->extended_data.DFU_state.bwPollTimeout_H = timeout.elements.H;
			if(!timeout.timeout) {
				event->active_events &= ~((uint16_t)USBDEM_SOF);
				DFU_state_refresh(usbdc, USBD_DFU_BWPOLLTIMEOUT_ELAPSED, 0, dfu);
			}
			break;

		default:
			break;
	}
}



static void DFU_init(USBDC_params *usbdc, DFU_interface *DFU, uint8_t DFU_interface) {
	USBDE_params *event;
	DFU->extended_data.usbdc = usbdc;
	DFU->extended_data.DFU_state.bStatus = DFU_BSTATUS_OK;
	usbdc->interfaces[DFU_interface].upper_layer_data.pvoid = DFU;
	DFU->extended_data.DFU_interface = DFU_interface;
	DFU->extended_data.DFU_event_ID = 
		USBDE_install(DFU_event, USBDEM_CONFIGURED | USBDEM_UNCONFIGURED, usbdc);
	event = USBDE_get_params(DFU->extended_data.DFU_event_ID, usbdc);
	event->upper_layer_data.uint8 = DFU_interface;
	
}



#if(USBD_DFU_APP_MODE_SUPPORT == USBD_FEATURE_ON)
void DFU_APP_init(DFU_interface *DFU) {
	memset((void*)DFU, 0, sizeof(DFU_interface));
}

void DFU_APP_install(USBDC_params *usbdc, DFU_interface *DFU, uint8_t DFU_interface) {
	DFU->extended_data.DFU_state.bState = DFU_BSTATE_APP_IDLE;
	usbdc->ep0_setup_interface_irq[DFU_interface] = DFU_RUN_TIME_MODE_Control_handler;
	DFU_init(usbdc, DFU, DFU_interface);
}
#endif



#if(USBD_DFU_DFU_MODE_SUPPORT == USBD_FEATURE_ON)
void DFU_DFU_init(DFU_interface *DFU) {
	memset((void*)DFU, 0, sizeof(DFU_interface));
}


void DFU_DFU_install(USBDC_params *usbdc, DFU_interface *DFU, uint8_t *buffer) {
	DFU->extended_data.DFU_state.bState = DFU_BSTATE_DFU_IDLE;
	usbdc->ep0_setup_interface_irq[0] = DFU_DFU_MODE_Control_handler;
	DFU->extended_data.DFU_transfer_buffer = buffer;
	DFU_init(usbdc, DFU, 0);
}
#endif




void DFU_set_timeout(DFU_interface *DFU, uint32_t timeout) {
	USBDE_params *event;
	DFU_timeout tim;

	tim.timeout = timeout;

	DFU->extended_data.DFU_state.bwPollTimeout_L = tim.elements.L;
	DFU->extended_data.DFU_state.bwPollTimeout_M = tim.elements.M;
	DFU->extended_data.DFU_state.bwPollTimeout_H = tim.elements.H;

	event = USBDE_get_params(DFU->extended_data.DFU_event_ID, DFU->extended_data.usbdc);
	event->active_events |= USBDEM_SOF;
}
