/*
================================================================================
 File: dfu.h
--------------------------------------------------------------------------------
 Author:        Piotr Wojtowicz
 Purpose:       USBD Device Firmware Upgrade layer
 Compiler:      ANSI C
 Hardware:      
 Doc:
--------------------------------------------------------------------------------
 Includes:      
--------------------------------------------------------------------------------
 Comments:                      UWAGA!!!
                Oprogramowanie to udostępnione jest do celów poglądowych,
                nie jest przetestowane pod kątem bezpieczeństwa ani spełnienia
                jakichkolwiek norm wymaganych aby urządzenie używające go
                mogło być dopuszczone do użytku, dlatego używasz go
                na własną odpowiedzialność a jego autor nie ponosi żadnego
                typu odpowiedzialności za jakiekolwiek skutki jego użycia.
                Jeżeli nie zgadzasz się z powyższym warunkiem proszę
                o nieużywanie tego oprogramowania.
================================================================================
*/

#ifndef DFU_H_
#define DFU_H_

#ifndef USBD_H_
#include "usbd.h"
#endif



#ifndef USBD_DFU_APP_MODE_SUPPORT
#define USBD_DFU_APP_MODE_SUPPORT                   USBD_FEATURE_ON
#endif


#ifndef USBD_DFU_DFU_MODE_SUPPORT
#define USBD_DFU_DFU_MODE_SUPPORT                   USBD_FEATURE_ON
#endif


#ifndef USBD_DFU_UPLOAD_SUPPORT
#define USBD_DFU_UPLOAD_SUPPORT                     USBD_FEATURE_ON
#endif


#ifndef USBD_DFU_DNLOAD_SUPPORT
#define USBD_DFU_DNLOAD_SUPPORT                     USBD_FEATURE_ON
#endif


#ifndef USBD_DFU_WILL_DETACH_SUPPORT
#define USBD_DFU_WILL_DETACH_SUPPORT                USBD_FEATURE_ON
#endif


#ifndef USBD_DFU_MANIFESTATION_TOLERANT_SUPPORT
#define USBD_DFU_MANIFESTATION_TOLERANT_SUPPORT     USBD_FEATURE_ON
#endif








typedef enum {
USBD_DFU_BREQUEST_DETACH        = 0,
USBD_DFU_BREQUEST_DNLOAD        = 1,
USBD_DFU_BREQUEST_UPLOAD        = 2,
USBD_DFU_BREQUEST_GETSTATUS     = 3,
USBD_DFU_BREQUEST_CLRSTATUS     = 4,
USBD_DFU_BREQUEST_GETSTATE      = 5,
USBD_DFU_BREQUEST_ABORT         = 6,

USBD_RESET                      = 7,
USBD_DFU_BWPOLLTIMEOUT_ELAPSED  = 8
} DFU_state_transition;



#define DFU_BMATTRIBUTES_BIT_CAN_DNLOAD               (1 << 0)
#define DFU_BMATTRIBUTES_BIT_CAN_UPLOAD               (1 << 1)
#define DFU_BMATTRIBUTES_BIT_MANIFESTATION_TOLERANT   (1 << 2)
#define DFU_BMATTRIBUTES_BIT_WILL_DETACH              (1 << 3)



typedef enum {
DFU_DETACH_Interface    = (USBD_BMREQUESTTYPE_SET_CLEAR_INTERFACE | USBD_BMREQUESTTYPE_CLASS | (USBD_DFU_BREQUEST_DETACH << 8)),
DFU_DNLOAD_Interface    = (USBD_BMREQUESTTYPE_SET_CLEAR_INTERFACE | USBD_BMREQUESTTYPE_CLASS | (USBD_DFU_BREQUEST_DNLOAD << 8)),
DFU_UPLOAD_Interface    = (USBD_BMREQUESTTYPE_GET_SYNCH_INTERFACE | USBD_BMREQUESTTYPE_CLASS | (USBD_DFU_BREQUEST_UPLOAD << 8)),
DFU_GETSTATUS_Interface = (USBD_BMREQUESTTYPE_GET_SYNCH_INTERFACE | USBD_BMREQUESTTYPE_CLASS | (USBD_DFU_BREQUEST_GETSTATUS << 8)),
DFU_CLRSTATUS_Interface = (USBD_BMREQUESTTYPE_SET_CLEAR_INTERFACE | USBD_BMREQUESTTYPE_CLASS | (USBD_DFU_BREQUEST_CLRSTATUS << 8)),
DFU_GETSTATE_Interface  = (USBD_BMREQUESTTYPE_GET_SYNCH_INTERFACE | USBD_BMREQUESTTYPE_CLASS | (USBD_DFU_BREQUEST_GETSTATE << 8)),
DFU_ABORT_Interface     = (USBD_BMREQUESTTYPE_SET_CLEAR_INTERFACE | USBD_BMREQUESTTYPE_CLASS | (USBD_DFU_BREQUEST_ABORT << 8))
} USBDR_DFU_types;



typedef enum {
	DFU_BSTATE_APP_IDLE = 0,
	DFU_BSTATE_APP_DETACH = 1,
	DFU_BSTATE_DFU_IDLE = 2,
	DFU_BSTATE_DFU_DNLOAD_SYNC = 3,
	DFU_BSTATE_DFU_DNBUSY = 4,
	DFU_BSTATE_DFU_DNLOAD_IDLE = 5,
	DFU_BSTATE_DFU_MANIFEST_SYNC = 6,
	DFU_BSTATE_DFU_MANIFEST = 7,
	DFU_BSTATE_DFU_MANIFEST_WAIT_RESET = 8,
	DFU_BSTATE_DFU_UPLOAD_IDLE = 9,
	DFU_BSTATE_DFU_ERROR = 10
}DFU_bState_types;

typedef enum {
	DFU_BSTATUS_OK,
	DFU_BSTATUS_ERR_TARGET,
	DFU_BSTATUS_ERR_FILE,
	DFU_BSTATUS_ERR_WRITE,
	DFU_BSTATUS_ERR_ERASE,
	DFU_BSTATUS_ERR_CHECK_ERASED,
	DFU_BSTATUS_ERR_PROG,
	DFU_BSTATUS_ERR_VERIFY,
	DFU_BSTATUS_ERR_ADDRESS,
	DFU_BSTATUS_ERR_NOTDONE,
	DFU_BSTATUS_ERR_FIRMWARE,
	DFU_BSTATUS_ERR_VENDOR,
	DFU_BSTATUS_ERR_USBR,
	DFU_BSTATUS_ERR_POR,
	DFU_BSTATUS_ERR_UNKNOWN,
	DFU_BSTATUS_ERR_STALLEDPKT
}DFU_bStatus_types;


typedef union {
	uint32_t timeout;
	struct {
		uint8_t L;
		uint8_t M;
		uint8_t H;
		uint8_t not_used;
	}elements;
}DFU_timeout;


typedef struct {
	uint8_t bStatus;
	uint8_t bwPollTimeout_L;
	uint8_t bwPollTimeout_M;
	uint8_t bwPollTimeout_H;
	uint8_t bState;
	uint8_t iString;
}DFU_status_type;


typedef struct {
	struct {
#if(USBD_DFU_DFU_MODE_SUPPORT == USBD_FEATURE_ON)
		// pointer to function called when DNLOAD data packet
		// has beed succesfully received
#if(USBD_DFU_DNLOAD_SUPPORT == USBD_FEATURE_ON)
		uint8_t (*DFU_DNLOAD_handler) (uint8_t **buffer, uint16_t length, uint16_t packet_number);
#endif

		// pointer to function called when UPLOAD request
		// has been received from Host
#if(USBD_DFU_UPLOAD_SUPPORT == USBD_FEATURE_ON)
		uint32_t (*DFU_UPLOAD_handler) (uint8_t **buffer, uint16_t length, uint16_t packet_number);
#endif

		// pointer to function called in one of
		// following states:
		// - DFU_BSTATE_APP_IDLE - switch to appIDLE mode from DFU mode
		//        usually upper layer have to reconfigure USBD
		// - DFU_BSTATE_DFU_IDLE -
		//      1 - switch to dfuIDLE mode from APP mode
		//          usually upper layer have to reconfigure USBD
		//      2 - after USBD_DFU_BREQUEST_GETSTATUS request
		//          received in DFU_BSTATE_DFU_MANIFEST_SYNC state
		//          when bitManifestationTolerant = 1
		// - DFU_BSTATE_APP_DETACH - this event occurs when we support
		//        detach sequence - upper layer have to perform
		//        USBD_deactivate - delay - USBD_activate sequence
		//        and reconfigure USBD as result of USBD killed sesion
		// - DFU_BSTATE_DFU_MANIFEST_SYNC -
		//      1 - after accepted DNLOAD zero-length packet
		//      2 - after Timeout elapsed in DFU_BSTATE_DFU_MANIFEST state
		//          and bitManifestationTolerant = 1
		// - DFU_BSTATE_DFU_MANIFEST after USBD_DFU_BREQUEST_GETSTATUS request
		//        received in DFU_BSTATE_DFU_MANIFEST_SYNC state
		// - DFU_BSTATE_DFU_MANIFEST_WAIT_RESET after Timeout elapsed
		//        in DFU_BSTATE_DFU_MANIFEST state and bitManifestationTolerant = 0
		void (*DFU_event_handler) (uint8_t state);

		// pointer to function that is called
		// when DFU layer wants check newly
		// programmed application
		uint8_t (*DFU_is_app_correct) (void);

		// pointer to function called when Host wants
		// switch to another alternate setting of DFU interface
#if(USBD_ALTERNATE_SETTING_SUPPORT == USBD_FEATURE_ON)
		void (*DFU_set_interface_handler) (uint8_t alternate_setting) = 0;
#endif
#endif
		
#if((USBD_DFU_APP_MODE_SUPPORT == USBD_FEATURE_ON) && (USBD_DFU_WILL_DETACH_SUPPORT == USBD_FEATURE_ON))
		uint32_t DFU_detach_timeout;
#endif

		uint8_t DFU_bmAttributes;
	}user_data;



	struct {
		USBDC_params *usbdc;

		int32_t DFU_event_ID;

		DFU_status_type DFU_state;

#if(USBD_DFU_DFU_MODE_SUPPORT == USBD_FEATURE_ON)
		// number of packet received from Host
		uint16_t DFU_transfer_packet_number;

		// pointer to buffer for data received from Host in DNLOAD
		// mode or for data to be send in UPLOAD mode.
		uint8_t *DFU_transfer_buffer;

		// indicates that device is in manifestation phase or not
#if((USBD_DFU_DNLOAD_SUPPORT == USBD_FEATURE_ON) && (USBD_DFU_MANIFESTATION_TOLERANT_SUPPORT == USBD_FEATURE_ON))
		uint8_t DFU_manifestation_phase = 0;
#endif
#endif

		// number of the DFU interface
		uint8_t DFU_interface;
	}extended_data;
}DFU_interface;



#if(USBD_DFU_APP_MODE_SUPPORT == USBD_FEATURE_ON)
void DFU_APP_init(DFU_interface *DFU);

void DFU_APP_install(USBDC_params *USBDC, DFU_interface *DFU, uint8_t DFU_interface);
#endif


#if(USBD_DFU_DFU_MODE_SUPPORT == USBD_FEATURE_ON)
void DFU_DFU_init(DFU_interface *DFU);

void DFU_DFU_install(USBDC_params *USBDC, DFU_interface *DFU, uint8_t *buffer);
#endif



void DFU_set_timeout(DFU_interface *DFU, uint32_t timeout);



#endif /*DFU_H_*/
