/*
================================================================================
 File: hid.c
--------------------------------------------------------------------------------
 Author:        Piotr Wojtowicz
 Purpose:       USBDevice HID class library
 Compiler:      ANSI C
 Hardware:      
 Doc:
--------------------------------------------------------------------------------
 Includes:      
--------------------------------------------------------------------------------
 Comments:                      UWAGA!!!
                Oprogramowanie to udostępnione jest do celów poglądowych,
                nie jest przetestowane pod kątem bezpieczeństwa ani spełnienia
                jakichkolwiek norm wymaganych aby urządzenie używające go
                mogło być dopuszczone do użytku, dlatego używasz go
                na własną odpowiedzialność a jego autor nie ponosi żadnego
                typu odpowiedzialności za jakiekolwiek skutki jego użycia.
                Jeżeli nie zgadzasz się z powyższym warunkiem proszę
                o nieużywanie tego oprogramowania.
================================================================================
*/

#ifndef HID_H_
#include "hid.h"
#endif

#include <string.h>




static const uint8_t HID_zero_const = 0;
static const uint8_t HID_one_const = 1;









static void HID_Control_close_transmition (USBDC_params *usbdc, USBDEP_params *ep) {
	USBD_send_status(0, 0, usbdc);
	usbdc->ep_irq[0] = 0;
}








static void HID_Control_handler (USBDC_params *usbdc, USBDEP_params *ep, uint8_t interface_num, USBDR_params *request) {
	HID_interface *interface = (HID_interface *)(usbdc->interfaces[interface_num].upper_layer_data.pvoid);
	HID_report *report;

	if(interface) {
		// sprawdzamy, czy mamy do czynienia z Request'em
		// bezposrednio adresowanym do klasy interfejsu
		// (bity 5 i 6 Type w polu bmRequestType
		// ustawione na Class czy tez Standard
		if(!(request->Request.RequestFields.bmRequestType & USBD_BMREQUESTTYPE_CLASS)) {
			switch (request->Request.WholeRequestValue) {

				// -------------------------------
				case HID_GET_DESCRIPTOR_INTERFACE:
				// -------------------------------
					// report descriptor
					if(request->wValue == 0x2200) {
						if(request->wLength >= (interface->user_data.report_descriptor_size)) {
							USBD_send_data_in((interface->user_data.report_descriptor), (interface->user_data.report_descriptor_size), 0, usbdc);
						}
						else {
							USBD_send_data_in((interface->user_data.report_descriptor), request->wLength, 0, usbdc);
						}
					}
					// TODO: PHYSICAL DESCRIPTOR
					break;

				// -------------------------------
				case HID_SET_DESCRIPTOR_INTERFACE:
				// -------------------------------
					USBD_send_stall(0, usbdc);
					break;


				default:
					break;
			}
		}
		// Class Request
		else {
			report = interface->user_data.reports_table[request->wValue & 0xFF];
			if(report) {
				switch (request->Request.WholeRequestValue) {

					// ---------------------------
					case HID_GET_REPORT_INTERFACE:
					// ---------------------------
						switch (request->wValue & 0xFF00) {
							case 0x100:
								if(request->wLength >= (report->report_in_size)) {
									USBD_send_data_in (report->report_in, report->report_in_size, 0, usbdc);
								}
								else {
									USBD_send_data_in (report->report_in, request->wLength, 0, usbdc);
								}
								break;

							case 0x200:
								if(request->wLength >= (report->report_out_size)) {
									USBD_send_data_in (report->report_out, report->report_out_size, 0, usbdc);
								}
								else {
									USBD_send_data_in (report->report_out, request->wLength, 0, usbdc);
								}
								break;

							case 0x300:
								if(request->wLength >= (report->report_feature_size)) {
									USBD_send_data_in (report->report_feature, report->report_feature_size, 0, usbdc);
								}
								else {
									USBD_send_data_in (report->report_feature, request->wLength, 0, usbdc);
								}
								break;

							default:
								break;
						}
						break;

					// ---------------------------
					case HID_SET_REPORT_INTERFACE:
					// ---------------------------
						switch (request->wValue & 0xFF00) {

							case 0x200:	//OUT Report
								USBD_recv_data_out (report->report_out, report->report_out_size, 0, usbdc);
								// if receive process ended
								if(!(ep->transfer_params.transfer_type & USBD_TRANSFER_PRESENT)) {
									if(ep->transfer_params.transfer_state != USBD_TOO_LESS_SPACE) {
										USBD_send_status(0, 0, usbdc);
									}
									else {
										ep->transfer_params.transfer_state = USBD_CLEAR_ALL_FLAGS;
									}
								}
								// receive is processing
								// HID_Control_close_transmition function will close
								// control transfer as result of DATA OUT end_of_transfer
								// event handler
								else {
									usbdc->ep_irq[0] = &HID_Control_close_transmition;
								}
								break;

							case 0x300:	//FEATURE Report
								USBD_recv_data_out (report->report_feature, report->report_feature_size, 0, usbdc);
								// if receive process ended
								if(!(ep->transfer_params.transfer_type & USBD_TRANSFER_PRESENT)) {
									if(ep->transfer_params.transfer_state != USBD_TOO_LESS_SPACE) {
										USBD_send_status(0, 0, usbdc);
									}
									else {
										ep->transfer_params.transfer_state = USBD_CLEAR_ALL_FLAGS;
									}
								}
								// receive is processing
								// HID_Control_close_transmition function will close
								// control transfer as result of DATA OUT end_of_transfer
								// event handler
								else {
									usbdc->ep_irq[0] = &HID_Control_close_transmition;
								}
								break;

							case 0x100:	//IN Report
								USBD_recv_data_out (report->report_in, report->report_in_size, 0, usbdc);
								// if receive process ended
								if(!(ep->transfer_params.transfer_type & USBD_TRANSFER_PRESENT)) {
									if(ep->transfer_params.transfer_state != USBD_TOO_LESS_SPACE) {
										USBD_send_status(0, 0, usbdc);
									}
									else {
										ep->transfer_params.transfer_state = USBD_CLEAR_ALL_FLAGS;
									}
								}
								// receive is processing
								// HID_Control_close_transmition function will close
								// control transfer as result of DATA OUT end_of_transfer
								// event handler
								else {
									usbdc->ep_irq[0] = &HID_Control_close_transmition;
								}
								break;

							default:
								break;
						}
						break;

					// -------------------------
					case HID_GET_IDLE_INTERFACE:
					// -------------------------
						USBD_send_data_in (&(report->idle_rate), 1, 0, usbdc);
						break;

					// -------------------------
					case HID_SET_IDLE_INTERFACE:
					// -------------------------
						if(request->wValue & 0xFF) {
							report->idle_rate = (uint8_t)((request->wValue >> 8) & 0xFF);
						}
						else {
							for(; (request->wValue & 0xFF) < interface->user_data.num_reports; request->wValue++) {
								report = interface->user_data.reports_table[request->wValue & 0xFF];
								if(report) {
									report->idle_rate = (uint8_t)((request->wValue >> 8) & 0xFF);
								}
							}
						}
						USBD_send_status(0, 0, usbdc);
						break;

					// -----------------------------
					case HID_GET_PROTOCOL_INTERFACE:
					// -----------------------------
						if(interface->user_data.boot_protocol_value) {
							USBD_send_data_in ((uint8_t*) &HID_one_const, 1, 0, usbdc);
						}
						else {
							USBD_send_data_in ((uint8_t*) &HID_zero_const, 1, 0, usbdc);
						}
						break;

					// -----------------------------
					case HID_SET_PROTOCOL_INTERFACE:
					// ----------------------------
						if(request->wValue) {
							interface->user_data.boot_protocol_value = interface->extended_data.hid_bInterfaceProtocol;
						}
						else {
							interface->user_data.boot_protocol_value = 0;
						}
						USBD_send_status(0, 0, usbdc);
						break;

					default:
						USBD_send_stall(0, usbdc);
						break;
				}
			}
		}
	}
}








static void HID_event(USBDC_params *usbdc, uint8_t flags, USBDE_params *event) {
	uint8_t interfaces_counter;
	uint8_t reports_counter;
	USBDEP_params *ep;
	HID_interface *interface;
	HID_report *report;
	uint8_t ep_in_number;
	uint8_t mps;
	uint8_t loop;
	union {
		void *pvoid;
		void (*hvoid)(USBDC_params *, uint8_t, USBDE_params *);
	}owner;

	if(flags == USBDEF_SOF) {
		// ponizsze pole struktury event uzywane jest jako licznik milisekund
		// wykorzystywany do odmierzania opoznien potrzebnych przy generowaniu
		// reportow
		event->upper_layer_data.uint32++;
		// przegladamy wszystkie interfejsy
		for(interfaces_counter = 0; interfaces_counter < HID_MAX_NUM_INTERFACES; interfaces_counter++) {
			owner.pvoid = usbdc->interfaces[interfaces_counter].owner;
			// sprawdzamy czy dany interfejs ma byc obslugiwany przez ta biblioteke
			if(owner.hvoid == HID_event) {
				interface = (HID_interface *)(usbdc->interfaces[interfaces_counter].upper_layer_data.pvoid);
				ep_in_number = interface->extended_data.ep_in_number;

				//przegladniecie wszystkich reportow
				for(reports_counter = 0; reports_counter < interface->user_data.num_reports; reports_counter++) {
					report = interface->user_data.reports_table[reports_counter];
					if(report) {
						// sprawdzamy, czy mamy wysylac reporty co okreslony czas
						// czy tez tylko wtedy jak ulegnie zmianie - gdy idle_rate
						// jest rowne 0 to wysylamy tylko po wykryciu zmiany w reporcie
						if(report->idle_rate) {
							// wysylamy co okreslony czas - sprawdzenie, czy nadszedl czas
							// wyslania reportu
							if(report->time_of_next_report <= event->upper_layer_data.uint32) {
								ep = USBDEP_get_params(ep_in_number, usbdc);
								// sprawdamy, czy nie trwa przypadkiem transmisja
								// na tym punkcie koncowym - jezeli tak, to konczymy
								// a funkcja HID_in_irq zajmie sie dalszym wysylaniem danych
								if(!(ep->transfer_params.transfer_type & USBD_TRANSFER_PRESENT)) {
									mps = ep->max_packet_size;
									// sprawdzenie, czy wysylany report zmiesci sie w jednym pakiecie
									// oraz czy uzywamy 2 buforow reportu
									if(report->report_in_second_buffer && (report->report_in_size > mps)) {
										// nie zmiesci sie i mamy 2 bufor - wysylamy pakiet o rozmiarze
										// takim jak rozmiar punktu koncowego (aby juz w miedzy czasie
										// dane byly transferowane przez peryferium)
										USBD_send_data_in (report->report_in, mps, ep_in_number, usbdc);

										// teraz jak czesc danych leci juz do Hosta
										// kopiujemy reszte naszego reportu do zapasowego bufora
										for(loop = mps; report->report_in_size > loop; loop++) {
											report->report_in_second_buffer[loop] = report->report_in[loop];
										}

										// nadpisujemy transfer IN - poprostu doklejamy skopiowane dane
										// (ktore sa juz w buforze zapasowym) do poczatku wyslanego wczesniej
										// pakietu danych (tak aby wyslac caly bufor jako jeden pakiet)
										USBD_send_data_in (&(report->report_in_second_buffer[mps]), report->report_in_size - mps, ep_in_number, usbdc);
									}
									else {
										USBD_send_data_in (report->report_in, report->report_in_size, ep_in_number, usbdc);
									}
									report->is_sending = 1;
									report->has_been_changed = 0;
									report->time_of_next_report = event->upper_layer_data.uint32 + ((uint32_t)report->idle_rate) * 4;
									interface->extended_data.report_in_use = reports_counter;
									break;
								}
							}
						}
						// wysylamy tylko wtedy, gdy report zostal zmodyfikowany
						else {
							if(report->has_been_changed) {
								ep = USBDEP_get_params(ep_in_number, usbdc);
								// sprawdamy, czy nie trwa przypadkiem transmisja
								// na tym punkcie koncowym - jezeli tak, to konczymy
								// a funkcja HID_in_irq zajmie sie dalszym wysylaniem danych
								if(!(ep->transfer_params.transfer_type & USBD_TRANSFER_PRESENT)) {
									mps = ep->max_packet_size;
									// sprawdzenie, czy wysylany report zmiesci sie w jednym pakiecie
									// oraz czy uzywamy 2 buforow reportu
									if(report->report_in_second_buffer && (report->report_in_size > mps)) {
										// nie zmiesci sie i mamy 2 bufor - wysylamy pakiet o rozmiarze
										// takim jak rozmiar punktu koncowego (aby juz w miedzy czasie
										// dane byly transferowane przez peryferium)
										USBD_send_data_in (report->report_in, mps, ep_in_number, usbdc);

										// teraz jak czesc danych leci juz do Hosta
										// kopiujemy reszte naszego reportu do zapasowego bufora
										for(loop = mps; report->report_in_size > loop; loop++) {
											report->report_in_second_buffer[loop] = report->report_in[loop];
										}

										// nadpisujemy transfer IN - poprostu doklejamy skopiowane dane
										// (ktore sa juz w buforze zapasowym) do poczatku wyslanego wczesniej
										// pakietu danych (tak aby wyslac caly bufor jako jeden pakiet)
										USBD_send_data_in (&(report->report_in_second_buffer[mps]), report->report_in_size - mps, ep_in_number, usbdc);
									}
									else {
										USBD_send_data_in (report->report_in, report->report_in_size, ep_in_number, usbdc);
									}
									report->is_sending = 1;
									report->has_been_changed = 0;
//									report->time_of_next_report = event->upper_layer_data.uint32 + ((uint32_t)report->idle_rate) * 4;
									interface->extended_data.report_in_use = reports_counter;
									break;
								}
							}
						}
					}
				}
			}
		}
	}
	else if(flags == USBDEF_CONFIGURED) {
		event->upper_layer_data.uint32 = 0;
		// przegladamy wszystkie interfejsy
		for(interfaces_counter = 0; interfaces_counter < HID_MAX_NUM_INTERFACES; interfaces_counter++) {
			owner.pvoid = usbdc->interfaces[interfaces_counter].owner;
			// sprawdzamy czy dany interfejs ma byc obslugiwany przez ta biblioteke
			if(owner.hvoid == HID_event) {
				interface = (HID_interface *)(usbdc->interfaces[interfaces_counter].upper_layer_data.pvoid);

				//przegladniecie wszystkich reportow, inicjalizacja
				for(reports_counter = 0; reports_counter < interface->user_data.num_reports; reports_counter++) {
					report = interface->user_data.reports_table[reports_counter];
					if(report) {
						report->time_of_next_report = ((uint32_t)report->idle_rate) * 4;
					}
				}
			}
		}
	}
	else if(flags == USBDEF_UNCONFIGURED) {
		// przegladamy wszystkie interfejsy
		for(interfaces_counter = 0; interfaces_counter < HID_MAX_NUM_INTERFACES; interfaces_counter++) {
			owner.pvoid = usbdc->interfaces[interfaces_counter].owner;
			// sprawdzamy czy dany interfejs ma byc obslugiwany przez ta biblioteke
			if(owner.hvoid == HID_event) {
				interface = (HID_interface *)(usbdc->interfaces[interfaces_counter].upper_layer_data.pvoid);
				interface->extended_data.report_in_use = 0;
			}
		}
	}
}








static void HID_in_irq (USBDC_params *usbdc, USBDEP_params *ep) {
	uint8_t counter;
	HID_interface *interface;
	HID_report *report;
	uint8_t max_report_number;
	uint8_t mps;
	uint8_t loop;
	USBDE_params *event;

	if(ep->transfer_params.transfer_state == USBD_END_OF_TRANSFER) {
		interface = (HID_interface *)(usbdc->interfaces[ep->interface].upper_layer_data.pvoid);
		event = USBDE_get_params(interface->extended_data.hid_event_id, usbdc);
		if(!event) {
			return;
		}
		counter = interface->extended_data.report_in_use;
		report = interface->user_data.reports_table[counter];

		max_report_number = interface->user_data.num_reports;
		mps = ep->max_packet_size;

		report->is_sending = 0;
		counter++;

		for(; counter < max_report_number; counter++) {
			report = interface->user_data.reports_table[counter];
			if(report) {
				// sprawdzamy, czy mamy wysylac reporty co okreslony czas
				// czy tez tylko wtedy jak ulegnie zmianie - gdy idle_rate
				// jest rowne 0 to wysylamy tylko po wykryciu zmiany w reporcie
				if(report->idle_rate) {
					// wysylamy co okreslony czas - sprawdzenie, czy nadszedl czas
					// wyslania reportu
					if(report->time_of_next_report <= event->upper_layer_data.uint32) {
						// sprawdzenie, czy wysylany report zmiesci sie w jednym pakiecie
						// oraz czy uzywamy 2 buforow reportu
						if(report->report_in_second_buffer && (report->report_in_size > mps)) {
							// nie zmiesci sie i mamy 2 bufor - wysylamy pakiet o rozmiarze
							// takim jak rozmiar punktu koncowego (aby juz w miedzy czasie
							// dane byly transferowane przez peryferium)
							USBD_send_data_in (report->report_in, mps, interface->extended_data.ep_in_number, usbdc);

							// teraz jak czesc danych leci juz do Hosta
							// kopiujemy reszte naszego reportu do zapasowego bufora
							for(loop = mps; report->report_in_size > loop; loop++) {
								report->report_in_second_buffer[loop] = report->report_in[loop];
							}

							// nadpisujemy transfer IN - poprostu doklejamy skopiowane dane
							// (ktore sa juz w buforze zapasowym) do poczatku wyslanego wczesniej
							// pakietu danych (tak aby wyslac caly bufor jako jeden pakiet)
							USBD_send_data_in (&(report->report_in_second_buffer[mps]), report->report_in_size - mps, interface->extended_data.ep_in_number, usbdc);
						}
						else {
							USBD_send_data_in (report->report_in, report->report_in_size, interface->extended_data.ep_in_number, usbdc);
						}
						report->is_sending = 1;
						report->has_been_changed = 0;
						report->time_of_next_report = event->upper_layer_data.uint32 + ((uint32_t)report->idle_rate) * 4;
						interface->extended_data.report_in_use = counter;
						return;
					}
				}
				// wysylamy tylko wtedy, gdy report zostal zmodyfikowany
				else {
					if(report->has_been_changed) {
						if(report->report_in_second_buffer && (report->report_in_size > mps)) {
							// nie zmiesci sie i mamy 2 bufor - wysylamy pakiet o rozmiarze
							// takim jak rozmiar punktu koncowego (aby juz w miedzy czasie
							// dane byly transferowane przez peryferium)
							USBD_send_data_in (report->report_in, mps, interface->extended_data.ep_in_number, usbdc);

							// teraz jak czesc danych leci juz do Hosta
							// kopiujemy reszte naszego reportu do zapasowego bufora
							for(loop = mps; report->report_in_size > loop; loop++) {
								report->report_in_second_buffer[loop] = report->report_in[loop];
							}

							// nadpisujemy transfer IN - poprostu doklejamy skopiowane dane
							// (ktore sa juz w buforze zapasowym) do poczatku wyslanego wczesniej
							// pakietu danych (tak aby wyslac caly bufor jako jeden pakiet)
							USBD_send_data_in (&(report->report_in_second_buffer[mps]), report->report_in_size - mps, interface->extended_data.ep_in_number, usbdc);
						}
						else {
							USBD_send_data_in (report->report_in, report->report_in_size, interface->extended_data.ep_in_number, usbdc);
						}
						report->is_sending = 1;
						report->has_been_changed = 0;
// 						report->time_of_next_report = event->upper_layer_data.uint32 + ((uint32_t)report->idle_rate) * 4;
						interface->extended_data.report_in_use = counter;
						return;
					}
				}
			}
		}
		interface->extended_data.report_in_use = 0;
	}
	else if(ep->transfer_params.transfer_state == USBD_TRANSFER_ABORTED) {
		interface = (HID_interface *)(usbdc->interfaces[ep->interface].upper_layer_data.pvoid);
		report = interface->user_data.reports_table[interface->extended_data.report_in_use];
		report->is_sending = 0;
	}
}










void HID_init(HID_interface *interface) {
	memset((void*)interface, 0, sizeof(HID_interface));
}










void HID_install (USBDC_params *usbdc, HID_interface *interface, uint8_t interface_number) {
	union {
		void *pvoid;
		void (*hvoid)(USBDC_params *, uint8_t, USBDE_params *);
	}owner;

	owner.hvoid = HID_event;

	usbdc->interfaces[interface_number].upper_layer_data.pvoid = interface;
	usbdc->interfaces[interface_number].owner = owner.pvoid;
	usbdc->ep_irq[interface->extended_data.ep_in_number] = HID_in_irq;
	usbdc->ep0_setup_interface_irq[interface_number] = HID_Control_handler;
	interface->extended_data.hid_event_id = USBDE_install(
			HID_event,
			USBDEM_SOF | USBDEM_CONFIGURED | USBDEM_UNCONFIGURED,
			usbdc);
}









#if(USBD_CCM_SUPPORT == USBD_FEATURE_ON)
static const uint8_t HID_interface_descriptor_pattern[] = {
		// Interface Descriptor
		0x09,							// bLength
		USBD_DESC_TYPE_INTERFACE,		// bDescriptorType
		0x00,							// bInterfaceNumber
		0x00,							// bAlternateSetting
		0x01,							// bNumEndpoints
		0x03,							// bInterfaceClass
		0x00,							// bInterfaceSubclass
		0x00,							// bInterfaceProtocol
		0x00,							// iInterface

		// Interface HID Descriptor
		0x09,							// bLength
		USBD_DESC_TYPE_HID,				// bDescriptorType
		0x01,							// bcdHID
		0x01,
		0x00,							// bCountryCode
		0x01,							// bNumDescriptors
		0x00,							// bDescriptorType
		0x00,							// bDescriptorLength
		0x00
};


static uint8_t HID_add_finish(USBDC_params *usbdc, uint8_t interface_num, CCM_session_params* session) {
	CCM_interface_params_get interface_params;
	CCM_ep_params_get ep_params;
	USBD_endpoint_descriptor *endpoint;
	HID_interface *interface_struct;
	uint16_t ep_desc_shift;
	uint8_t counter;

	// ----------------------------------------------------------------------
	// parse interface
	// ----------------------------------------------------------------------

	// now we gets informations of an interface
	if(CCM_FINISH_get_interface_params(&interface_params, interface_num, session)) {
		return(CCM_FINISH_ERROR);
	}

	// HID Interface has no slave interfaces - check it
	if(interface_params.association.master.num_slaves != 0) {
		return(CCM_FINISH_ERROR);
	}

	// HID Interface has maximum two endpoints - check it
	if(interface_params.num_endpoints > 2) {
		return(CCM_FINISH_ERROR);
	}

	interface_struct = (HID_interface *)(interface_params.pvoid);

	HID_init(interface_struct);

	// set pattern of the interface descriptor
	memcpy(
			interface_params.descriptor,
			HID_interface_descriptor_pattern,
			sizeof(HID_interface_descriptor_pattern));

	// HID_descriptor have specific structure - at its end
	// we can have additional fields if we use physical descriptors
	// following variable indicates this additional space size
	ep_desc_shift = interface_params.desc_size;
	ep_desc_shift -= sizeof(USBD_interface_descriptor) + sizeof(HID_descriptor);
	ep_desc_shift -= interface_params.num_endpoints * sizeof(USBD_endpoint_descriptor);

	// descriptors pointers
	interface_struct->extended_data.interface_descriptor = (USBD_interface_descriptor*)(interface_params.descriptor);
	interface_struct->extended_data.hid_descriptor = (HID_descriptor *)(interface_params.descriptor + sizeof(USBD_interface_descriptor));

	// descriptors corrections
	interface_struct->extended_data.interface_descriptor->bInterfaceNumber = interface_num;
	interface_struct->extended_data.hid_descriptor->bLength += ep_desc_shift;


	// ----------------------------------------------------------------------
	// parse endpoints
	// ----------------------------------------------------------------------
	interface_struct->extended_data.ep_in_number = 0;
	interface_struct->extended_data.ep_out_number = 0;

	for(counter = 0; counter < interface_params.num_endpoints; counter++) {
		// gets parameters of an interrupt endpoint
		// used by the HID Interface
		if(CCM_FINISH_get_ep_params(&ep_params, counter, interface_num, session)) {
			return(CCM_FINISH_ERROR);
		}

		// IN Endpoint
		if((ep_params.type & USBDEP_TYPE_INTERRUPT_IN) == USBDEP_TYPE_INTERRUPT_IN) {
			endpoint = (USBD_endpoint_descriptor*)(interface_params.descriptor + ep_desc_shift + sizeof(USBD_interface_descriptor) + sizeof(HID_descriptor));

			// if this ep was set in previous iteration of the loop
			// and now we have found an endpoint with this same type
			// then we return error status because we can not have
			// two identical endpoints
			if(interface_struct->extended_data.ep_in_number) {
				return(CCM_FINISH_ERROR);
			}

			// save ep number
			interface_struct->extended_data.ep_in_number = ep_params.ep_num;

			// set endpoint descriptor fields
			endpoint->bEndpointAddress = ep_params.ep_num | USBDEP_DESC_DIR_IN;
			endpoint->bInterval = HID_DEFAULT_IN_BINTERVAL;

			interface_struct->extended_data.in_bInterval = &(endpoint->bInterval);
		}
		// OUT Endpoint
		else if((ep_params.type & USBDEP_TYPE_INTERRUPT_OUT) == USBDEP_TYPE_INTERRUPT_OUT) {
			endpoint = (USBD_endpoint_descriptor*)(interface_params.descriptor + ep_desc_shift + sizeof(USBD_interface_descriptor) + sizeof(HID_descriptor) + sizeof(USBD_endpoint_descriptor));

			// if this ep was set in previous iteration of the loop
			// and now we have found an endpoint with this same type
			// then we return error status because we can not have
			// two identical endpoints
			if(interface_struct->extended_data.ep_out_number) {
				return(CCM_FINISH_ERROR);
			}

			// save ep number
			interface_struct->extended_data.ep_out_number = ep_params.ep_num;

			// set endpoint descriptor fields
			endpoint->bEndpointAddress |= ep_params.ep_num | USBDEP_DESC_DIR_OUT;
			endpoint->bInterval = HID_DEFAULT_OUT_BINTERVAL;

			interface_struct->extended_data.out_bInterval = &(endpoint->bInterval);
		}
		else {
			continue;
		}

		// set endpoint descriptor fields
		endpoint->bLength = 7;
		endpoint->bDescriptorType = USBD_DESC_TYPE_ENDPOINT;
		endpoint->bmAttributes = USBDEP_DESC_TRANSFER_TYPE_INTERRUPT;
		endpoint->wMaxPacketSize.L = ep_params.max_packet_size.uint8.L;
		endpoint->wMaxPacketSize.H = ep_params.max_packet_size.uint8.H;
	}

	HID_install(usbdc, interface_struct, interface_num);

	return(CCM_ERROR_NONE);
}


uint8_t HID_add_item(HID_interface *interface, uint8_t use_ep_in, uint8_t use_ep_out, uint8_t num_physical_descriptors, CCM_session_params* session) {
	uint8_t result;
	union {
		uint8_t interface;
		uint16_t ret;
		uint8_t uint8_tab[2];
	}hid_interface;
	CCM_ep_params_add ep_params;
	uint8_t desc_size;

	if(!interface || !session) {
		return(CCM_ERROR_BAD_PARAMS);
	}

	desc_size = (uint16_t)num_physical_descriptors;
	desc_size *= 3;
	desc_size += sizeof(USBD_interface_descriptor) + sizeof(HID_descriptor);
	if(use_ep_in) {
		desc_size += sizeof(USBD_endpoint_descriptor);
	}
	if(use_ep_out) {
		desc_size += sizeof(USBD_endpoint_descriptor);
	}

	// ----------------------------------------------------------------------
	// HID Interface
	// ----------------------------------------------------------------------
	// add hid interface
	hid_interface.ret = CCM_add_interface(interface, HID_add_finish, desc_size, session);

	// try result
	if(hid_interface.ret >= USBD_MAX_NUM_INTERFACES) {
		return(hid_interface.uint8_tab[1]);
	}

	// --------------------------------------------------
	// In Endpoint
	// --------------------------------------------------
	if(use_ep_in) {
		// prepare parameters of this endpoint
		ep_params.ep_id = 0;
		ep_params.type = USBDEP_TYPE_INTERRUPT_IN;
		ep_params.max_packet_size.uint16 = USBDEP_HS_BULK_MAX_POSSIBLE_PACKET_SIZE;
		ep_params.max_packet_size_priority = 60;
		ep_params.interface_num = hid_interface.interface;

		// add INTERRUPT_IN endpoint
		result = CCM_add_ep(&ep_params, session);

		// CCM_add_ep failed
		if(result) {
			CCM_remove_last_interface(session);
			return(result);
		}
	}

	// --------------------------------------------------
	// Out Endpoint
	// --------------------------------------------------
	if(use_ep_out) {
		// prepare parameters of this endpoint
		ep_params.ep_id = 1;
		ep_params.type = USBDEP_TYPE_INTERRUPT_OUT;
		ep_params.max_packet_size.uint16 = USBDEP_HS_BULK_MAX_POSSIBLE_PACKET_SIZE;
		ep_params.max_packet_size_priority = 60;
		ep_params.interface_num = hid_interface.interface;

		// add INTERRUPT_OUT endpoint
		result = CCM_add_ep(&ep_params, session);

		// CCM_add_ep failed
		if(result) {
			CCM_remove_last_interface(session);
			return(result);
		}
	}

	// hid interface reserved successfully
	return(CCM_ERROR_NONE);
}




void HID_make_it_bootable(HID_interface *interface, uint8_t protocol_value) {
	if(!interface) {
		return;
	}

	if(!(interface->extended_data.interface_descriptor)) {
		return;
	}

	if(protocol_value > 2) {
		return;
	}

	if(protocol_value) {
		interface->extended_data.interface_descriptor->bInterfaceSubClass = 1;
	}
	else {
		interface->extended_data.interface_descriptor->bInterfaceSubClass = 0;
	}
	interface->extended_data.interface_descriptor->bInterfaceProtocol = protocol_value;
	interface->extended_data.hid_bInterfaceProtocol = protocol_value;
}




void HID_set_reports_table(HID_interface *interface, HID_report **reports_table, uint8_t num_reports) {
	if(!interface || !reports_table || !num_reports) {
		return;
	}

	interface->user_data.reports_table = reports_table;
	interface->user_data.num_reports = num_reports;
}




void HID_set_report_descriptor(HID_interface *interface, uint8_t *descriptor, uint16_t desc_length) {
	union {
		uint16_t uint16;
		uint8_t uint8_tab[2];
	}length;

	if(!interface) {
		return;
	}

	if(!(interface->extended_data.hid_descriptor)) {
		return;
	}

	interface->extended_data.hid_descriptor->bDescriptorType_2 = USBD_DESC_TYPE_REPORT;
	length.uint16 = desc_length;
	interface->extended_data.hid_descriptor->wDescriptorLength_L = length.uint8_tab[0];
	interface->extended_data.hid_descriptor->wDescriptorLength_H = length.uint8_tab[1];

	interface->user_data.report_descriptor = descriptor;
	interface->user_data.report_descriptor_size = desc_length;
}
#endif
