/*
================================================================================
 File: hid.c
--------------------------------------------------------------------------------
 Author:        Piotr Wojtowicz
 Purpose:       USBDevice HID class library
 Compiler:      ANSI C
 Hardware:      
 Doc:
--------------------------------------------------------------------------------
 Includes:      
--------------------------------------------------------------------------------
 Comments:                      UWAGA!!!
                Oprogramowanie to udostępnione jest do celów poglądowych,
                nie jest przetestowane pod kątem bezpieczeństwa ani spełnienia
                jakichkolwiek norm wymaganych aby urządzenie używające go
                mogło być dopuszczone do użytku, dlatego używasz go
                na własną odpowiedzialność a jego autor nie ponosi żadnego
                typu odpowiedzialności za jakiekolwiek skutki jego użycia.
                Jeżeli nie zgadzasz się z powyższym warunkiem proszę
                o nieużywanie tego oprogramowania.
================================================================================
*/

#ifndef HID_H_
#define HID_H_

#include <stdint.h>

#ifndef USBD_H_
#include "usbd.h"
#endif



#ifndef HID_MAX_NUM_INTERFACES
#define HID_MAX_NUM_INTERFACES					USBD_MAX_NUM_INTERFACES
#endif

#ifndef HID_PHYSICAL_DESCRIPTORS_SUPPORT
#define HID_PHYSICAL_DESCRIPTORS_SUPPORT		USBD_FEATURE_ON
#endif

#ifndef HID_DEFAULT_IN_BINTERVAL
#define HID_DEFAULT_IN_BINTERVAL				1
#endif

#ifndef HID_DEFAULT_OUT_BINTERVAL
#define HID_DEFAULT_OUT_BINTERVAL				1
#endif



#define USBD_DESC_TYPE_HID						0x21
#define USBD_DESC_TYPE_REPORT					0x22
#define USBD_DESC_TYPE_PHYSICAL					0x23



#define HID_BINTERFACEPROTOCOL_TYPE_NONE		0
#define HID_BINTERFACEPROTOCOL_TYPE_KEYBOARD	1
#define HID_BINTERFACEPROTOCOL_TYPE_MOUSE		2



typedef enum {
USBD_HID_BREQUEST_GET_REPORT    = 0x01,
USBD_HID_BREQUEST_SET_REPORT    = 0x09,

USBD_HID_BREQUEST_GET_IDLE      = 0x02,
USBD_HID_BREQUEST_SET_IDLE      = 0x0A,

USBD_HID_BREQUEST_GET_PROTOCOL  = 0x03,
USBD_HID_BREQUEST_SET_PROTOCOL  = 0x0B
}USBDR_bRequest_HID_types;




typedef enum {
HID_GET_DESCRIPTOR_INTERFACE	= (USBD_BMREQUESTTYPE_GET_SYNCH_INTERFACE | (USBD_BREQUEST_GET_DESCRIPTOR << 8)),
HID_SET_DESCRIPTOR_INTERFACE	= (USBD_BMREQUESTTYPE_SET_CLEAR_INTERFACE | (USBD_BREQUEST_SET_DESCRIPTOR << 8)),

HID_GET_REPORT_INTERFACE		= (USBD_BMREQUESTTYPE_GET_SYNCH_INTERFACE | USBD_BMREQUESTTYPE_CLASS | (USBD_HID_BREQUEST_GET_REPORT << 8)),
HID_SET_REPORT_INTERFACE		= (USBD_BMREQUESTTYPE_SET_CLEAR_INTERFACE | USBD_BMREQUESTTYPE_CLASS | (USBD_HID_BREQUEST_SET_REPORT << 8)),

HID_GET_IDLE_INTERFACE			= (USBD_BMREQUESTTYPE_GET_SYNCH_INTERFACE | USBD_BMREQUESTTYPE_CLASS | (USBD_HID_BREQUEST_GET_IDLE << 8)),
HID_SET_IDLE_INTERFACE			= (USBD_BMREQUESTTYPE_SET_CLEAR_INTERFACE | USBD_BMREQUESTTYPE_CLASS | (USBD_HID_BREQUEST_SET_IDLE << 8)),

HID_GET_PROTOCOL_INTERFACE		= (USBD_BMREQUESTTYPE_GET_SYNCH_INTERFACE | USBD_BMREQUESTTYPE_CLASS | (USBD_HID_BREQUEST_GET_PROTOCOL << 8)),
HID_SET_PROTOCOL_INTERFACE		= (USBD_BMREQUESTTYPE_SET_CLEAR_INTERFACE | USBD_BMREQUESTTYPE_CLASS | (USBD_HID_BREQUEST_SET_PROTOCOL << 8))
}USBDR_HID_types;




typedef struct {
	uint8_t bLength;
	uint8_t bDescriptorType;
	uint8_t bcdHID_L;
	uint8_t bcdHID_H;
	uint8_t bCountryCode;
	uint8_t bNumDescriptors;
	uint8_t bDescriptorType_2;
	uint8_t wDescriptorLength_L;
	uint8_t wDescriptorLength_H;
}HID_descriptor;




typedef struct {
	// wskaznik do tablicy pelniacej role pierwszego bufora,
	// w ktorym znajduja sie dane do wyslania
	uint8_t *report_in;

	// wskaznik do drugiej tablicy takiej samej jak Report_IN
	// 		UWAGA !!!
	// jezeli nie jest uzywany drugi bufor,
	// wowczas ten wskaznik nalezy wypelnic zerem
	uint8_t *report_in_second_buffer;

	// wskaznik do tablicy pelniacej role bufora
	// odbierajacego reporty
	uint8_t *report_out;

	uint8_t *report_feature;

	// rozmiar bufora nadawczego
	// 		UWAGA !!!
	// jezeli uywane sa numery reportow, wowczas pierwsza
	// komorka bufora reportu (zarowno pierwszego
	// jak i rezerwowego) zawiera numer reportu, dlatego
	// rozmiar reportu musi tez uwzgledniac
	// dlugosc numeru reportu
	uint8_t  report_in_size;
	uint8_t  report_out_size;
	uint8_t  report_feature_size;
	uint8_t  idle_rate;
	uint32_t time_of_next_report;
	uint8_t  has_been_changed;
	uint8_t  is_sending;
} HID_report;


typedef struct {
	// struktura z polami najczesciej uzywanymi przez uzytkownika
	// wymaga ustawienia wszystkich pol, chyba ze zaznaczone jest inaczej
	struct {
		// wskaznik do deskryptora reportow
		uint8_t *report_descriptor;

		// rozmiar deskryptora reportow
		uint16_t report_descriptor_size;

		// zmienna mowiaca o tym, czy pracujemy aktualnie
		// w trybie BOOT czy REPORT (dla urzadzen bootowalnych)
		// pole zerowane przy inicjowaniu struktury, nie wymaga
		// ustawiania wartosci poczatkowej
		uint8_t boot_protocol_value;

		// liczba reportow zaimplementowanych w interfejsie -
		// jezeli uzywany jest tylko jeden report
		// i nie uzywane sa numery reportow (Report_ID)
		// wowczas zmienna ta wynosi jeden
		// jezeli uzywane sa numery reportow, wowczas
		// ta zmienna musi uwzgledniac tez nieuzywany report 0
		// dlatego zmienna ta musi byc o jeden wieksza
		// od faktycznej liczby reportow, np dla 2 reportow
		// zmienna ta wynosi 3
		uint8_t num_reports;

		// wskaznik do tablicy zawierajacej wskazniki
		// do struktor opisujacych reporty
		HID_report **reports_table;
	}user_data;



	// struktura z polami zazwyczaj nie uzywanymi przez uzytkownika
	// jesli uzywamy wsparcia ze strony biblioteki USBD_CCM wowczas
	// pola te sa automatycznie ustawiane przy dodawaniu interfejsu
	struct {
#if(USBD_CCM_SUPPORT == USBD_FEATURE_ON)
		// jesli uzywamy endpointa typu INTERRUPT IN
		// wowczas jest to wskaznik do pola bInterval deskryptora
		// tego endpointa - w przeciwnym przypadku zero
		uint8_t *in_bInterval;

		// jesli uzywamy endpointa typu INTERRUPT OUT
		// wowczas jest to wskaznik do pola bInterval deskryptora
		// tego endpointa - w przeciwnym przypadku zero
		uint8_t *out_bInterval;

		// wskaznik do deskryptora HID tego interfejsu
		HID_descriptor *hid_descriptor;

		// wskaznik do deskryptora interfejsu
		USBD_interface_descriptor *interface_descriptor;
#endif

		// zmienna wykorzystywana przez warstwe HID
		// przechowuje ID eventu HID, potrzebny przy
		// prowadzeniu transmisji danych
		uint32_t hid_event_id;

		// numer reportu, ktory jest w danej chwili przetwarzany
		uint8_t report_in_use;

		// zmienna zawierajaca kopie pola bInterfaceProtocol
		// deskryptora interfejsu
		uint8_t hid_bInterfaceProtocol;


		// numer Endpointa typu IN uzywanego przez dany interfejs
		uint8_t ep_in_number;

		// numer Endpointu typu OUT ewentualnie uzywanego
		// przez dany interfejs - jezeli nie uzywany, wowczas 0
		uint8_t ep_out_number;
	}extended_data;



} HID_interface;




void HID_init(HID_interface *interface);

void HID_install (USBDC_params *usbdc, HID_interface *interface, uint8_t interface_number);

#if(USBD_CCM_SUPPORT == USBD_FEATURE_ON)
uint8_t HID_add_item(HID_interface *interface, uint8_t use_ep_in, uint8_t use_ep_out, uint8_t num_physical_descriptors, CCM_session_params* session);

/*
 * Zmienia ustawienia w deskryptorze interfejsu - modyfikuje pola
 * bInterfaceSubClass i bInterfaceProtocol w zaleznosci
 * od parametru protocol_value
 *
 * nie modyfikuje pol HID_interface->user_data
 **/
void HID_make_it_bootable(HID_interface *interface, uint8_t protocol_value);

/*
 * Podpina deskryptor reportu do struktury "interface"
 *
 * modyfikowane pola HID_interface->user_data:
 *   - reports_table
 *   - num_reports
 **/
void HID_set_reports_table(HID_interface *interface, HID_report **reports_table, uint8_t num_reports);

/*
 * Podpina deskryptor reportu do struktury "interface"
 *
 * modyfikowane pola HID_interface->user_data:
 *   - report_descriptor
 *   - report_descriptor_size
 **/
void HID_set_report_descriptor(HID_interface *interface, uint8_t *descriptor, uint16_t desc_length);
#endif



#endif
