/*
================================================================================
 File: usbd.h
--------------------------------------------------------------------------------
 Author:        Piotr Wojtowicz
 Purpose:       USBDevice virtual layer
 Compiler:      ANSI C
 Hardware:      
 Doc:
--------------------------------------------------------------------------------
 Includes:      
--------------------------------------------------------------------------------
 Comments:                      UWAGA!!!
                Oprogramowanie to udostępnione jest do celów poglądowych,
                nie jest przetestowane pod kątem bezpieczeństwa ani spełnienia
                jakichkolwiek norm wymaganych aby urządzenie używające go
                mogło być dopuszczone do użytku, dlatego używasz go
                na własną odpowiedzialność a jego autor nie ponosi żadnego
                typu odpowiedzialności za jakiekolwiek skutki jego użycia.
                Jeżeli nie zgadzasz się z powyższym warunkiem proszę
                o nieużywanie tego oprogramowania.
================================================================================
*/

#ifndef USBD_H_
#include "usbd.h"
#endif

#include <string.h>










static USBD_params *USBD_CORE_port_pointer = 0;








void USBD_install_port (USBD_params *usbd_private, USBDC_params *usbdc) {
	USBD_CORE_port_pointer = usbd_private;
}







USBD_device_descriptor *USBD_get_dev_descriptor_pointer(USBDC_params *usbdc) {
	if(USBD_CORE_port_pointer) {
		if(USBD_CORE_port_pointer->session_activity & 0xF0) {
			return(0);
		}
		return(USBD_CORE_port_pointer->device_descriptor);
	}
	return(0);
}








void USBDC_init (USBDC_params *usbdc) {
	memset((void*)usbdc, 0, sizeof(USBDC_params));
}







uint8_t USBDC_get_active_number (USBDC_params *usbdc) {
	if(USBD_CORE_port_pointer) {
		return(USBD_CORE_port_pointer->active_config_number);
	}
	return(0);
}








#if(USBD_ZERO_CONFIG_SUPPORT == USBD_FEATURE_ON)
void USBDZC_uninstall (USBDC_params *usbdc) {
	if(USBD_CORE_port_pointer) {
		USBD_CORE_port_pointer->configurations[0] = 0;
	}
}
#endif








#if(USBD_MULTI_SESSION_SUPPORT == USBD_FEATURE_ON)
void USBDC_uninstall (USBDC_params *usbdc) {
	uint8_t counter;

	if(USBD_CORE_port_pointer) {
		if(USBD_CORE_port_pointer->session_activity & 0xF0) {
			return;
		}

		// wstepnie ustalamy numer konfiguracji do odpiecia
		// na podstawie jej numeru w deskryptorze konfiguracji
		counter = usbdc->configuration_descriptor[5];
		
		// teraz sprawdzamy, czy pod komorka tablicy konfiguracji
		// wskazywana przez ustalony wczesniej numer znajduje sie rzeczywiscie
		// odpinana konfiguracja lub czy ten numer nie jest przypadkiem
		// wiekszy od licznika konfiguracji znajdujacego sie
		// w deskryptorze urzadzenia - w takim przypadku przeszukujemy
		// tablice konfiguracji
		if((USBD_CORE_port_pointer->configurations[counter] != usbdc) || (counter > USBD_CORE_port_pointer->device_descriptor->bNumConfigurations)) {
			for(counter = 1; counter <= USBD_CORE_port_pointer->device_descriptor->bNumConfigurations; counter++) {
				if(USBD_CORE_port_pointer->configurations[counter] == usbdc) {
					break;
				}
			}
		}
		// jezeli zmienna counter jest poprawna (wskazuje na komorke
		// tablicy konfiguracji zawierajaca rzeczywiscie nasza konfiguracje)
		// to mozemy teraz dokonac dalszych czynnosci zwiazanych z odinstalowywaniem
		if(counter <= USBD_CORE_port_pointer->device_descriptor->bNumConfigurations) {
			// przesuwamy wszystkie nastepne konfiguracje w gore
			while(counter < USBD_CORE_port_pointer->device_descriptor->bNumConfigurations) {
				USBD_CORE_port_pointer->configurations[counter] = USBD_CORE_port_pointer->configurations[counter + 1];
				// dla kazdej presuwanej konfiguracji ustalamy jej nowy numer
				USBD_CORE_port_pointer->configurations[counter]->configuration_descriptor[5] = counter;
				counter++;
			}
			USBD_CORE_port_pointer->configurations[USBD_CORE_port_pointer->device_descriptor->bNumConfigurations] = 0;
			USBD_CORE_port_pointer->device_descriptor->bNumConfigurations--;
#if(USBD_MAX_NUM_STRINGS > 0)
			USBDC_uninstall_strings(usbdc);
#endif
		}
	}
}
#endif








#if(USBD_MULTI_SESSION_SUPPORT == USBD_FEATURE_ON)
void USBDC_uninstall_all (USBDC_params *usbdc) {
	uint8_t counter;

	if(USBD_CORE_port_pointer) {
		if(USBD_CORE_port_pointer->session_activity & 0xF0) {
			return;
		}

		for(counter = 1; counter <= USBD_CORE_port_pointer->device_descriptor->bNumConfigurations; counter++) {
			USBD_CORE_port_pointer->configurations[counter] = 0;
		}
		USBD_CORE_port_pointer->device_descriptor->bNumConfigurations = 0;

#if(USBD_MAX_NUM_STRINGS > 0)
		// uninstall all strings except device strings
		USBD_CORE_port_pointer->installed_strings_count = 1;
		if(USBD_CORE_port_pointer->device_descriptor->iManufacturer && (USBD_CORE_port_pointer->installed_strings_count < (USBD_MAX_NUM_STRINGS + 1))) {
			USBD_CORE_port_pointer->installed_strings_count++;
		}
		if(USBD_CORE_port_pointer->device_descriptor->iProduct && (USBD_CORE_port_pointer->installed_strings_count < (USBD_MAX_NUM_STRINGS + 1))) {
			USBD_CORE_port_pointer->installed_strings_count++;
		}
		if(USBD_CORE_port_pointer->device_descriptor->iSerialNumber && (USBD_CORE_port_pointer->installed_strings_count < (USBD_MAX_NUM_STRINGS + 1))) {
			USBD_CORE_port_pointer->installed_strings_count++;
		}
#endif
	}
}
#endif








#if((USBD_MULTI_SESSION_SUPPORT == USBD_FEATURE_ON) && (USBD_MAX_NUM_STRINGS > 0))
void USBDC_uninstall_strings (USBDC_params *usbdc) {
	uint8_t counter1;
	uint8_t counter2;
	uint8_t counter3;
	uint8_t uninstalled_strings_count;

	if(USBD_CORE_port_pointer) {
		if(USBD_CORE_port_pointer->session_activity & 0xF0) {
			return;
		}

		counter1 = 0;

		// przegladamy po kolei wszystkie zainstalowane stringi
		while(counter1 < USBD_CORE_port_pointer->installed_strings_count) {
			// znalezlismy pierwszy string, ktory spelnia warunek usuniecia
			if(USBD_CORE_port_pointer->strings[counter1].usbdc == usbdc) {
				counter2 = counter1 + 1;
				uninstalled_strings_count = 1;
				// teraz szukamy kolejnych stringow spelniajacych warunek
				// ale szukamy poczawszy od nastepnego stringa
				// takie dzialanie wykonujemy, gdyz konfiguracje
				// zazwyczaj w tym samym momencie instaluja wszystkie
				// swoje stringi - dlatego od razu lepiej zidentyfikowac
				// wieksza liczbe stringow do usuniecia znajdujacych sie
				// jeden po drugim - zaoszczedzamy czas wielokrotnego przenoszenia
				while((USBD_CORE_port_pointer->strings[counter2].usbdc == usbdc) && (counter2 < USBD_CORE_port_pointer->installed_strings_count)) {
					counter2++;
					uninstalled_strings_count++;
				}
				// przesuwamy nastepne stringo w gore o tyle pozycji, ile stringow kasujemy
				for(counter3 = 0; counter2 + counter3 < USBD_CORE_port_pointer->installed_strings_count; counter3++) {
					USBD_CORE_port_pointer->strings[counter1 + counter3].string = USBD_CORE_port_pointer->strings[counter2 + counter3].string;
					USBD_CORE_port_pointer->strings[counter1 + counter3].index_string = USBD_CORE_port_pointer->strings[counter2 + counter3].index_string;
					USBD_CORE_port_pointer->strings[counter1 + counter3].usbdc = USBD_CORE_port_pointer->strings[counter2 + counter3].usbdc;
				}
				// zmniejszamy liczbe zainstalowanych stringow
				USBD_CORE_port_pointer->installed_strings_count -= uninstalled_strings_count;
				
				// jednoczesnie nie zwiekszamy licznika przejzanych stringow!!!
				// dzieje sie tak dlatego, gdyz w miejsce usunietych wpisow
				// te wystepujace po nich i dlatego dalej w miejscu indeksowanym
				// za pomoca zmiennej counter1 mamy nie sprawdzony string
			}
			else {
				counter1++;
			}
		}
	}
}
#endif








#if(USBD_MAX_NUM_STRINGS > 0)
void USBDS_install (uint8_t *string, uint8_t *index_string, USBDC_params *usbdc) {
	uint8_t counter;

	if(USBD_CORE_port_pointer) {
		if(USBD_CORE_port_pointer->session_activity & 0xF0) {
			return;
		}

		if(USBD_CORE_port_pointer->installed_strings_count < (USBD_MAX_NUM_STRINGS + 1)) {
			for(counter = 0; counter < USBD_CORE_port_pointer->installed_strings_count; counter++) {
				if((USBD_CORE_port_pointer->strings[counter].string == string)
					&& (USBD_CORE_port_pointer->strings[counter].index_string == index_string)
						&& (USBD_CORE_port_pointer->strings[counter].usbdc == usbdc)) {
					return;
				}
			}
			USBD_CORE_port_pointer->strings[USBD_CORE_port_pointer->installed_strings_count].string = string;
			USBD_CORE_port_pointer->strings[USBD_CORE_port_pointer->installed_strings_count].index_string = index_string;
			USBD_CORE_port_pointer->strings[USBD_CORE_port_pointer->installed_strings_count].usbdc = usbdc;
			USBD_CORE_port_pointer->installed_strings_count++;
		}
	}
}
#endif








#if((USBD_MULTI_SESSION_SUPPORT == USBD_FEATURE_ON) && (USBD_MAX_NUM_STRINGS > 0))
void USBDS_uninstall (uint8_t *string, uint8_t *index_string, USBDC_params *usbdc) {
	uint8_t counter;

	if(USBD_CORE_port_pointer) {
		if(USBD_CORE_port_pointer->session_activity & 0xF0) {
			return;
		}

		for(counter = 0; counter < USBD_CORE_port_pointer->installed_strings_count; counter++) {
			if((USBD_CORE_port_pointer->strings[counter].string == string)
				&& (USBD_CORE_port_pointer->strings[counter].index_string == index_string)
					&& (USBD_CORE_port_pointer->strings[counter].usbdc == usbdc)) {
				USBD_CORE_port_pointer->installed_strings_count--;
				for(; counter < USBD_CORE_port_pointer->installed_strings_count; counter++) {
					USBD_CORE_port_pointer->strings[counter].string = USBD_CORE_port_pointer->strings[counter + 1].string;
					USBD_CORE_port_pointer->strings[counter].index_string = USBD_CORE_port_pointer->strings[counter + 1].index_string;
					USBD_CORE_port_pointer->strings[counter].usbdc = USBD_CORE_port_pointer->strings[counter + 1].usbdc;
				}
				break;
			}
		}
	}
}
#endif








#if((USBD_MULTI_SESSION_SUPPORT == USBD_FEATURE_ON) && (USBD_MAX_NUM_STRINGS > 0))
void USBDS_uninstall_all (USBDC_params *usbdc) {
	if(USBD_CORE_port_pointer) {
		if(USBD_CORE_port_pointer->session_activity & 0xF0) {
			return;
		}

		USBD_CORE_port_pointer->installed_strings_count = 1;
		USBD_CORE_port_pointer->device_descriptor->iManufacturer = 0;
		USBD_CORE_port_pointer->device_descriptor->iProduct = 0;
		USBD_CORE_port_pointer->device_descriptor->iSerialNumber = 0;
	}
}
#endif








void USBDS_change_Language_string(uint8_t *new_language_string, USBDC_params *usbdc) {
	if(USBD_CORE_port_pointer) {
		if(USBD_CORE_port_pointer->session_activity & 0xF0) {
			return;
		}

		USBD_CORE_port_pointer->strings[0].string = new_language_string;
		USBD_CORE_port_pointer->strings[0].index_string = 0;
		USBD_CORE_port_pointer->strings[0].usbdc = 0;
	}
}








#if(USBD_MAX_NUM_EVENTS > 0)
int32_t USBDE_install (void (*event)(USBDC_params *, uint8_t, USBDE_params *), uint16_t active_events, USBDC_params *usbdc) {
	uint8_t events_counter;
	USBDE_params *USBDE;

	if(event && usbdc) {
		USBDE = usbdc->events;
		// sprawdzenie, czy dany event nie jest juz zainstalowany
		for(events_counter = 0; events_counter < USBD_MAX_NUM_EVENTS; events_counter++) {
			if(USBDE[events_counter].event == event) {
				return((int32_t)events_counter);
			}
		}
		// zainstalowanie eventu - znalezienie wolnego miejsca w tablicy
		// i umieszczenie tam instalowanego Eventu
		for(events_counter = 0; events_counter < USBD_MAX_NUM_EVENTS; events_counter++) {
			if(!USBDE[events_counter].event) {
				USBDE->event = event;
				USBDE->active_events = active_events;
				return((int32_t)events_counter);
			}
		}
	}
	return(-1);
}
#endif








#if(USBD_MAX_NUM_EVENTS > 0)
USBDE_params *USBDE_get_params (int32_t event_id, USBDC_params *usbdc) {
	if((event_id < USBD_MAX_NUM_EVENTS) && usbdc) {
		return(&(usbdc->events[event_id]));
	}
	else {
		return(0);
	}
}
#endif








#if((USBD_MULTI_SESSION_SUPPORT == USBD_FEATURE_ON) && (USBD_MAX_NUM_EVENTS > 0))
void USBDE_uninstall (int32_t event_id, USBDC_params *usbdc) {
	if((event_id < USBD_MAX_NUM_EVENTS) && usbdc) {
		usbdc->events[event_id].event = 0;
		usbdc->events[event_id].active_events = 0;
	}
}
#endif








#if((USBD_MULTI_SESSION_SUPPORT == USBD_FEATURE_ON) && (USBD_MAX_NUM_EVENTS > 0))
void USBDE_uninstall_all (USBDC_params *usbdc) {
	uint8_t events_counter;
	USBDE_params *USBDE;

	if(usbdc) {
		USBDE = usbdc->events;
		for(events_counter = 0; events_counter < USBD_MAX_NUM_EVENTS; events_counter++) {
			USBDE[events_counter].event = 0;
			USBDE[events_counter].active_events = 0;
		}
	}
}
#endif
