/*
================================================================================
 File: usbd.h
--------------------------------------------------------------------------------
 Author:        Piotr Wojtowicz
 Purpose:       USBDevice virtual layer
 Compiler:      ANSI C
 Hardware:      
 Doc:
--------------------------------------------------------------------------------
 Includes:      
--------------------------------------------------------------------------------
 Comments:                      UWAGA!!!
                Oprogramowanie to udostępnione jest do celów poglądowych,
                nie jest przetestowane pod kątem bezpieczeństwa ani spełnienia
                jakichkolwiek norm wymaganych aby urządzenie używające go
                mogło być dopuszczone do użytku, dlatego używasz go
                na własną odpowiedzialność a jego autor nie ponosi żadnego
                typu odpowiedzialności za jakiekolwiek skutki jego użycia.
                Jeżeli nie zgadzasz się z powyższym warunkiem proszę
                o nieużywanie tego oprogramowania.
================================================================================
*/

#ifndef USBD_H_
#define USBD_H_

#include <stdint.h>

#ifndef USBD_CONFIG_H_
#include "usbd_config.h"
#endif

#ifndef USBD_CDP_H_
#include "usbd_cdp.h"
#endif

#ifndef USBD_PORT_H_
#include "usbd_port.h"
#endif


// objasnienia uzywanych skrotow:
// USBD   - USB Device
// EP     - Endpoint
// USBDS  - USBD String
// USBDEP - USBD Endpoint
// USBDI  - USBD Interface
// USBDC  - USBD Configuration
// USBDST - USBD Status
// USBDR  - USBD Request
// USBDE  - USBD Event
// USBDF  - USBD Event Flag
// USBDEM - USBD Event Mask
// CCM    - Configurations and Classes Manager



/* *****************************************************************************
			parametry pracy USBD
   ***************************************************************************** */
#define USBD_FEATURE_OFF    0
#define USBD_FEATURE_ON     1

// jezeli zdefiniowane, to istnieja konfiguracje z interfejsami posiadajacymi alternatywne ustawienia
// i podczas enumeracji istnieje wsparcie dla polecenia SET_INTERFACE_Interface
#ifndef USBD_ALTERNATE_SETTING_SUPPORT
#define USBD_ALTERNATE_SETTING_SUPPORT          USBD_FEATURE_ON
#endif

// jezeli zdefiniowane wowczas wewnatrz struktury konfiguracyjnej
// istnieja elementy wspomagajace wyzsze biblioteki obslugujace np.
// klasy urzadzen
#ifndef USBD_UPPER_LAYER_SUPPORT
#define USBD_UPPER_LAYER_SUPPORT                USBD_FEATURE_ON
#endif

// okresla, czy uzywamy biblioteki CCM - "Configurations
// and Classes Manager" - biblioteka ta sluzy do latwego
// tworzenia konfiguracji
#ifndef USBD_CCM_SUPPORT
#define USBD_CCM_SUPPORT                        USBD_FEATURE_ON
#endif

// okresla czy mamy wsparcie sprzetowe dla wykrywania napiecia VBUS
#ifndef USBD_VBUS_DETECT_SUPPORT
#define USBD_VBUS_DETECT_SUPPORT                USBD_FEATURE_ON
#endif

// okresla czy mamy wsparcie sprzetowe dla sterowania rezystorem Pull-Up na linii DP
#ifndef USBD_DP_PULL_UP_SUPPORT
#define USBD_DP_PULL_UP_SUPPORT                 USBD_FEATURE_ON
#endif

// okresla czy mamy wsparcie dla specyficznych urzadzen okreslanych
// przez producenta (np wsparcie dla pola Vendor w Request-ach)
#ifndef USBD_VENDOR_SUPPORT
#define USBD_VENDOR_SUPPORT                     USBD_FEATURE_ON
#endif

// okresla, czy bedziemy rekonfigurowac USBD wielokrotnie w czasie
// dzialania urzadenia - jezeli nie, wowczas ciala funkcji odpowiedzialnych
// za modyfikacje (nie instalowanie!) nie istnieja
//          UWAGA !!!
// gdy wlaczone wowczas niektore funkcje modyfikujace zmieniaja
// ustawienia w deskryptorach konfiguracji, dlatego nalezy ow deskryptory
// trzymac w pamieci RAM, gdy uzywamy tej opcji
#ifndef USBD_MULTI_SESSION_SUPPORT
#define USBD_MULTI_SESSION_SUPPORT              USBD_FEATURE_ON
#endif

// okresla, czy wspieramy tzw konfiguracje zeorwa. Jesli tak,
// wowczas za pomoca funkcji USBDZC_install mozna ja podpiac.
// Ta konfiguracja ma numer zerowy, jej deskryptor (jesli istnieje)
// jest ignorowany, Host jej nie widzi wiec nie prowadzi z nia
// transmisji. Mozna jednak podpinac do niej Eventy i monitorowac
// status USBD (dla tych celow zostala stworzona)
#ifndef USBD_ZERO_CONFIG_SUPPORT
#define USBD_ZERO_CONFIG_SUPPORT                USBD_FEATURE_ON
#endif

// okresla, czy uzywamy endpointow z podwojnym buforem
// domyslnie wlaczone, wylacza sie w celu zmniejszenia ilosci kodu
#ifndef USBDEP_BULK_SUPPORT
#define USBDEP_BULK_SUPPORT                     USBD_FEATURE_ON
#endif

// okresla, czy uzywamy endpointow z podwojnym buforem
// domyslnie wlaczone, wylacza sie w celu zmniejszenia ilosci kodu
#ifndef USBDEP_INTERRUPT_SUPPORT
#define USBDEP_INTERRUPT_SUPPORT                USBD_FEATURE_ON
#endif

// okresla, czy uzywamy endpointow z podwojnym buforem
// domyslnie wlaczone, wylacza sie w celu zmniejszenia ilosci kodu
#ifndef USBDEP_ISOCHRONOUS_SUPPORT
#define USBDEP_ISOCHRONOUS_SUPPORT              USBD_FEATURE_ON
#endif

// okresla czy uzywamy opcji pozwalajacej na prace krokowa przy odbiorze danych
// poprzez blokowanie nadchodzacych paczek az do momentu wywolania
// funkcji USBDEP_DATA_OUT_ready
#ifndef USBDEP_DATA_OUT_WAIT_FOR_READY_SUPPORT
#define USBDEP_DATA_OUT_WAIT_FOR_READY_SUPPORT  USBD_FEATURE_ON
#endif

// definicje okreslajace czy dla danych zdazen ma byc wywolywana
// funkcja na ktora wskazuje USBD_event_handler - jesli dana definicja
// jest aktywna wowczas funkcja bedzie wywolywana przy okreslonym zdazeniu
// definicje te roznia sie tym od zmiemmej active_events ze te definicje wylaczaja
// z etapu kompilacji kod odpowiedzialny za wywolywanie danego eventu
//          UWAGA!!!
// mozna rowniez wylaczyc globalnie wszystkie eventy z etapu kompilacji
// poprzez ustawienie liczby eventow na 0
#ifndef USBDE_ATTACHED_SUPPORT
#define USBDE_ATTACHED_SUPPORT					USBD_FEATURE_ON
#endif

#ifndef USBDE_DETACHED_SUPPORT
#define USBDE_DETACHED_SUPPORT					USBD_FEATURE_ON
#endif

#ifndef USBDE_POWERED_SUPPORT
#define USBDE_POWERED_SUPPORT					USBD_FEATURE_ON
#endif

#ifndef USBDE_UNPOWERED_SUPPORT
#define USBDE_UNPOWERED_SUPPORT					USBD_FEATURE_ON
#endif

#ifndef USBDE_RESET_SUPPORT
#define USBDE_RESET_SUPPORT						USBD_FEATURE_ON
#endif

#ifndef USBDE_ADDRESSED_SUPPORT
#define USBDE_ADDRESSED_SUPPORT					USBD_FEATURE_ON
#endif

#ifndef USBDE_CONFIGURED_SUPPORT
#define USBDE_CONFIGURED_SUPPORT				USBD_FEATURE_ON
#endif

#ifndef USBDE_UNCONFIGURED_SUPPORT
#define USBDE_UNCONFIGURED_SUPPORT				USBD_FEATURE_ON
#endif

#ifndef USBDE_SUSPENDED_SUPPORT
#define USBDE_SUSPENDED_SUPPORT					USBD_FEATURE_ON
#endif

#ifndef USBDE_WAKEUP_SUPPORT
#define USBDE_WAKEUP_SUPPORT					USBD_FEATURE_ON
#endif

#ifndef USBDE_SOF_SUPPORT
#define USBDE_SOF_SUPPORT						USBD_FEATURE_ON
#endif

// maksymalna liczba konfiguracji, jakie mozna podpiac do DEVICE'a
#ifndef USBD_MAX_NUM_CONFIGURATIONS
    #define USBD_MAX_NUM_CONFIGURATIONS         1
#else
    #if(USBD_MAX_NUM_CONFIGURATIONS > 255)
        #undef USBD_MAX_NUM_CONFIGURATIONS
        #define USBD_MAX_NUM_CONFIGURATIONS     255
    #endif
#endif

// liczba Interfejsow jakie moze posiadac kazda konfiguracja
#ifndef USBD_MAX_NUM_INTERFACES
    #define USBD_MAX_NUM_INTERFACES             1
#else
    #if(USBD_MAX_NUM_INTERFACES > 256)
        #undef USBD_MAX_NUM_INTERFACES
        #define USBD_MAX_NUM_INTERFACES         256
    #endif
#endif

// liczba Endpointow jakie posiada USBD
#ifndef USBD_MAX_NUM_ENDPOINTS
    #define USBD_MAX_NUM_ENDPOINTS              1
#else
    #if(USBD_MAX_NUM_ENDPOINTS > 16)
        #undef USBD_MAX_NUM_ENDPOINTS
        #define USBD_MAX_NUM_ENDPOINTS          16
    #endif
#endif

// liczba stringow, jakie moga byc dolaczone funkcja USBD_install_string
#ifndef USBD_MAX_NUM_STRINGS
    #define USBD_MAX_NUM_STRINGS                0
#else
    #if(USBD_MAX_NUM_STRINGS > 256)
        #undef USBD_MAX_NUM_STRINGS
        #define USBD_MAX_NUM_STRINGS            256
    #endif
#endif

// maksymalna liczba eventow jakie moze posiadac konfiguracja
#ifndef USBD_MAX_NUM_EVENTS
    #define USBD_MAX_NUM_EVENTS                 0
#else
    #if(USBD_MAX_NUM_EVENTS > 256)
        #undef USBD_MAX_NUM_EVENTS
        #define USBD_MAX_NUM_EVENTS             256
    #endif
#endif



#ifndef USBDEP_0_MAX_POSSIBLE_PACKET_SIZE
#define USBDEP_0_MAX_POSSIBLE_PACKET_SIZE       8
#endif

#ifndef USBDEP_1_MAX_POSSIBLE_PACKET_SIZE
#define USBDEP_1_MAX_POSSIBLE_PACKET_SIZE       0
#endif

#ifndef USBDEP_2_MAX_POSSIBLE_PACKET_SIZE
#define USBDEP_2_MAX_POSSIBLE_PACKET_SIZE       0
#endif

#ifndef USBDEP_3_MAX_POSSIBLE_PACKET_SIZE
#define USBDEP_3_MAX_POSSIBLE_PACKET_SIZE       0
#endif

#ifndef USBDEP_4_MAX_POSSIBLE_PACKET_SIZE
#define USBDEP_4_MAX_POSSIBLE_PACKET_SIZE       0
#endif

#ifndef USBDEP_5_MAX_POSSIBLE_PACKET_SIZE
#define USBDEP_5_MAX_POSSIBLE_PACKET_SIZE       0
#endif

#ifndef USBDEP_6_MAX_POSSIBLE_PACKET_SIZE
#define USBDEP_6_MAX_POSSIBLE_PACKET_SIZE       0
#endif

#ifndef USBDEP_7_MAX_POSSIBLE_PACKET_SIZE
#define USBDEP_7_MAX_POSSIBLE_PACKET_SIZE       0
#endif

#ifndef USBDEP_8_MAX_POSSIBLE_PACKET_SIZE
#define USBDEP_8_MAX_POSSIBLE_PACKET_SIZE       0
#endif

#ifndef USBDEP_9_MAX_POSSIBLE_PACKET_SIZE
#define USBDEP_9_MAX_POSSIBLE_PACKET_SIZE       0
#endif

#ifndef USBDEP_10_MAX_POSSIBLE_PACKET_SIZE
#define USBDEP_10_MAX_POSSIBLE_PACKET_SIZE      0
#endif

#ifndef USBDEP_11_MAX_POSSIBLE_PACKET_SIZE
#define USBDEP_11_MAX_POSSIBLE_PACKET_SIZE      0
#endif

#ifndef USBDEP_12_MAX_POSSIBLE_PACKET_SIZE
#define USBDEP_12_MAX_POSSIBLE_PACKET_SIZE      0
#endif

#ifndef USBDEP_13_MAX_POSSIBLE_PACKET_SIZE
#define USBDEP_13_MAX_POSSIBLE_PACKET_SIZE      0
#endif

#ifndef USBDEP_14_MAX_POSSIBLE_PACKET_SIZE
#define USBDEP_14_MAX_POSSIBLE_PACKET_SIZE      0
#endif

#ifndef USBDEP_15_MAX_POSSIBLE_PACKET_SIZE
#define USBDEP_15_MAX_POSSIBLE_PACKET_SIZE      0
#endif

#ifndef USBDEP_SUMARIZED_BUFFERS_MEMORY_SIZE
#define USBDEP_SUMARIZED_BUFFERS_MEMORY_SIZE    (USBDEP_0_MAX_POSSIBLE_PACKET_SIZE + \
			USBDEP_1_MAX_POSSIBLE_PACKET_SIZE + \
			USBDEP_2_MAX_POSSIBLE_PACKET_SIZE + \
			USBDEP_3_MAX_POSSIBLE_PACKET_SIZE + \
			USBDEP_4_MAX_POSSIBLE_PACKET_SIZE + \
			USBDEP_5_MAX_POSSIBLE_PACKET_SIZE + \
			USBDEP_6_MAX_POSSIBLE_PACKET_SIZE + \
			USBDEP_7_MAX_POSSIBLE_PACKET_SIZE + \
			USBDEP_8_MAX_POSSIBLE_PACKET_SIZE + \
			USBDEP_9_MAX_POSSIBLE_PACKET_SIZE + \
			USBDEP_10_MAX_POSSIBLE_PACKET_SIZE + \
			USBDEP_11_MAX_POSSIBLE_PACKET_SIZE + \
			USBDEP_12_MAX_POSSIBLE_PACKET_SIZE + \
			USBDEP_13_MAX_POSSIBLE_PACKET_SIZE + \
			USBDEP_14_MAX_POSSIBLE_PACKET_SIZE + \
			USBDEP_15_MAX_POSSIBLE_PACKET_SIZE)
#endif


#define x_USBDEP_MAX_POSSIBLE_PACKET_SIZE(a)    ( USBDEP_ ## a ## _MAX_POSSIBLE_PACKET_SIZE )
///					UWAGA!!!
///	ponizsze makro nie moze miec w parametrze zmiennej, tylko konkretna wartosc !!!
///	w przypadku gdy chcemy uzyskac maksymalny rozmiar Endpoint-u indeksujac go
/// np inna zmienna	nalezy uzyc funkcji USBDEP_get_max_possible_packet_size
#define _USBDEP_MAX_POSSIBLE_PACKET_SIZE(a)     x_USBDEP_MAX_POSSIBLE_PACKET_SIZE(a)



#ifndef USBD_ENTER_IRQ
#define USBD_ENTER_IRQ(IRQ_CHANELL)
#endif

#ifndef USBD_EXIT_IRQ
#define USBD_EXIT_IRQ(IRQ_CHANELL)
#endif

#ifndef USBD_ENTER_CRITICAL
#define USBD_ENTER_CRITICAL()
#endif

#ifndef USBD_EXIT_CRITICAL
#define USBD_EXIT_CRITICAL()
#endif




/* *****************************************************************************
			typy enumeracyjne opisujace niektore pola uzywanych struktur
   ***************************************************************************** */

// STANDARD REQUESTS (pole bRequest)
typedef enum {
	USBD_BREQUEST_GET_STATUS            = 0,
	USBD_BREQUEST_CLEAR_FEATURE         = 1,
//  RESERVED
	USBD_BREQUEST_SET_FEATURE           = 3,
//  RESERVED
	USBD_BREQUEST_SET_ADDRESS           = 5,
	USBD_BREQUEST_GET_DESCRIPTOR        = 6,
	USBD_BREQUEST_SET_DESCRIPTOR        = 7,
	USBD_BREQUEST_GET_CONFIGURATION     = 8,
	USBD_BREQUEST_SET_CONFIGURATION     = 9,
	USBD_BREQUEST_GET_INTERFACE         = 10,
	USBD_BREQUEST_SET_INTERFACE         = 11,
	USBD_BREQUEST_SYNCH_FRAME           = 12
} USBDR_bRequest_standard_types;



// bmRequestType dla STANDARD REQUESTS
// rodzaje pola dla standardowych zadan
typedef enum {
	USBD_BMREQUESTTYPE_ZERO                 = 0x00,
	USBD_BMREQUESTTYPE_INTERFACE            = 0x01,
	USBD_BMREQUESTTYPE_ENDPOINT             = 0x02,

	USBD_BMREQUESTTYPE_IN                   = 0x80,
	USBD_BMREQUESTTYPE_OUT                  = 0x00,



	USBD_BMREQUESTTYPE_SET_CLEAR_ZERO       = (USBD_BMREQUESTTYPE_OUT | USBD_BMREQUESTTYPE_ZERO),
	USBD_BMREQUESTTYPE_SET_CLEAR_INTERFACE  = (USBD_BMREQUESTTYPE_OUT | USBD_BMREQUESTTYPE_INTERFACE),
	USBD_BMREQUESTTYPE_SET_CLEAR_ENDPOINT   = (USBD_BMREQUESTTYPE_OUT | USBD_BMREQUESTTYPE_ENDPOINT),

	USBD_BMREQUESTTYPE_GET_SYNCH_ZERO       = (USBD_BMREQUESTTYPE_IN  | USBD_BMREQUESTTYPE_ZERO),
	USBD_BMREQUESTTYPE_GET_SYNCH_INTERFACE  = (USBD_BMREQUESTTYPE_IN  | USBD_BMREQUESTTYPE_INTERFACE),
	USBD_BMREQUESTTYPE_GET_SYNCH_ENDPOINT   = (USBD_BMREQUESTTYPE_IN  | USBD_BMREQUESTTYPE_ENDPOINT),



	USBD_BMREQUESTTYPE_DESTINATION          = 0x03,
	USBD_BMREQUESTTYPE_DIRECTION            = 0x80,
	USBD_BMREQUESTTYPE_TYPE                 = 0x60,

	USBD_BMREQUESTTYPE_CLASS                = 0x20,
	USBD_BMREQUESTTYPE_VENDOR               = 0x40
} USBDR_bmRequestType_standard_types;

/* *****************************************************************************
			REQUESTS ((bmRequestType << 8) | bmRequest)
   ***************************************************************************** */
typedef enum {
	USBD_CLEAR_FEATURE_ZERO         = (USBD_BMREQUESTTYPE_SET_CLEAR_ZERO       | (USBD_BREQUEST_CLEAR_FEATURE << 8)),
	USBD_CLEAR_FEATURE_INTERFACE    = (USBD_BMREQUESTTYPE_SET_CLEAR_INTERFACE  | (USBD_BREQUEST_CLEAR_FEATURE << 8)),
	USBD_CLEAR_FEATURE_ENDPOINT     = (USBD_BMREQUESTTYPE_SET_CLEAR_ENDPOINT   | (USBD_BREQUEST_CLEAR_FEATURE << 8)),

	USBD_SET_FEATURE_ZERO           = (USBD_BMREQUESTTYPE_SET_CLEAR_ZERO       | (USBD_BREQUEST_SET_FEATURE << 8)),
	USBD_SET_FEATURE_INTERFACE      = (USBD_BMREQUESTTYPE_SET_CLEAR_INTERFACE  | (USBD_BREQUEST_SET_FEATURE << 8)),
	USBD_SET_FEATURE_ENDPOINT       = (USBD_BMREQUESTTYPE_SET_CLEAR_ENDPOINT   | (USBD_BREQUEST_SET_FEATURE << 8)),

	USBD_GET_STATUS_ZERO            = (USBD_BMREQUESTTYPE_GET_SYNCH_ZERO       | (USBD_BREQUEST_GET_STATUS << 8)),
	USBD_GET_STATUS_INTERFACE		= (USBD_BMREQUESTTYPE_GET_SYNCH_INTERFACE  | (USBD_BREQUEST_GET_STATUS << 8)),
	USBD_GET_STATUS_ENDPOINT        = (USBD_BMREQUESTTYPE_GET_SYNCH_ENDPOINT   | (USBD_BREQUEST_GET_STATUS << 8)),

	USBD_GET_CONFIGURATION_ZERO     = (USBD_BMREQUESTTYPE_GET_SYNCH_ZERO       | (USBD_BREQUEST_GET_CONFIGURATION << 8)),

	USBD_SET_CONFIGURATION_ZERO     = (USBD_BMREQUESTTYPE_SET_CLEAR_ZERO       | (USBD_BREQUEST_SET_CONFIGURATION << 8)),

	USBD_GET_DESCRIPTOR_ZERO        = (USBD_BMREQUESTTYPE_GET_SYNCH_ZERO       | (USBD_BREQUEST_GET_DESCRIPTOR << 8)),

	USBD_SET_DESCRIPTOR_ZERO        = (USBD_BMREQUESTTYPE_SET_CLEAR_ZERO       | (USBD_BREQUEST_SET_DESCRIPTOR << 8)),

	USBD_GET_INTERFACE_INTERFACE    = (USBD_BMREQUESTTYPE_GET_SYNCH_INTERFACE  | (USBD_BREQUEST_GET_INTERFACE << 8)),

	USBD_SET_INTERFACE_INTERFACE    = (USBD_BMREQUESTTYPE_SET_CLEAR_INTERFACE  | (USBD_BREQUEST_SET_INTERFACE << 8)),

	USBD_SET_ADDRESS_ZERO           = (USBD_BMREQUESTTYPE_SET_CLEAR_ZERO       | (USBD_BREQUEST_SET_ADDRESS << 8)),

	USBD_SYNCH_FRAME_ENDPOINT       = (USBD_BMREQUESTTYPE_GET_SYNCH_ENDPOINT   | (USBD_BREQUEST_SYNCH_FRAME << 8))
} USBDR_standard_types;





typedef enum {
	USBDEP_FS_CONTROL_MAX_POSSIBLE_PACKET_SIZE      = 64,
	USBDEP_FS_INTERRUPT_MAX_POSSIBLE_PACKET_SIZE    = 64,
	USBDEP_FS_BULK_MAX_POSSIBLE_PACKET_SIZE         = 64,
	USBDEP_FS_ISOCHRONOUS_MAX_POSSIBLE_PACKET_SIZE  = 1023,

	USBDEP_HS_CONTROL_MAX_POSSIBLE_PACKET_SIZE      = 64,
	USBDEP_HS_INTERRUPT_MAX_POSSIBLE_PACKET_SIZE    = 1024,
	USBDEP_HS_BULK_MAX_POSSIBLE_PACKET_SIZE         = 512,
	USBDEP_HS_ISOCHRONOUS_MAX_POSSIBLE_PACKET_SIZE  = 1024
}USBDEP_standard_types_max_packet_size;



typedef enum {
	USBDEP_TYPE_CONTROL_IN          = (0x01 << 0),
	USBDEP_TYPE_CONTROL_OUT         = (0x01 << 1),
	USBDEP_TYPE_CONTROL             = (USBDEP_TYPE_CONTROL_IN | USBDEP_TYPE_CONTROL_OUT),
	USBDEP_TYPE_INTERRUPT_IN        = (0x01 << 2),
	USBDEP_TYPE_INTERRUPT_OUT       = (0x01 << 3),
	USBDEP_TYPE_INTERRUPT           = (USBDEP_TYPE_INTERRUPT_IN | USBDEP_TYPE_INTERRUPT_OUT),
	USBDEP_TYPE_BULK_IN             = (0x01 << 4),
	USBDEP_TYPE_BULK_OUT            = (0x01 << 5),
	USBDEP_TYPE_BULK                = (USBDEP_TYPE_BULK_IN | USBDEP_TYPE_BULK_OUT),
	USBDEP_TYPE_ISOCHRONOUS_IN      = (0x01 << 6),
	USBDEP_TYPE_ISOCHRONOUS_OUT     = (0x01 << 7),
	USBDEP_TYPE_ISOCHRONOUS         = (USBDEP_TYPE_ISOCHRONOUS_IN | USBDEP_TYPE_ISOCHRONOUS_OUT)
}USBDEP_types;



typedef enum {
// transfer_type moze przybierac nastepujace wartosci:
	USBD_IDLE                       = 0x00,
	USBD_CONTROL                    = 0x02,
	USBD_DATA                       = 0x04,

	USBD_TRANSFER_DATA_CORRUPTION	= 0x40,
// it works for DATA OUT only !!!
	USBD_TRANSFER_IS_LOCKED         = 0x20,

	USBD_TRANSFER_PRESENT           = 0x10,

	USBD_TRANSFER_DIRECTION_IN      = 0x00,
	USBD_TRANSFER_DIRECTION_OUT     = 0x01,

	USBD_CONTROL_IN                 = (USBD_CONTROL | USBD_TRANSFER_DIRECTION_IN),
	USBD_CONTROL_OUT                = (USBD_CONTROL | USBD_TRANSFER_DIRECTION_OUT),

	USBD_DATA_IN                    = (USBD_DATA | USBD_TRANSFER_DIRECTION_IN),
	USBD_DATA_OUT                   = (USBD_DATA | USBD_TRANSFER_DIRECTION_OUT)
}USBDEP_transfer_type;



// transfer_state moze przybierac nastepujace wartosci:
typedef enum {
	USBD_IDLE_OR_BACKGROUND = 0,
	USBD_PACKET_ERROR,
	USBD_PACKET_LOSS,
	USBD_TOO_LESS_SPACE,
	USBD_TOO_LESS_SPACE_FOR_NEXT_PACKET,
	USBD_END_OF_TRANSFER,
	USBD_TRANSFER_ABORTED,
	USBD_CLEAR_ALL_FLAGS,
	USBD_STATUS_STAGE
} USBDEP_transfer_state;



// definicje dla pola USBD_status struktury USBDC_params
// objasnienie USBDST - USB Device status
typedef enum {
	USBDST_ATTACHED      = (1 << 7),
	USBDST_POWERED       = (1 << 6),
	USBDST_DEFAULT       = (1 << 5),
	USBDST_ADDRESSED     = (1 << 4),
	USBDST_CONFIGURED    = (1 << 3),
	USBDST_SUSPENDED     = (1 << 2)
}USBDST_types;



// definicje dla pola USBD_flags struktury USBDC_params
// objasnienie USBDEF - USB Device events flag
typedef enum {
	USBDEF_NO_FLAG_SET = 0,
	USBDEF_ATTACHED,
	USBDEF_DETACHED,
	USBDEF_POWERED,
	USBDEF_UNPOWERED,
	USBDEF_RESET,
	USBDEF_ADDRESSED,
	USBDEF_CONFIGURED,
	USBDEF_UNCONFIGURED,
	USBDEF_SUSPENDED,
	USBDEF_WAKEUP,
	USBDEF_SOF
} USBDEF_types;



// definicje okreslajace pozycje pol bitowych w zmiennej active_events
// objasnienie USBDEM - USB Device event mask
typedef enum {
	USBDEM_ATTACHED     = (1 << USBDEF_ATTACHED),
	USBDEM_DETACHED     = (1 << USBDEF_DETACHED),
	USBDEM_POWERED      = (1 << USBDEF_POWERED),
	USBDEM_UNPOWERED    = (1 << USBDEF_UNPOWERED),
	USBDEM_RESET        = (1 << USBDEF_RESET),
	USBDEM_ADDRESSED    = (1 << USBDEF_ADDRESSED),
	USBDEM_CONFIGURED   = (1 << USBDEF_CONFIGURED),
	USBDEM_UNCONFIGURED = (1 << USBDEF_UNCONFIGURED),
	USBDEM_SUSPENDED    = (1 << USBDEF_SUSPENDED),
	USBDEM_WAKEUP       = (1 << USBDEF_WAKEUP),
	USBDEM_SOF          = (1 << USBDEF_SOF),
	
	USBDEM_ALL          = (USBDEM_ATTACHED | USBDEM_DETACHED
							| USBDEM_POWERED | USBDEM_UNPOWERED
							| USBDEM_RESET | USBDEM_ADDRESSED
							| USBDEM_CONFIGURED | USBDEM_UNCONFIGURED
							| USBDEM_SUSPENDED | USBDEM_WAKEUP
							| USBDEM_SOF)
} USBDEM_types;






/* *****************************************************************************
			struktury uzywane w USBD API
   ***************************************************************************** */


/**
 * USBDR_params - USB device request parameters
 * 
 * This structure represents composition of USB requests.
 * Structure fields names are not standarized (small characters)
 * because USBDR_params structure represents
 * standard USB Request packet. In this reason names are that same
 * like in USB specification
 */
typedef struct {
	union {
		uint16_t WholeRequestValue;
		struct {
			uint8_t bmRequestType;
			uint8_t bRequest;
		}RequestFields;
	}Request;
	uint16_t wValue;
	uint16_t wIndex;
	uint16_t wLength;
} USBDR_params;




/**
 * USBDEP_params - USB device endpoint parameters
 * 
 * This structure contain all necessary parameters of endpoint
 */
typedef struct {
	// zawiera maksymalny rozmiar danych jakie moze odebrac/wyslac
	// Endpoint
	uint16_t max_packet_size;

	// zawiera status Endpointa a dokladniej jeden wazny bit - flage
	// Halt
	uint16_t status;

	// wkazuje address w pamięci deskryptora
	// opsujacego dany Endpoint - sluzy do ponownego
	// ustawienia parametrow Endpointa po odebraniu zadania
	// CLEAR_FEATURE_ENDPOINT kasujacego flage Halt kiedy to na
	// podstawie deskryptora konfigurowany jest rejestr danego
	// Endpointa
	USBD_endpoint_descriptor *endpoint_descriptor;

	// zawiera numer danego Endpointa
	uint8_t number;

	// zawiera numer interfejsu, ktory uzywa danego Endpointa
	uint8_t interface;
	
	struct {

		// zawiera dlugosc danych, ktore sa aktualnie transmitowane
		// wartosc ta zawiera sie w wartosci data_left_length.
		// W przypadku odbierania danych wartosc ta jest zawsze rowna zero
		uint16_t data_buffered_length;

		// zawiera dlugosc danych do wyslania lub
		// miejsce pozostale w pamieci na dane odbierane dla endpointa
		uint32_t data_left_length;

		// zawiera dlugosc danych ktore zostaly juz wyslane
		// lub dlugosc danych jakie zostaly w sumie odebrane i zapisane
		// pod dana przestrzenia
		uint32_t data_transferred_length;

		// zawiera adres, pod ktorym znajduja sie dane do wyslania
		// lub miejsce na dane do odebrania dla endpointa
		uint8_t *data;

		// zawiera informacje o tym jaki typ transmisji jest prowadzony
		// i czy transmisja jest prowadzona
		uint8_t transfer_type;

		// zariera informacje o tym jaki jest stan transmisji
		uint8_t transfer_state;
		
	}transfer_params;
}USBDEP_params;




/**
 * USBDI_params - USB device interface parameters
 * 
 * This structure contain all necessary parameters of each interface
 * This structure is part of USBDC_params sructure
 */
typedef struct {
	// zawiera wskaznik do deskryptora opisujacego interfejs w danej chwili
	USBD_interface_descriptor *descriptor;

#if(USBD_UPPER_LAYER_SUPPORT == USBD_FEATURE_ON)
	// przechowuje dane, ktore nie sa w zaden sposob uzywane
	// przez biblioteke USBD. Parametr ten moze byc uzywany przez
	// biblioteki odpowiedzialne za obsluge interfejsow
	// i dla ich potrzeby parametr ten istnieje.
	// W zaleznosci, ile danych interfejs potrzebuje przechowac
	// parametr ponizszy to unia - w przypadku pojedynczego parametru
	// przechowujemy go bezposrednio w upper_layer_data, w przypadku
	// wiekszej ich liczby uzywamy go jako wskaznika do struktury
	union {
		void *pvoid;
		uint32_t uint32;
		uint16_t uint16;
		uint16_t uint16_tab[2];
		uint8_t uint8;
		uint8_t uint8_tab[4];
	}upper_layer_data;
	
	// za pomoca tego wskaznika biblioteka wyzszej klasy moze sprawdzic
	// czy dany interfejs do niej nalezy
	void *owner;
#endif

#if(USBD_ALTERNATE_SETTING_SUPPORT == USBD_FEATURE_ON)
	// zawierajaca maksymalna liczbe alternatywnych ustawien
	// dla danego interfejsu
	uint8_t max_alternate_setting;
#endif
}USBDI_params;



struct _USBDC_params;

/**
 * USBDE_params - USB device event parameters
 * 
 * This structure contain all necessary parameters of each event
 * This structure is part of USBDC_params sructure
 */
struct _USBDE_params {
	// wskaznik do Eventu
	void (*event) (struct _USBDC_params *usbdc, uint8_t usbd_flags, struct _USBDE_params *event);

#if(USBD_UPPER_LAYER_SUPPORT == USBD_FEATURE_ON)
	// przechowuje dane, ktore nie sa w zaden sposob uzywane
	// przez biblioteke USBD. Parametr ten moze byc uzywany przez
	// podpiety event i dla niego istnieje.
	// W zaleznosci, ile danych Event potrzebuje przechowac
	// parametr ponizszy to unia - w przypadku pojedynczego parametru
	// przechowujemy go bezposrednio w upper_layer_data, w przypadku
	// wiekszej ich liczby uzywamy go jako wskaznika do struktury
	union {
		void *pvoid;
		uint32_t uint32;
		uint16_t uint16;
		uint16_t uint16_tab[2];
		uint8_t uint8;
		uint8_t uint8_tab[4];
	}upper_layer_data;
#endif

	// pola bitowe mowiace, ktore eventy sa aktywne
	// jezeli dany bit jest ustawiony wowczas event zostanie wywolany
	uint16_t active_events;
};
typedef struct _USBDE_params USBDE_params;



/**
 * USBDC_params - USB device configuration parameters
 * 
 * This structure contain all necessary parameters of configuration.
 * It is necessary to create USBDC_params structure before
 * run USB device. This structure contain parameters of installed
 * configuration that are requied to configure USB peripherial.
 * Includes also pointers to events handlers, endpoints data transfer
 * handlers, interfaces control transfer handlers etc. In this reason
 * this structure have to exist till USB peripheral runs.
 */
struct _USBDC_params {
	// tablica wskaznikow do funkcji obslugujacych poszczegolne
	// endpointy - jezeli dany endpoint nie jest uzywany,
	// nalezy wowczas dana komorke wypelnic nalezy zerem
	void (*ep_irq[USBD_MAX_NUM_ENDPOINTS]) (struct _USBDC_params *usbdc, USBDEP_params *ep);

	// tablica wskazników do funkcji odpowiedzialnych za rozszerzona
	// enumeracje.
	void (*ep0_setup_interface_irq[USBD_MAX_NUM_INTERFACES]) (struct _USBDC_params *usbdc, USBDEP_params *ep, uint8_t interface, USBDR_params *request);

#if(USBD_VENDOR_SUPPORT == USBD_FEATURE_ON)
	// tablica wskazników do funkcji odpowiedzialnych za rozszerzona
	// enumeracje typu vendor
	void (*ep0_setup_vendor_irq) (struct _USBDC_params *usbdc, USBDEP_params *ep, USBDR_params *request);
#endif

	// jezeli dany bit tej zmiennej jest ustawiony, wowczas USBD nie moze
	// zmienic stanu flagi HALT danego Endpoint'a - wtedy wyzsza warstwa
	// musi obsluzyc to polecenie w funkcji EP0_irq_SETUP_handler podpietej
	// do interfejsu, ktory jest wlascicielem danego endpointa
	uint16_t ep_halt_mask;

#if (USBDEP_DATA_OUT_WAIT_FOR_READY_SUPPORT == USBD_FEATURE_ON)
	// jezeli dany bit tej zmiennej jest ustawiony, wowczas USBD zezwala
	// na odbior kolejnych paczek danych dopoki wyzsza warstwa na to nie pozwoli
	uint16_t ep_data_out_wait_for_ready_mask;
#endif

#if ((USBD_ALTERNATE_SETTING_SUPPORT == USBD_FEATURE_ON) || (USBD_UPPER_LAYER_SUPPORT == USBD_FEATURE_ON))
	// tablica struktur opisujacych interfejsy danej konfiguracji
	USBDI_params interfaces[USBD_MAX_NUM_INTERFACES];
#endif

#if(USBD_UPPER_LAYER_SUPPORT == USBD_FEATURE_ON)
	// zawiera dane pomocne przy prowadzeniu transmisji kontrolnej
	// przez wyzsze warstwy
	// moze byc modyfikowane podczas transmisji kontrolnej
	// W zaleznosci, ile danych Event potrzebuje przechowac
	// parametr ponizszy to unia - w przypadku pojedynczego parametru
	// przechowujemy go bezposrednio w upper_layer_data, w przypadku
	// wiekszej ich liczby uzywamy go jako wskaznika do struktury
	union {
		void *pvoid;
		uint32_t uint32;
		uint16_t uint16;
		uint16_t uint16_tab[2];
		uint8_t uint8;
		uint8_t uint8_tab[4];
	}upper_layer_data;
#endif

	// zawiera wskaznik do tablicy w ktorej znajduje sie deskryptor
	// konfiguracji w takiej postaci, jak sie go wysyla do Hosta
	// nalezy zainicjowac wskaznikiem do deskryptora konfiguracji
	uint8_t *configuration_descriptor;

	// wskaznik do zmiennej przechowujacej status USBD
	// pole to mozna caly czas odczytywac i okresla status USBD w danej
	// chwili
	uint8_t *usbd_status;

#if(USBD_MAX_NUM_EVENTS > 0)
	// tablica przechowujaca Eventy danej konfiguracji
	USBDE_params events[USBD_MAX_NUM_EVENTS];
#endif
};
typedef struct _USBDC_params USBDC_params;



/**
 * USBDS_params - USB device string parameters
 * 
 * This structure contain all necessary parameters of each string
 * Used just in usbd.c file to store installed strings parameters
 */
typedef struct {
	// wskaznik do stringa
	uint8_t *string;

	// wskaznik do komorki odpowiedniego deskryptora gdzie przechowywany
	// jest indeks do danego stringa (iString)
	uint8_t *index_string;

	// wskaznik do konfiguracji ktora uzywa danego stringa
	USBDC_params *usbdc;
}USBDS_params;



/**
 * USBD_params - USB device parameters
 * 
 * This structure contains USB Device standard private elements
 */
typedef struct {
	// mowi, czy urzadzenie jest aktywnie polaczone z Hostem, czy nie
	// czyli czy urzadzenie zostalo przynajmniej zresetowane przez Host
	uint8_t session_activity;

	// zawiera numer aktualnie uzywanej konfiguracji
	uint8_t active_config_number;

	// licznik zainstalowanych stringow
	uint8_t installed_strings_count;

	// tablica struktur ze wskaznikami do stringow i parametrow opisujacych je
	USBDS_params strings[USBD_MAX_NUM_STRINGS + 1];

	// zawiera tablice wskaznikow do struktor opisujacych poszczegolne konfiguracje
	USBDC_params *configurations[USBD_MAX_NUM_CONFIGURATIONS + 1];

	// zawiera deskryptor urzadzenia
	USBD_device_descriptor *device_descriptor;
}USBD_params;









/* *****************************************************************************
				FUNKCJE
   ***************************************************************************** */
/**
 *  Funkcja uzywana tylko przez porty USBD. Jej zadaniem jest instalacja
 *  elementow portu w usbd.c core
 * 
 *  \param usbdc wskaznik do konfiguracji kojazonej z danym USBD
 *  \param USBD_private wskaznik do elementow przywatnych portu
 *  zarzadzanych przez usbd.c
 */
void USBD_install_port (USBD_params *usbd_private, USBDC_params *usbdc);


/**
 *  W przyszlych wersjach USBD bedzie to funkcja -
 *  tu jest to puste makro ktorego zadaniem jest jedynie utrzymanie
 *  kompatybilnosci z przyszlymi wersjami USBD
 *
 *  \param usbdc wskaznik do konfiguracji kojazonej z danym USBD
 *  \param usbd wskaznik do urzadzenia USBD zdefiniowanego w naglowku portu
 */
void USBDC_bind_to_port(void *usbd, USBDC_params *usbdc);



/**
 *  Funkcja aktywujaca USBD - nalezy uzyc po zainstalowaniu wszystkich konfiguracji
 * 
 *  \param usbdc wskaznik do konfiguracji wywolujacej ta funkcje
 */
void USBD_activate (USBDC_params *usbdc);



/**
 *  Funkcja deaktywujaca USBD
 * 
 *  \param usbdc wskaznik do konfiguracji wywolujacej ta funkcje
 */
#if(USBD_MULTI_SESSION_SUPPORT == USBD_FEATURE_ON)
void USBD_deactivate (USBDC_params *usbdc);
#endif



/**
 *  Funkcja pobierajaca numer aktualnej ramki USB
 * 
 *  \param usbdc wskaznik do konfiguracji wywolujacej ta funkcje
 */
uint16_t USBD_get_frame_number(USBDC_params *usbdc);



/**
 *  Funkcja zwracajaca wskaznik do deskryptora urzadzenia
 *  ktory mozna modyfikowac
 * 
 *  \param usbdc wskaznik do konfiguracji wywolujacej ta funkcje
 *  \return wskaznik do deskryptora urzadzenia
 */
USBD_device_descriptor *USBD_get_dev_descriptor_pointer(USBDC_params *usbdc);



/**
 *  Funkcja instalujaca konfiguracje w urzadzeniu USBD
 *  Wymagane jest zainstalowanie konfiguracji w urzadzeniu
 *  przed jakimikolwiek operacjami dostepu do niego gdyz tylko
 *  poprawnie zainstalowane konfiguracje moga komunikowac sie z USBD.
 * 
 *  \param usbdc wskaznik do konfiguracji wywolujacej ta funkcje
 *  jednoczesnie jest to wskaznik do instalowanej konfiguracji
 *  \return numer zainstalowanej konfiguracji ktory nalezy umiescic
 *  wewnatrz deskryptora konfiguracji w polu bConfigurationValue
 */
uint8_t USBDC_install (USBDC_params *usbdc);



/**
 *  Funkcja instalujaca konfiguracje zerowa w urzadzeniu USBD
 *  Konfiguracja zerowa jest rodzajem konfiguracji różniaca sie
 *  od innych tym, ze nie jest nigdy aktywna (dlatego nie zwraca
 *  numeru zainstalwoanej konfigoracji). Stosuje sie ja jedynie
 *  w celu sprawowania czegos w rodzaju pobocznej kontroli
 *  stanu pracy USBD.
 * 
 *  \param usbdc wskaznik do konfiguracji wywolujacej ta funkcje
 *  jednoczesnie jest to wskaznik do instalowanej konfiguracji zerowej
 */
#if(USBD_ZERO_CONFIG_SUPPORT == USBD_FEATURE_ON)
void USBDZC_install (USBDC_params *usbdc);
#endif



/**
 *  Funkcja odinstalowujaca konfiguracje zerowa z urzadzenia USBD
 * 
 *  \param usbdc wskaznik do konfiguracji wywolujacej ta funkcje
 */
#if(USBD_ZERO_CONFIG_SUPPORT == USBD_FEATURE_ON)
void USBDZC_uninstall (USBDC_params *usbdc);
#endif



/**
 *  Funkcja odinstalowujaca konfiguracje urzadzenia USBD
 *  UWAGA !!!
 *  ta funkcja modyfikuje numer konfiguracji w deskryptorze konfiguracji
 *  daltego jezeli uzywamy jej wowczas deskryptory konfiguracji
 *  wszystkich zainstalowanych konfiguracji musza znajdowac sie w pamieci RAM!!!
 * 
 *  \param usbdc wskaznik do konfiguracji wywolujacej ta funkcje
 */
#if(USBD_MULTI_SESSION_SUPPORT == USBD_FEATURE_ON)
void USBDC_uninstall (USBDC_params *usbdc);
#endif



/**
 *  Funkcja odinstalowujaca wszystkie konfiguracje urzadzenia USBD
 * 
 *  \param usbdc wskaznik do konfiguracji wywolujacej ta funkcje
 */
#if(USBD_MULTI_SESSION_SUPPORT == USBD_FEATURE_ON)
void USBDC_uninstall_all (USBDC_params *usbdc);
#endif



/**
 *  Funkcja inicjalizujaca strukture typu USBDC_params
 * 
 *  \param usbdc wskaznik do konfiguracji wywolujacej ta funkcje
 *  a za razem wskaznik do struktury inicjalizowanej
 */
void USBDC_init (USBDC_params *usbdc);



/**
 *  Funkcja zwracajaca numer aktywnej (obecnie ustawionej) konfiguracji
 * 
 *  \param usbdc wskaznik do konfiguracji wywolujacej ta funkcje
 *  \return numer aktywnej konfiguracji
 */
uint8_t USBDC_get_active_number (USBDC_params *usbdc);



/**
 *  Funkcja odinstalowujaca wszystkie stringi nalezace do danej konfiguracji
 * 
 *  \param usbdc wskaznik do konfiguracji wywolujacej ta funkcje
 */
#if((USBD_MULTI_SESSION_SUPPORT == USBD_FEATURE_ON) && (USBD_MAX_NUM_STRINGS > 0))
void USBDC_uninstall_strings (USBDC_params *usbdc);
#endif



/**
 *  Funkcja instalujaca stringi w urzadzeniu USBD
 *  w ktorym zainstalowana jest juz konfiguracja usbdc
 * 
 *  \param string wskaznik do instalowanego stringa
 *  \param index_string wskaznik do komorki w (jakims) deskryptorze
 *  ktora deklaruje instalwoany wlasnie string
 *  \param usbdc wskaznik do konfiguracji wywolujacej ta funkcje
 */
#if(USBD_MAX_NUM_STRINGS > 0)
void USBDS_install (uint8_t *string, uint8_t *index_string, USBDC_params *usbdc);
#endif



/**
 *  Funkcja odinstalujaca string z urzadzenia USBD
 *  w ktorym zainstalowana jest juz konfiguracja usbdc
 * 
 *  \param string wskaznik do instalowanego stringa
 *  \param index_string wskaznik do komorki w (jakims) deskryptorze
 *  ktora deklaruje instalwoany wlasnie string
 *  \param usbdc wskaznik do konfiguracji wywolujacej ta funkcje
 */
#if((USBD_MULTI_SESSION_SUPPORT == USBD_FEATURE_ON) && (USBD_MAX_NUM_STRINGS > 0))
void USBDS_uninstall (uint8_t *string, uint8_t *index_string, USBDC_params *usbdc);
#endif



/**
 *  Funkcja odinstalujaca wszystkie stringi
 *  (poza Language_String_Descriptor) z urzadzenia USBD
 *  w ktorym zainstalowana jest juz konfiguracja usbdc
 * 
 *  \param usbdc wskaznik do konfiguracji wywolujacej ta funkcje
 */
#if((USBD_MULTI_SESSION_SUPPORT == USBD_FEATURE_ON) && (USBD_MAX_NUM_STRINGS > 0))
void USBDS_uninstall_all (USBDC_params *usbdc);
#endif



/**
 *  Funkcja instaluje nowy Language_String_Descriptor
 *  w urzadzeniu USBD w ktorym zainstalowana jest juz
 *  konfiguracja usbdc
 * 
 *  \param new_language_string wskaznik do nowego stringa jezyka
 *  \param usbdc wskaznik do konfiguracji wywolujacej ta funkcje
 */
void USBDS_change_Language_string(uint8_t *new_language_string, USBDC_params *usbdc);



/**
 *  Funkcja instaluje event w konfiguracji usbdc
 * 
 *  \param event wskaznik do funkcji eventu
 *  \param active_events pole bitowe wskazujace, w przypadku
 *  ktorych zdarzen USBD dany event ma byc wywolany
 *  \param usbdc wskaznik do konfiguracji wywolujacej ta funkcje
 *  \return numer ID zainstalowanego eventu
 */
#if(USBD_MAX_NUM_EVENTS > 0)
int32_t USBDE_install (void (*event)(USBDC_params *, uint8_t, USBDE_params *), uint16_t active_events, USBDC_params *usbdc);
#endif



/**
 *  Funkcja pobiera wskaznik do struktury z parametrami
 *  eventu o zadanym ID zainstalowanym w konfiguracji usbdc
 * 
 *  \param event_id numer ID uzywany w celu zidentyfikowania eventu
 *  \param usbdc wskaznik do konfiguracji wywolujacej ta funkcje
 *  \return wskaznik do struktury z parametrami eventu
 */
#if(USBD_MAX_NUM_EVENTS > 0)
USBDE_params *USBDE_get_params (int32_t event_id, USBDC_params *usbdc);
#endif



/**
 *  Funkcja odinstalowuje event o zadanym ID z konfiguracji usbdc
 * 
 *  \param event_id numer ID uzywany w celu zidentyfikowania eventu
 *  \param usbdc wskaznik do konfiguracji wywolujacej ta funkcje
 */
#if((USBD_MULTI_SESSION_SUPPORT == USBD_FEATURE_ON) && (USBD_MAX_NUM_EVENTS > 0))
void USBDE_uninstall (int32_t event_id, USBDC_params *usbdc);
#endif



/**
 *  Funkcja odinstalowuje wszystkie eventy z konfiguracji usbdc
 * 
 *  \param usbdc wskaznik do konfiguracji wywolujacej ta funkcje
 */
#if((USBD_MULTI_SESSION_SUPPORT == USBD_FEATURE_ON) && (USBD_MAX_NUM_EVENTS > 0))
void USBDE_uninstall_all (USBDC_params *usbdc);
#endif



/**
 *  Funkcja zwraca wskaznik do struktury typu USBDEP_params
 *  opisujacej dany Endpoint
 *
 *  \param ep_number numer Endpointa ktorego parametry
 *  chcemy uzyskac
 *  \param usbdc wskaznik do konfiguracji wywolujacej ta funkcje
 *  \return wskaznik do parametrow danego Endpointa
 */
USBDEP_params* USBDEP_get_params (uint8_t ep_number, USBDC_params *usbdc);



/**
 *  Funkcja zwraca maksymalny mozliwy rozmiar bufora
 *  punktu koncowego danego Endpointa
 * 
 *  \param ep_number numer Endpointa ktorego parametry
 *  chcemy uzyskac
 *  \param usbdc wskaznik do konfiguracji wywolujacej ta funkcje
 *  \return zwraca maksymalny mozliwy rozmiar bufora
 *  punktu koncowego danego Endpointa
 */
uint16_t USBDEP_get_max_possible_packet_size (uint8_t ep_number, USBDC_params *usbdc);



/**
 *  Funkcja zezwala na odbior kolejnej paczki danych podczas transmisji DATA OUT
 *  jezeli bit odpowiadajacy numerowi danego punktu koncowego w zmiennej
 *  ep_data_out_wait_for_ready_mask struktury konfiguracyjnej jest ustawiony
 * 
 *  \param ep_number numer Endpointa na ktorym odblokowujemy transmisje
 *  \param usbdc wskaznik do konfiguracji wywolujacej ta funkcje
 */
void USBDEP_data_out_ready(uint8_t ep_number, USBDC_params *usbdc);



/**
 *  Funkcja wysylajaca pakiet danych o zerowej dlugosci przez wskazany punkt koncowy
 * 
 *  \param handler - handler do funkcji ktora ma zostac wywolana
 *  po wyslaniu pakietu zerowego
 *  \param ep_number numer Endpointa przez ktory wysylamy pakiet zerowy 
 *  \param usbdc wskaznik do konfiguracji wywolujacej ta funkcje
 */
void USBD_send_status(void (*handler)(USBDC_params*, USBDEP_params *), uint8_t ep_number, USBDC_params *usbdc);



/**
 *  Funkcja wysylajaca potwierdzenie STALL przez zadany punkt koncowy
 * 
 *  \param ep_number numer Endpointa przez ktory wysylamy pakiet STALL 
 *  \param usbdc wskaznik do konfiguracji wywolujacej ta funkcje
 */
void USBD_send_stall(uint8_t ep_number, USBDC_params *usbdc);



/**
 *  Funkcja wysylajaca sygnal RESUME do Hosta
 * 
 *  \param usbdc wskaznik do konfiguracji wywolujacej ta funkcje
 */
void USBD_send_resume(USBDC_params *usbdc);



/**
 *  Funkcja wysylajaca dane liniowe o zadanej dlugosci.
 *  Transfer dojdzie do skutku, jezeli konfiguracja, ktora je wysyla
 *  (usbdc) jest w danym momencie aktywna
 * 
 *  \param data wskaznik do danych do wyslania
 *  \param length dlugosc danych do wyslania
 *  \param ep_number numer Endpointa przez ktory wysylamy dane
 *  \param usbdc wskaznik do konfiguracji wywolujacej ta funkcje
 */
void USBD_send_data_in (uint8_t* data, uint32_t length, uint8_t ep_number, USBDC_params *usbdc);



/**
 *  Funkcja odbierajaca dane liniowe o zadanej dlugosci.
 *  Transfer dojdzie do skutku, jezeli konfiguracja, ktora je wysyla
 *  (usbdc) jest w danym momencie aktywna
 * 
 *  \param data wskaznik do przestrzeni w pamieci na odbierane dane
 *  \param length dlugosc danych do odebrania
 *  \param ep_number numer Endpointa przez ktory odbieramy dane
 *  \param usbdc wskaznik do konfiguracji wywolujacej ta funkcje
 */
void USBD_recv_data_out (uint8_t* data, uint32_t length, uint8_t ep_number, USBDC_params *usbdc);



/**
 *  Funkcja ustawiajaca flage HALT danego Endpointu
 *  a co za tym idzie blokujaca go calkowicie
 * 
 *  \param ep_number numer Endpointa ktory blokujemy
 *  \param usbdc wskaznik do konfiguracji wywolujacej ta funkcje
 */
void USBD_halt (uint8_t ep_number, USBDC_params *usbdc);



/**
 *  Funkcja kasujaca flage HALT danego Endpointu
 *  a co za tym idzie odblokowujaca i reinicjalizujaca go
 * 
 *  \param ep_number numer Endpointa ktory odblokowujemy
 *  \param usbdc wskaznik do konfiguracji wywolujacej ta funkcje
 */
void USBD_unhalt (uint8_t ep_number, USBDC_params *usbdc);


#if(USBD_CCM_SUPPORT == USBD_FEATURE_ON)
#ifndef USBD_CCM_H_
#include "usbd_ccm.h"
#endif
#endif

#endif
