/*
================================================================================
 File: usbd_port.c
--------------------------------------------------------------------------------
 Author:        Piotr Wojtowicz
 Purpose:       USBDevice virtual layer
 Compiler:      ANSI C
 Hardware:      
 Doc:
--------------------------------------------------------------------------------
 Includes:      
--------------------------------------------------------------------------------
 Comments:                      UWAGA!!!
                Oprogramowanie to udostępnione jest do celów poglądowych,
                nie jest przetestowane pod kątem bezpieczeństwa ani spełnienia
                jakichkolwiek norm wymaganych aby urządzenie używające go
                mogło być dopuszczone do użytku, dlatego używasz go
                na własną odpowiedzialność a jego autor nie ponosi żadnego
                typu odpowiedzialności za jakiekolwiek skutki jego użycia.
                Jeżeli nie zgadzasz się z powyższym warunkiem proszę
                o nieużywanie tego oprogramowania.
================================================================================
*/

#ifndef USBD_H_
#include "usbd.h"
#endif

#ifndef USBD_PORT_TRASH_H_
#include "usbd_port_trash.h"
#endif

#ifndef USBD_STM32_HARDWARE_H_
#include "usbd_stm32_hardware.h"
#endif

#include <string.h>





typedef struct {
	uint8_t num_fifos;
	uint8_t fifo_counter;
	uint16_t fifo_data_size[2];
	uint32_t *fifo_OUT_data_position;
	uint8_t partial_read_additional_byte;
	uint8_t partial_read;
	uint8_t EP_type;
}USBDEP_private_params;










static USBDEP_params USBD_EP_params[USBD_MAX_NUM_ENDPOINTS];

static USBDEP_private_params USBDEP_private[USBD_MAX_NUM_ENDPOINTS];

static void (*USBDEP_hc_irq[USBD_MAX_NUM_ENDPOINTS])(uint8_t ep_number);

static uint8_t USBDC_active_number;

static USBD_params USBD_private_data;




static uint8_t USBD_first_installation_processed = 0;







/* *****************************************************************************
		tablice przechowujace parametry potrzebne do transmisji
   ***************************************************************************** */
static const uint16_t USBDEP_max_possible_packet_size[USBD_MAX_NUM_ENDPOINTS] = {
	USBDEP_0_MAX_POSSIBLE_PACKET_SIZE
#if (USBD_MAX_NUM_ENDPOINTS > 1)
	,
	USBDEP_1_MAX_POSSIBLE_PACKET_SIZE
#endif
#if (USBD_MAX_NUM_ENDPOINTS > 2)
	,
	USBDEP_2_MAX_POSSIBLE_PACKET_SIZE
#endif
#if (USBD_MAX_NUM_ENDPOINTS > 3)
	,
	USBDEP_3_MAX_POSSIBLE_PACKET_SIZE
#endif
#if (USBD_MAX_NUM_ENDPOINTS > 4)
	,
	USBDEP_4_MAX_POSSIBLE_PACKET_SIZE
#endif
#if (USBD_MAX_NUM_ENDPOINTS > 5)
	,
	USBDEP_5_MAX_POSSIBLE_PACKET_SIZE
#endif
#if (USBD_MAX_NUM_ENDPOINTS > 6)
	,
	USBDEP_6_MAX_POSSIBLE_PACKET_SIZE
#endif
#if (USBD_MAX_NUM_ENDPOINTS > 7)
	,
	USBDEP_7_MAX_POSSIBLE_PACKET_SIZE
#endif
#if (USBD_MAX_NUM_ENDPOINTS > 8)
	,
	USBDEP_8_MAX_POSSIBLE_PACKET_SIZE
#endif
#if (USBD_MAX_NUM_ENDPOINTS > 9)
	,
	USBDEP_9_MAX_POSSIBLE_PACKET_SIZE
#endif
#if (USBD_MAX_NUM_ENDPOINTS > 10)
	,
	USBDEP_10_MAX_POSSIBLE_PACKET_SIZE
#endif
#if (USBD_MAX_NUM_ENDPOINTS > 11)
	,
	USBDEP_11_MAX_POSSIBLE_PACKET_SIZE
#endif
#if (USBD_MAX_NUM_ENDPOINTS > 12)
	,
	USBDEP_12_MAX_POSSIBLE_PACKET_SIZE
#endif
#if (USBD_MAX_NUM_ENDPOINTS > 13)
	,
	USBDEP_13_MAX_POSSIBLE_PACKET_SIZE
#endif
#if (USBD_MAX_NUM_ENDPOINTS > 14)
	,
	USBDEP_14_MAX_POSSIBLE_PACKET_SIZE
#endif
#if (USBD_MAX_NUM_ENDPOINTS > 15)
	,
	USBDEP_15_MAX_POSSIBLE_PACKET_SIZE
#endif
};





static const uint8_t USBDEP_supported_types[USBD_MAX_NUM_ENDPOINTS] = {
	USBDEP_TYPE_CONTROL
#if (USBD_MAX_NUM_ENDPOINTS > 1)
	,
	USBDEP_TYPE_CONTROL | USBDEP_TYPE_INTERRUPT | USBDEP_TYPE_BULK | USBDEP_TYPE_ISOCHRONOUS
#endif
#if (USBD_MAX_NUM_ENDPOINTS > 2)
	,
	USBDEP_TYPE_CONTROL | USBDEP_TYPE_INTERRUPT | USBDEP_TYPE_BULK | USBDEP_TYPE_ISOCHRONOUS
#endif
#if (USBD_MAX_NUM_ENDPOINTS > 3)
	,
	USBDEP_TYPE_CONTROL | USBDEP_TYPE_INTERRUPT | USBDEP_TYPE_BULK | USBDEP_TYPE_ISOCHRONOUS
#endif
#if (USBD_MAX_NUM_ENDPOINTS > 4)
	,
	USBDEP_TYPE_CONTROL | USBDEP_TYPE_INTERRUPT | USBDEP_TYPE_BULK | USBDEP_TYPE_ISOCHRONOUS
#endif
#if (USBD_MAX_NUM_ENDPOINTS > 5)
	,
	USBDEP_TYPE_CONTROL | USBDEP_TYPE_INTERRUPT | USBDEP_TYPE_BULK | USBDEP_TYPE_ISOCHRONOUS
#endif
#if (USBD_MAX_NUM_ENDPOINTS > 6)
	,
	USBDEP_TYPE_CONTROL | USBDEP_TYPE_INTERRUPT | USBDEP_TYPE_BULK | USBDEP_TYPE_ISOCHRONOUS
#endif
#if (USBD_MAX_NUM_ENDPOINTS > 7)
	,
	USBDEP_TYPE_CONTROL | USBDEP_TYPE_INTERRUPT | USBDEP_TYPE_BULK | USBDEP_TYPE_ISOCHRONOUS
#endif
#if (USBD_MAX_NUM_ENDPOINTS > 8)
	,
	0
#endif
#if (USBD_MAX_NUM_ENDPOINTS > 9)
	,
	0
#endif
#if (USBD_MAX_NUM_ENDPOINTS > 10)
	,
	0
#endif
#if (USBD_MAX_NUM_ENDPOINTS > 11)
	,
	0
#endif
#if (USBD_MAX_NUM_ENDPOINTS > 12)
	,
	0
#endif
#if (USBD_MAX_NUM_ENDPOINTS > 13)
	,
	0
#endif
#if (USBD_MAX_NUM_ENDPOINTS > 14)
	,
	0
#endif
#if (USBD_MAX_NUM_ENDPOINTS > 15)
	,
	0
#endif
};







#if(USBD_MAX_NUM_ENDPOINTS > 1)
// licznik uzywany podczas konfigurowania urzadzenia przez Hosta
// potrzebny przy ustalaniu adresow buforow poszczegolnych Endpointow
static uint16_t USBDEP_buffer_busy_space_size = 0;
#endif

// przechowuje status USBD
static uint16_t USBD_device_status;

#if(((USBDEP_BULK_SUPPORT == USBD_FEATURE_ON) || (USBDEP_ISOCHRONOUS_SUPPORT == USBD_FEATURE_ON)) && (USBD_MAX_NUM_ENDPOINTS > 1))
// pole bitowe wskazujace czy dany endpoint uzywa aktualnie podwojnego bufora
// Uzywane podczas transmisji przez Endpoity z opcja PING-PONG
static uint16_t USBDEP_double_buffer;
#endif

// zawiera tablice wskaznikow do struktor opisujacych poszczegolne konfiguracje
static USBDC_params *USBD_configurations[USBD_MAX_NUM_CONFIGURATIONS + 1];
#if(USBD_ZERO_CONFIG_SUPPORT == USBD_FEATURE_ON)
static uint8_t USBDZC_state = 0;
#endif

// jest to wskaznik uzywany przy enumeracji - wskazuje na strukture opisujaca
// aktywna konfiguracje lub jest rowny 0 jezeli nie zostala ustawiona zadna konfiguracja
static USBDC_params *pUSBDC_active = 0;

// wskaznik do funkcji ktora ma zostac wykonana po wyslaniu statusu (transmisja kontrolna,
// zerowy pakiet)
static void (*USBD_status_in_complete_irq) (USBDC_params*, USBDEP_params *EP);

// zmienna uzywana przy ustawianiu adresu urzadzenia
static uint16_t USBD_received_address;

// zmienna przechowujaca status widziany przez konfiguracje, ktore nie sa aktywne
static uint8_t USBD_status_masked;

// zmienna przechowujaca status widziany przez konfiguracje, ktora jest aktywna
static uint8_t USBD_status;

#if((USBDEP_BULK_SUPPORT == USBD_FEATURE_ON) && (USBD_MAX_NUM_ENDPOINTS > 1))
// zmienna mowiaca, czy jestesmy w czasie wykonywania przerwnia USB wyzszego priorytetu
static uint8_t USBDEP_HP_state = 0;
#endif






/* *****************************************************************************
				DEVICE DESCRIPTOR
   ***************************************************************************** */
static USBD_device_descriptor USBD_Device_Descriptor = {
	sizeof(USBD_device_descriptor),		// bLength
	USBD_DESC_TYPE_DEVICE,				// bDescriptorType
	0x0200,								// bcdUSB
	0x00,								// bDeviceClass
	0x00,								// bDeviceSubclass
	0x00,								// bDeviceProtocol
	USBDEP_0_MAX_POSSIBLE_PACKET_SIZE,	// bMaxPacketSize0
	0x0000,								// idVendor
	0x0000,								// idProduct
	0x0100,								// bcdDevice
	0x00,								// iManufacturer
	0x00,								// iProduct
	0x00,								// iSerialNumber
	0x00								// bNumConfigurations;
};


/* *****************************************************************************
				LANGUAGE STRING DESCRIPTOR
   ***************************************************************************** */
static const uint8_t USBD_Language_String_Descriptor[] = {
	0x04,   			// bLength
	0x03,    			// bDescriptorType: string descriptor
	0x09,    			// Supported language: English - United States
	0x04
};









static const uint8_t USBD_interface_status[] = {
0,
0
};







static const uint8_t USBD_zero_const[] = {
0
};







static void USBD_EP_OUT_SB_irq (uint8_t ep_number);
static void USBD_EP_IN_SB_irq (uint8_t ep_number);

#if((USBDEP_BULK_SUPPORT == USBD_FEATURE_ON) && (USBD_MAX_NUM_ENDPOINTS > 1))
static void USBDEP_perform_bulk_out_start_second_step(uint8_t ep_number, USBDEP_params *EP);
static void USBDEP_perform_bulk_out_start_first_step(uint8_t ep_number);
static void USBDEP_perform_bulk_out(uint8_t ep_number);
static void USBDEP_perform_bulk_in(uint8_t ep_number);
#endif

#if((USBDEP_ISOCHRONOUS_SUPPORT == USBD_FEATURE_ON) && (USBD_MAX_NUM_ENDPOINTS > 1))
static void USBDEP_perform_isochronous_out(uint8_t ep_number);
static void USBDEP_perform_isochronous_in(uint8_t ep_number);
#endif










USBDEP_params* USBDEP_get_params(uint8_t ep_number, USBDC_params *usbdc) {
	if(ep_number >= USBD_MAX_NUM_ENDPOINTS) {
		return(0);
	}
	return(&USBD_EP_params[ep_number]);
}








uint16_t USBDEP_get_max_possible_packet_size (uint8_t ep_number, USBDC_params *usbdc) {
	if(ep_number >= USBD_MAX_NUM_ENDPOINTS) {
		return(0);
	}
	return(USBDEP_max_possible_packet_size[ep_number]);
}








#if(USBD_CCM_SUPPORT == USBD_FEATURE_ON)
uint8_t CCM_get_ep_supported_types(uint8_t ep_number, USBDC_params *usbdc) {
	if(ep_number >= USBD_MAX_NUM_ENDPOINTS) {
		return(0);
	}
	return(USBDEP_supported_types[ep_number]);
}
#endif








#if(USBD_CCM_SUPPORT == USBD_FEATURE_ON)
uint8_t CCM_get_free_ep(USBDC_params *usbdc, CCM_ep_params_add *ep_params, CCM_session_params* session, uint8_t *ep_index) {
	uint8_t ep_number;
	uint8_t type;

	for(ep_number = 1; ep_number < USBD_MAX_NUM_ENDPOINTS; ep_number++) {
		if(!(session->ep_state & (0x1 << ep_number))) {
			type = CCM_get_ep_supported_types(ep_number, usbdc);
			if(ep_params->type == (type & ep_params->type)) {
				session->ep_state |= (0x1 << ep_number);
				break;
			}
		}
	}

	if(ep_number == USBD_MAX_NUM_ENDPOINTS) {
		ep_number = 0;
	}

	*ep_index = ep_number;

	if(!ep_number) {
		return(CCM_ERROR_TOO_MANY_ENDPOINTS);
	}
	else {
		return(CCM_ERROR_NONE);
	}
}
#endif








#if(USBD_CCM_SUPPORT == USBD_FEATURE_ON)
static uint8_t USBD_find_the_bigest_val(uint8_t *priority) {
	uint8_t count;
	uint8_t max_prior = 0;

	for(count = 1; count < USBD_MAX_NUM_ENDPOINTS; count++) {
		if(priority[count] > max_prior) {
			max_prior = priority[count];
		}
	}

	return(max_prior);
}

static void USBD_resize_endpoints(
		CCM_session_params* session,
		uint16_t *mps_sum,
		uint8_t *original_mps,
		uint8_t *priority,
		uint8_t *modified_priority,
		uint8_t low_mps_limit)
	{
	uint16_t max_prior;
	uint8_t state;
	uint8_t counter;
	union {
		uint32_t uint32;
		uint16_t uint16;
	}val;

	while(*mps_sum > (USBDEP_SUMARIZED_BUFFERS_MEMORY_SIZE - (2 * USBDEP_0_MAX_POSSIBLE_PACKET_SIZE))) {
		state = 0;
		max_prior = USBD_find_the_bigest_val(modified_priority);
		for(counter = 1; counter < USBD_MAX_NUM_ENDPOINTS; counter++) {

			if((modified_priority[counter] >= max_prior) && (session->endpoint[counter].max_packet_size > low_mps_limit)) {
				if(session->endpoint[counter].type & USBDEP_TYPE_CONTROL) {
					session->endpoint[counter].max_packet_size >>= 1;
					*mps_sum -= ((session->endpoint[counter].max_packet_size << 1) + 2);
				}
				else {
					session->endpoint[counter].max_packet_size--;
				}
				val.uint32 = 0;
				val.uint16 = priority[counter];
				val.uint32 *= session->endpoint[counter].max_packet_size;
				val.uint32 /= original_mps[counter];
				modified_priority[counter] = val.uint16;

				if(session->endpoint[counter].type & (USBDEP_TYPE_CONTROL | USBDEP_TYPE_BULK | USBDEP_TYPE_ISOCHRONOUS)) {
					*mps_sum -= 2;
				}
				else {
					*mps_sum -= 1;
				}

				state = 1;
			}
		}

		if(!state) {
			break;
		}
	}
}

uint8_t CCM_adjust_endpoints(USBDC_params *usbdc, CCM_session_params* session) {
	uint8_t counter;
	uint16_t mps_sum = 0;
	uint16_t mandatory_mps_sum = 0;
	uint8_t num_mandatory_ep = 0;
	uint8_t original_mps[USBD_MAX_NUM_ENDPOINTS];
	uint8_t priority[USBD_MAX_NUM_ENDPOINTS];
	uint8_t modified_priority[USBD_MAX_NUM_ENDPOINTS];

	for(counter = 1; counter < USBD_MAX_NUM_ENDPOINTS; counter++) {
		mps_sum += session->endpoint[counter].max_packet_size;
		if(session->endpoint[counter].max_packet_size) {
			original_mps[counter] = session->endpoint[counter].max_packet_size;
			priority[counter] =  255 - session->endpoint[counter].max_packet_size_priority;
			modified_priority[counter] =  255 - session->endpoint[counter].max_packet_size_priority;

			if(session->endpoint[counter].max_packet_size_priority == 255) {
				mandatory_mps_sum += session->endpoint[counter].max_packet_size;
				num_mandatory_ep++;
			}

			if(session->endpoint[counter].type & (USBDEP_TYPE_CONTROL | USBDEP_TYPE_BULK | USBDEP_TYPE_ISOCHRONOUS)) {
				mps_sum += session->endpoint[counter].max_packet_size;
				if(session->endpoint[counter].max_packet_size_priority == 255) {
					mandatory_mps_sum += session->endpoint[counter].max_packet_size;
				}
			}
		}
		else {
			original_mps[counter] = 0;
			priority[counter] = 0;
			modified_priority[counter] = 0;
		}
	}

	if(mandatory_mps_sum > ((USBDEP_SUMARIZED_BUFFERS_MEMORY_SIZE - (2 * USBDEP_0_MAX_POSSIBLE_PACKET_SIZE)) - (session->num_reserved_endpoints * 16))) {
		return(CCM_ERROR_TOO_BIG_EP_MANDATORY_BUFFERS_SPACE_SUM);
	}

	if(mps_sum <= (USBDEP_SUMARIZED_BUFFERS_MEMORY_SIZE - (2 * USBDEP_0_MAX_POSSIBLE_PACKET_SIZE))) {
		return(CCM_ERROR_NONE);
	}

	USBD_resize_endpoints(session, &mps_sum, original_mps, priority, modified_priority, 16);

	if(mps_sum > (USBDEP_SUMARIZED_BUFFERS_MEMORY_SIZE - (2 * USBDEP_0_MAX_POSSIBLE_PACKET_SIZE))) {
		USBD_resize_endpoints(session, &mps_sum, original_mps, priority, modified_priority, 8);
	}

	// TODO: test it
	for(counter = 1; counter < USBD_MAX_NUM_ENDPOINTS; counter++) {
		if(session->endpoint[counter].max_packet_size) {
			session->endpoint[counter].max_packet_size_priority = 255 - modified_priority[counter];
		}
	}

	return(CCM_ERROR_NONE);
}
#endif







#if(USBD_MAX_NUM_ENDPOINTS > 1)
static void USBD_Endpoint_configure (USBD_endpoint_descriptor *USBDEP, uint16_t *ep_max_used_packet_size) {
	USBDEP_params *EP;
	union {
		uint32_t uint32;
		uint8_t uint8;
	} ep_number;
	uint16_t mps;
	USBDEP_private_params *EPP;

	ep_number.uint32 = 0;
	ep_number.uint8 = USBDEP->bEndpointAddress & 0xF;

	if(ep_number.uint8 >= USBD_MAX_NUM_ENDPOINTS) {
		return;
	}

	EP = &USBD_EP_params[ep_number.uint8];
	EPP = &USBDEP_private[ep_number.uint8];

	// wypelnienie struktury opisujacej endpoint
	EP->number = ep_number.uint8;
	EP->max_packet_size = (((uint16_t)USBDEP->wMaxPacketSize.L) | (((uint16_t)USBDEP->wMaxPacketSize.H) << 8)) & 0x7FF;
	EP->status = 0;
	// pole "interface" wypelniane jest jezeli potrzeba
	// przez funkcje wywolujaca USBD_Endpoint_configure
	EP->endpoint_descriptor = USBDEP;
	EP->transfer_params.data = 0;
	EP->transfer_params.data_left_length = 0;
	EP->transfer_params.data_buffered_length = 0;
	EP->transfer_params.data_transferred_length = 0;
	EP->transfer_params.transfer_type = USBD_IDLE;
	EP->transfer_params.transfer_state = USBD_IDLE_OR_BACKGROUND;

	// skasowanie wszystkich bitow w rejestrze sterujacym
	// w razie, gdyby Endpoint nie byl wylaczany przed wywolaniem funkcji
	// USBD_Endpoint_configure (np przy rzadaniu USB_CLEAR_FEATURE_ENDPOINT)
	USBD_STM32_REG->EP_REG[ep_number.uint8] &= USBDEP_STM32_STAT_RX | USBDEP_STM32_STAT_TX | USBDEP_STM32_DTOG_RX | USBDEP_STM32_DTOG_TX;

	// wypelnienie czesci parametrow prywatnych endpointa
	EPP->fifo_counter = 0;
	EPP->fifo_data_size[0] = 0;
	EPP->fifo_data_size[1] = 0;
	EPP->fifo_OUT_data_position = 0;
	EPP->partial_read_additional_byte = 0;
	EPP->partial_read = 0;
	EPP->EP_type = USBDEP->bmAttributes & 0x3;

	USBDEP_hc_irq[ep_number.uint8] = 0;
	mps = EP->max_packet_size;

	switch(USBDEP->bmAttributes & 0x3) {
		case USBDEP_DESC_TRANSFER_TYPE_CONTROL:
			EPP->num_fifos = 1;

			// wyzerowanie rozmiaru danych do wyslania
			USBD_STM32_EPMEM[ep_number.uint8].SB.tx_count = 0;

			// ustawienie dopuszczalnego rozmiaru odbieranych danych
			USBDEP_SB_set_rx_max_possible_count(ep_number.uint8, mps);

			if(USBDEP_buffer_busy_space_size) {
				// wpisanie do tablicy buforow adresu bufora uzywanego
				// do transmisji wyjsciowej przez ten endpoint
				USBDEP_set_rx_buf_addr(ep_number.uint8, USBDEP_buffer_busy_space_size);

				// zwiekszenie licznika zajetej przestrzeni bufora
				USBDEP_buffer_busy_space_size += ep_max_used_packet_size[ep_number.uint8];

				// wpisanie do tablicy buforow adresu bufora uzywanego
				// do transmisji wejsciowej przez ten endpoint
				USBDEP_set_tx_buf_addr(ep_number.uint8, USBDEP_buffer_busy_space_size);

				// zwiekszenie licznika zajetej przestrzeni bufora
				USBDEP_buffer_busy_space_size += ep_max_used_packet_size[ep_number.uint8];
			}

			// ustawienie rejestru kontrolnego endpointu
			USBD_STM32_REG->EP_REG[ep_number.uint8] = USBDEP_STM32_RX_VALID | USBDEP_STM32_TX_NAK | USBDEP_STM32_CONTROL | ep_number.uint32;
			break;

#if (USBDEP_ISOCHRONOUS_SUPPORT == USBD_FEATURE_ON)
		case USBDEP_DESC_TRANSFER_TYPE_ISOCHRONOUS:
			EPP->num_fifos = 1;

			if(USBDEP_buffer_busy_space_size) {
				// wpisanie do tablicy buforow adresu bufora uzywanego
				// do transmisji wyjsciowej przez ten endpoint
				USBDEP_set_rx_buf_addr(ep_number.uint8, USBDEP_buffer_busy_space_size);

				// zwiekszenie licznika zajetej przestrzeni bufora
				USBDEP_buffer_busy_space_size += ep_max_used_packet_size[ep_number.uint8];

				// wpisanie do tablicy buforow adresu bufora uzywanego
				// do transmisji wejsciowej przez ten endpoint
				USBDEP_set_tx_buf_addr(ep_number.uint8, USBDEP_buffer_busy_space_size);

				// zwiekszenie licznika zajetej przestrzeni bufora
				USBDEP_buffer_busy_space_size += ep_max_used_packet_size[ep_number.uint8];
			}

			// OUT
			if(!(USBDEP->bEndpointAddress & 0x80)) {
				// ustawienie dopuszczalnego rozmiaru odbieranych danych
				USBDEP_DB_set_rx_max_possible_count(ep_number.uint8, mps);

				USBDEP_hc_irq[ep_number.uint8] = USBDEP_perform_isochronous_out;

				// ustawienie rejestru kontrolnego endpointu
				USBD_STM32_REG->EP_REG[ep_number.uint8] = USBDEP_STM32_RX_VALID | USBDEP_STM32_TX_DIS | USBDEP_STM32_ISOCHRONOUS | ep_number.uint32;
			}
			// IN
			else {
				// wyzerowanie rozmiaru danych do wyslania
				USBD_STM32_EPMEM[ep_number.uint8].DB[0].count = 0;
				USBD_STM32_EPMEM[ep_number.uint8].DB[1].count = 0;

				USBDEP_hc_irq[ep_number.uint8] = USBDEP_perform_isochronous_in;

				// ustawienie rejestru kontrolnego endpointu
				USBD_STM32_REG->EP_REG[ep_number.uint8] = USBDEP_STM32_RX_DIS | USBDEP_STM32_TX_VALID | USBDEP_STM32_ISOCHRONOUS | ep_number.uint32;
			}
			break;
#endif

#if (USBDEP_BULK_SUPPORT == USBD_FEATURE_ON)
		case USBDEP_DESC_TRANSFER_TYPE_BULK:
			EPP->num_fifos = 2;

			if(USBDEP_buffer_busy_space_size) {
				// wpisanie do tablicy buforow adresu bufora uzywanego
				// do transmisji wyjsciowej przez ten endpoint
				USBDEP_set_rx_buf_addr(ep_number.uint8, USBDEP_buffer_busy_space_size);

				// zwiekszenie licznika zajetej przestrzeni bufora
				USBDEP_buffer_busy_space_size += ep_max_used_packet_size[ep_number.uint8];

				// wpisanie do tablicy buforow adresu bufora uzywanego
				// do transmisji wejsciowej przez ten endpoint
				USBDEP_set_tx_buf_addr(ep_number.uint8, USBDEP_buffer_busy_space_size);

				// zwiekszenie licznika zajetej przestrzeni bufora
				USBDEP_buffer_busy_space_size += ep_max_used_packet_size[ep_number.uint8];
			}

			// OUT
			if(!(USBDEP->bEndpointAddress & 0x80)) {
				// ustawienie dopuszczalnego rozmiaru odbieranych danych
				USBDEP_DB_set_rx_max_possible_count(ep_number.uint8, mps);

				USBDEP_hc_irq[ep_number.uint8] = USBDEP_perform_bulk_out_start_first_step;

				// ustawienie rejestru kontrolnego endpointu
				USBD_STM32_REG->EP_REG[ep_number.uint8] = USBDEP_STM32_RX_VALID | USBDEP_STM32_TX_DIS | USBDEP_STM32_BULK | USBDEP_STM32_KIND | ep_number.uint32;
			}
			// IN
			else {
				// wyzerowanie rozmiaru danych do wyslania
				USBD_STM32_EPMEM[ep_number.uint8].DB[0].count = 0;
				USBD_STM32_EPMEM[ep_number.uint8].DB[1].count = 0;

				USBDEP_hc_irq[ep_number.uint8] = USBDEP_perform_bulk_in;

				// ustawienie rejestru kontrolnego endpointu
				USBD_STM32_REG->EP_REG[ep_number.uint8] = USBDEP_STM32_RX_DIS | USBDEP_STM32_TX_NAK | USBDEP_STM32_BULK | USBDEP_STM32_KIND | ep_number.uint32;
			}
			break;
#endif

#if (USBDEP_INTERRUPT_SUPPORT == USBD_FEATURE_ON)
		case USBDEP_DESC_TRANSFER_TYPE_INTERRUPT:
			EPP->num_fifos = 1;

			// OUT
			if(!(USBDEP->bEndpointAddress & 0x80)) {
				// ustawienie dopuszczalnego rozmiaru odbieranych danych
				USBDEP_SB_set_rx_max_possible_count(ep_number.uint8, mps);

				if(USBDEP_buffer_busy_space_size) {
					// wpisanie do tablicy buforow adresu bufora uzywanego
					// do transmisji wyjsciowej przez ten endpoint
					USBDEP_set_rx_buf_addr(ep_number.uint8, USBDEP_buffer_busy_space_size);

					// zwiekszenie licznika zajetej przestrzeni bufora
					USBDEP_buffer_busy_space_size += ep_max_used_packet_size[ep_number.uint8];
				}

				// ustawienie rejestru kontrolnego endpointu
				USBD_STM32_REG->EP_REG[ep_number.uint8] = USBDEP_STM32_RX_VALID | USBDEP_STM32_TX_DIS | USBDEP_STM32_INTERRUPT | ep_number.uint32;
			}
			// IN
			else {
				// wyzerowanie rozmiaru danych do wyslania
				USBD_STM32_EPMEM[ep_number.uint8].SB.tx_count = 0;

				if(USBDEP_buffer_busy_space_size) {
					// wpisanie do tablicy buforow adresu bufora uzywanego
					// do transmisji wejsciowej przez ten endpoint
					USBDEP_set_tx_buf_addr(ep_number.uint8, USBDEP_buffer_busy_space_size);

					// zwiekszenie licznika zajetej przestrzeni bufora
					USBDEP_buffer_busy_space_size += ep_max_used_packet_size[ep_number.uint8];
				}

				// ustawienie rejestru kontrolnego endpointu
				USBD_STM32_REG->EP_REG[ep_number.uint8] = USBDEP_STM32_RX_DIS | USBDEP_STM32_TX_NAK | USBDEP_STM32_INTERRUPT | ep_number.uint32;
			}
			break;
#endif

		default:
			break;
	}
}
#endif





static void USBD_configure (void) {
#if((USBD_MAX_NUM_ENDPOINTS > 1) || (USBD_ALTERNATE_SETTING_SUPPORT == USBD_FEATURE_ON) || (USBD_UPPER_LAYER_SUPPORT == USBD_FEATURE_ON))
	uint8_t *descriptor = pUSBDC_active->configuration_descriptor;
	uint16_t interface_address;
	uint8_t num_interfaces = descriptor[4];
	uint8_t count1;
	
#if(USBD_MAX_NUM_ENDPOINTS > 1)
	USBDEP_params *EP;
	uint8_t num_endpoints;
	uint16_t endpoint_address;
	uint8_t count2;
	uint16_t ep_max_used_packet_size[USBD_MAX_NUM_ENDPOINTS];
	
	USBDCD_create_ep_max_used_packet_size_table(descriptor, ep_max_used_packet_size);

#if((USBDEP_BULK_SUPPORT == USBD_FEATURE_ON) || (USBDEP_ISOCHRONOUS_SUPPORT == USBD_FEATURE_ON))
	USBDEP_double_buffer = 0;
#endif
	USBDEP_buffer_busy_space_size = 64 + (2 * USBDEP_0_MAX_POSSIBLE_PACKET_SIZE);
#endif
	
#if((USBD_ALTERNATE_SETTING_SUPPORT == USBD_FEATURE_ON) || (USBD_UPPER_LAYER_SUPPORT == USBD_FEATURE_ON))
	for (count1 = 0; count1 < num_interfaces; count1++) {
		pUSBDC_active->interfaces[count1].descriptor = 0;
	#if(USBD_ALTERNATE_SETTING_SUPPORT == USBD_FEATURE_ON)
		pUSBDC_active->interfaces[count1].max_alternate_setting = USBDCD_find_interface_max_alternate_setting(descriptor, count1);
	#endif
	}
#endif

#if(USBD_MAX_NUM_ENDPOINTS > 1)
	for (count1=1; count1 < USBD_MAX_NUM_ENDPOINTS; count1++) {
		EP = &USBD_EP_params[count1];
		USBD_STM32_REG->EP_REG[count1] &= USBDEP_STM32_STAT_RX | USBDEP_STM32_STAT_TX | USBDEP_STM32_DTOG_RX | USBDEP_STM32_DTOG_TX;
		EP->transfer_params.data = 0;
		EP->transfer_params.data_left_length = 0;
		EP->transfer_params.data_buffered_length = 0;
		EP->transfer_params.data_transferred_length = 0;
		EP->transfer_params.transfer_type = USBD_IDLE;
		EP->transfer_params.transfer_state = USBD_IDLE_OR_BACKGROUND;
	}
#endif
	for (count1=0; count1 < num_interfaces; count1++) {
		interface_address = USBDCD_find_interface_address(descriptor, count1, 0);
		if(interface_address) {
#if((USBD_ALTERNATE_SETTING_SUPPORT == USBD_FEATURE_ON) || (USBD_UPPER_LAYER_SUPPORT == USBD_FEATURE_ON))
			pUSBDC_active->interfaces[count1].descriptor = (USBD_interface_descriptor *)&descriptor[interface_address];
#endif
#if(USBD_MAX_NUM_ENDPOINTS > 1)
			num_endpoints = descriptor[interface_address + 4];
			endpoint_address = interface_address;
			for (count2=0; count2 < num_endpoints; count2++) {
				endpoint_address = USBDCD_find_next_endpoint_address (descriptor, endpoint_address);
				if(endpoint_address) {
					USBD_Endpoint_configure((USBD_endpoint_descriptor *)&descriptor[endpoint_address], ep_max_used_packet_size);
					EP = &USBD_EP_params[((descriptor[endpoint_address + 2]) & 0xF)];
					EP->interface = descriptor[interface_address + 2];
				}
			}
#endif
		}
	}
#if(USBD_MAX_NUM_ENDPOINTS > 1)
	USBDEP_buffer_busy_space_size = 0;
#endif
#endif
}







#if(USBD_ALTERNATE_SETTING_SUPPORT == USBD_FEATURE_ON)
static void USBD_reconfigure (uint8_t Interface_number, uint8_t Alternate_Setting, uint8_t new_Alternate_Setting) {
	uint8_t *descriptor = pUSBDC_active->configuration_descriptor;
	uint16_t interface_address;
#if(USBD_MAX_NUM_ENDPOINTS > 1)
	uint8_t num_endpoints;
	uint16_t endpoint_address;
	uint8_t count2;
	uint8_t ep_number;
	USBDEP_params *EP;

	USBDEP_buffer_busy_space_size = 0;
	interface_address = USBDCD_find_interface_address(descriptor, Interface_number, Alternate_Setting);
	if(interface_address) {
		pUSBDC_active->interfaces[Interface_number].descriptor = 0;
		num_endpoints = descriptor[interface_address + 4];
		endpoint_address = interface_address;
		for (count2=0; count2 < num_endpoints; count2++) {
			endpoint_address = USBDCD_find_next_endpoint_address (descriptor, endpoint_address);
			if(endpoint_address) {
				ep_number = (descriptor[endpoint_address + 2]) & 0xF;
				EP = &USBD_EP_params[ep_number];
				if(ep_number < USBD_MAX_NUM_ENDPOINTS) {
					EP->endpoint_descriptor = 0;
					USBD_STM32_REG->EP_REG[ep_number] &= USBDEP_STM32_STAT_RX | USBDEP_STM32_STAT_TX | USBDEP_STM32_DTOG_RX | USBDEP_STM32_DTOG_TX;
					
#if((USBDEP_BULK_SUPPORT == USBD_FEATURE_ON) || (USBDEP_ISOCHRONOUS_SUPPORT == USBD_FEATURE_ON))
					USBDEP_double_buffer &= ~((uint16_t)(0x1 << ep_number));
#endif
					EP->transfer_params.data = 0;
					EP->transfer_params.data_left_length = 0;
					EP->transfer_params.data_buffered_length = 0;
					EP->transfer_params.data_transferred_length = 0;
					EP->transfer_params.transfer_type = USBD_IDLE;
					EP->transfer_params.transfer_state = USBD_IDLE_OR_BACKGROUND;
					EP->interface = 0;
				}
			}
		}	
	}
#endif
	interface_address = USBDCD_find_interface_address(descriptor, Interface_number, new_Alternate_Setting);
	if(interface_address) {
		pUSBDC_active->interfaces[Interface_number].descriptor = (USBD_interface_descriptor *)&descriptor[interface_address];
#if(USBD_MAX_NUM_ENDPOINTS > 1)
		num_endpoints = descriptor[interface_address + 4];
		endpoint_address = interface_address;
		for (count2=0; count2 < num_endpoints; count2++) {
			endpoint_address = USBDCD_find_next_endpoint_address (descriptor, endpoint_address);
			if(endpoint_address) {
				USBD_Endpoint_configure((USBD_endpoint_descriptor *)&descriptor[endpoint_address], 0);
				ep_number = (descriptor[endpoint_address + 2]) & 0xF;
				EP = &USBD_EP_params[ep_number];
				EP->interface = Interface_number;
			}
		}
#endif
	}
}
#endif









void USBD_send_status(void (*handler)(USBDC_params*, USBDEP_params *), uint8_t ep_number, USBDC_params *usbdc) {
	uint32_t ep_reg_mirror;
	USBDEP_params *EP;

	// sprawdzenie czy konfiguracja ktora wywolala ta funkcje ma prawo do jej uzycia
	// czyli czy dana konfiguracja jest aktywna oraz czy numer punktu koncowego
	// jest mniejszy od maksymalnego dozwolonego
	if((usbdc == pUSBDC_active)
			&& (ep_number < USBD_MAX_NUM_ENDPOINTS)) {
		EP = &USBD_EP_params[ep_number];
		EP->transfer_params.data = 0;
		EP->transfer_params.transfer_type = USBD_CONTROL_IN | USBD_TRANSFER_PRESENT;
		EP->transfer_params.transfer_state = USBD_STATUS_STAGE;
		EP->transfer_params.data_buffered_length = 0;
		EP->transfer_params.data_left_length = 0;

		// ustawienie wskaznika do funkcji jaka ma byc wywolana po zakonczeniu transmisji
		USBD_status_in_complete_irq = handler;

		// ustawienie zerowej dlgosci wysylanych danych
		USBDEP_SB_set_tx_count(EP->number, 0);

		// wyslanie pakietu
		USBDEP_reg_modify(ep_number, ep_reg_mirror, USBDEP_STM32_TX_VALID | USBDEP_STM32_RX_VALID,
				USBDEP_STM32_TYPE | USBDEP_STM32_ADDR | USBDEP_STM32_KIND | USBDEP_STM32_STAT_TX | USBDEP_STM32_STAT_RX, USBDEP_STM32_CTR_RX);
	}
}








void USBD_send_stall(uint8_t ep_number, USBDC_params *usbdc) {
	uint32_t ep_reg_mirror;

	// sprawdzenie czy konfiguracja ktora wywolala ta funkcje ma prawo do jej uzycia
	// czyli czy dana konfiguracja jest aktywna oraz czy numer punktu koncowego
	// jest mniejszy od maksymalnego dozwolonego
	if((usbdc == pUSBDC_active)
			&& (ep_number < USBD_MAX_NUM_ENDPOINTS)) {
		USBDEP_reg_modify(ep_number, ep_reg_mirror, USBDEP_STM32_TX_STALL | USBDEP_STM32_RX_STALL,
				USBDEP_STM32_TYPE | USBDEP_STM32_ADDR | USBDEP_STM32_KIND | USBDEP_STM32_STAT_TX | USBDEP_STM32_STAT_RX, USBDEP_STM32_CTR_RX);
	}
}







void USBD_send_resume(USBDC_params *usbdc) {
	
}







void USBD_halt (uint8_t ep_number, USBDC_params *usbdc) {
#if(USBD_MAX_NUM_ENDPOINTS > 1)
	uint32_t ep_reg_mirror;
	// sprawdzenie czy konfiguracja ktora wywolala ta funkcje ma prawo do jej uzycia
	// czyli czy dana konfiguracja jest aktywna oraz czy numer punktu koncowego
	// jest mniejszy od maksymalnego dozwolonego
	if((usbdc == pUSBDC_active)
			&& (ep_number < USBD_MAX_NUM_ENDPOINTS)) {
		// ustawienie statusu Endpointa na HALT
		USBD_EP_params[ep_number].status = 1;

		// ustawienie bitow USBDEP_STM32_STAT_TX i USBDEP_STM32_STAT_RX na STALL
		USBDEP_reg_modify(ep_number, ep_reg_mirror, USBDEP_STM32_TX_STALL | USBDEP_STM32_RX_STALL,
				USBDEP_STM32_TYPE | USBDEP_STM32_ADDR | USBDEP_STM32_KIND | USBDEP_STM32_STAT_TX | USBDEP_STM32_STAT_RX, 0);
	}
#endif
}







void USBD_unhalt (uint8_t ep_number, USBDC_params *usbdc) {
#if(USBD_MAX_NUM_ENDPOINTS > 1)
	// sprawdzenie czy konfiguracja ktora wywolala ta funkcje ma prawo do jej uzycia
	// czyli czy dana konfiguracja jest aktywna oraz czy numer punktu koncowego
	// jest mniejszy od maksymalnego dozwolonego
	if((usbdc == pUSBDC_active)
			&& (ep_number < USBD_MAX_NUM_ENDPOINTS)) {
		USBDEP_params *EP = &USBD_EP_params[ep_number];

		// skasowanie statusu HALT danego Endpointa
		// wykonywane jest przez funkcje USBD_Endpoint_configure
		// wiec tu nie ma potrzeby kasowac jej
		// EP->status = 0;

		// skonfigurowanie danego Endpointa
		USBD_Endpoint_configure (EP->endpoint_descriptor, 0);
	}
#endif
}








static void USBDEP_load_data_to_buffer (uint8_t* data, uint32_t* buffer, uint32_t length) {
	uint8_t additional_byte;

	// zmienna additional_byte mowi, czy mamy do czynienia
	// z niepazysta dlugoscia danych, ktore wysylamy.
	// jest to potrzebne, gdyz operujemy na buforze 16-bitowym
	// i jezeli mamy niepazysta dlugosc danych, to na koncu
	// musimy zaladowac pojedynczy bajt do bufora zamiast 2 na raz
	additional_byte = (uint8_t)(length & 0x1);

	// zmiejszenie length 2 razy (gdyz wysylamy po 2 bajty na raz)
	// a ta zmienna jest uzywana do krecenia petla ladujaca dane do bufora
	length >>= 1;
	
	// ladowanie danych do bufora - po 2 bajty
	while (length) {
		*buffer = ((uint32_t)*data) | (((uint32_t)data[1]) << 8);
		data += 2;
		buffer++;
		length--;
	}
	// zaladowanie dodatkowego bajtu do bufora,
	// jezeli dlugosc danych byla niepazysta
	if(additional_byte) {
		*buffer = (uint32_t)*data;
		data++;
	}
}







static uint32_t* USBDEP_load_data_from_buffer (uint8_t* data, uint32_t* buffer, uint32_t length) {
	// zmienna additional_byte mowi, czy mamy do czynienia
	// z niepazysta dlugoscia danych, ktore wysylamy.
	// jest to potrzebne, gdyz operujemy na buforze 16-bitowym
	// i jezeli mamy niepazysta dlugosc danych, to na koncu
	// musimy zaladowac pojedynczy bajt do bufora zamiast 2 na raz
	uint8_t additional_byte = (uint8_t)(length & 0x1);

	// zmiejszenie length 2 razy (gdyz wysylamy po 2 bajty na raz)
	// a ta zmienna jest uzywana do krecenia petla ladujaca dane do bufora
	length >>= 1;
	
	// ladowanie danych do bufora - po 2 bajty
	while (length) {
		*data = *((uint8_t*)buffer);
		data++;
		*data = ((uint8_t*)buffer)[1];
		data++;
		buffer++;
		length--;
	}

	// odczytanie dodatkowego bajtu z bufora,
	// jezeli dlugosc danych byla niepazysta
	if(additional_byte) {
		*data = *((uint8_t*)buffer);
	}
	return(buffer);
}







#define USBDEP_get_num_free_IN_fifos(ep_number)	(USBDEP_private[(ep_number)].num_fifos - USBDEP_private[(ep_number)].fifo_counter)
/*
static uint8_t USBDEP_get_num_free_IN_fifos(uint8_t ep_number) {
	USBDEP_private_params *EPP = &USBDEP_private[ep_number];
	return(EPP->num_fifos - EPP->fifo_counter);
}//*/








#define USBDEP_get_num_busy_IN_fifos(ep_number)	(USBDEP_private[(ep_number)].fifo_counter)
/*
static uint8_t USBDEP_get_num_busy_IN_fifos(uint8_t ep_number) {
	USBDEP_private_params *EPP = &USBDEP_private[ep_number];
	return(EPP->fifo_counter);
}//*/








static void USBDEP_get_fifo_IN(uint8_t *data, uint16_t length, uint8_t ep_number) {
	uint32_t ep_reg_mirror;
	USBDEP_private_params *EPP = &USBDEP_private[ep_number];
	
	if(EPP->fifo_counter >= EPP->num_fifos) {
		EPP->fifo_counter = EPP->num_fifos;
		return;
	}

	EPP->fifo_counter++;

#if((USBDEP_ISOCHRONOUS_SUPPORT == USBD_FEATURE_ON) && (USBD_MAX_NUM_ENDPOINTS > 1))
	if(EPP->EP_type == USBDEP_DESC_TRANSFER_TYPE_ISOCHRONOUS) {
		// poinformowanie sprzet o dlugosci danych, ktore wysylamy
		USBDEP_ISO_set_tx_count(ep_number, length);

		if(length) {
			// przeladowanie danych do bufora
			USBDEP_load_data_to_buffer (data, (uint32_t*)USBDEP_ISO_get_tx_buffer_addr(ep_number), length);
		}
		

		EPP->fifo_data_size[0] = length;
	}
	else
#endif
#if((USBDEP_BULK_SUPPORT == USBD_FEATURE_ON) && (USBD_MAX_NUM_ENDPOINTS > 1))
	if(EPP->num_fifos == 2) {
		if(EPP->fifo_counter == 1) {
			// modyfikacja rejestru sterujacego USBD - przestawienie statusu na VALID
			USBDEP_reg_modify(
					ep_number,
					ep_reg_mirror,
					USBDEP_STM32_TX_VALID,
					USBDEP_STM32_TYPE | USBDEP_STM32_ADDR | USBDEP_STM32_KIND | USBDEP_STM32_STAT_TX,
					0);

			// poinformowanie sprzet o dlugosci danych, ktore wysylamy
			USBDEP_DB_set_tx_count(ep_number, length);

			// przeladowanie danych do bufora
			USBDEP_load_data_to_buffer (data, (uint32_t*)USBDEP_DB_get_tx_buffer_addr(ep_number), length);

			// modyfikacja rejestru sterujacego USBD - przestawienie buforow sprzetowy<->programowy (DTOG)
			USBDEP_reg_modify(
					ep_number,
					ep_reg_mirror,
					USBDEP_STM32_TX_VALID,
					USBDEP_STM32_TYPE | USBDEP_STM32_ADDR | USBDEP_STM32_KIND | USBDEP_STM32_STAT_TX,
					USBDEP_STM32_DTOG_RX);

			EPP->fifo_data_size[0] = length;
		}
		else {

			// poinformowanie sprzet o dlugosci danych, ktore wysylamy
			USBDEP_DB_set_tx_count(ep_number, length);

			// przeladowanie danych do bufora
			USBDEP_load_data_to_buffer (data, (uint32_t*)USBDEP_DB_get_tx_buffer_addr(ep_number), length);

			EPP->fifo_data_size[1] = length;
		}
	}
	else
#endif
	if(EPP->num_fifos == 1) {
		// poinformowanie sprzet o dlugosci danych, ktore wysylamy
		USBDEP_SB_set_tx_count(ep_number, length);

		// przeladowanie danych do bufora
		USBDEP_load_data_to_buffer (data, (uint32_t*)USBDEP_SB_get_tx_buffer_addr(ep_number), length);

		// modyfikacja rejestru sterujacego USBD
		USBDEP_reg_modify(ep_number,
				ep_reg_mirror,
				USBDEP_STM32_TX_VALID,
				USBDEP_STM32_TYPE | USBDEP_STM32_ADDR | USBDEP_STM32_KIND | USBDEP_STM32_STAT_TX,
				USBDEP_STM32_CTR_RX);
		

		EPP->fifo_data_size[0] = length;
		
	}
}








static uint16_t USBDEP_release_fifo_IN(uint8_t ep_number) {
	USBDEP_private_params *EPP = &USBDEP_private[ep_number];
	uint16_t release_data_size = EPP->fifo_data_size[0];
#if((USBDEP_BULK_SUPPORT == USBD_FEATURE_ON) && (USBD_MAX_NUM_ENDPOINTS > 1))
	uint32_t ep_reg_mirror;
#endif

	if(EPP->fifo_counter) {
		EPP->fifo_counter--;
	}

#if((USBDEP_BULK_SUPPORT == USBD_FEATURE_ON) && (USBD_MAX_NUM_ENDPOINTS > 1))
	if(EPP->num_fifos == 2) {
		if(EPP->fifo_counter) {
			// zezwolenie na wyslanie oczekujacego bufora
			USBDEP_reg_modify(ep_number,
					ep_reg_mirror,
					USBDEP_STM32_TX_VALID,
					USBDEP_STM32_TYPE | USBDEP_STM32_ADDR | USBDEP_STM32_KIND | USBDEP_STM32_STAT_TX,
					USBDEP_STM32_DTOG_RX);

			EPP->fifo_data_size[0] = EPP->fifo_data_size[1];
			EPP->fifo_data_size[1] = 0;
		}
		else {
			EPP->fifo_data_size[0] = 0;
		}
	}
	else 
#endif
	if(EPP->num_fifos == 1) {
		EPP->fifo_data_size[0] = 0;
	}
	
	return(release_data_size);
}








static uint8_t USBDEP_is_fifo_OUT_data_available(uint8_t ep_number) {
	USBDEP_private_params *EPP = &USBDEP_private[ep_number];
	uint8_t result = 0;

#if((USBDEP_ISOCHRONOUS_SUPPORT == USBD_FEATURE_ON) && (USBD_MAX_NUM_ENDPOINTS > 1))
	if(EPP->EP_type == USBDEP_DESC_TRANSFER_TYPE_ISOCHRONOUS) {
		if(EPP->fifo_counter) {
			result = 1;
		}
	}
	else
#endif
#if((USBDEP_BULK_SUPPORT == USBD_FEATURE_ON) && (USBD_MAX_NUM_ENDPOINTS > 1))
	if(EPP->num_fifos == 2) {
		if(((USBD_STM32_REG->EP_REG[ep_number] & USBDEP_STM32_DTOG_TX) == ((USBD_STM32_REG->EP_REG[ep_number] & USBDEP_STM32_DTOG_RX) >> 8)) && (EPP->fifo_counter)) {
			result = 1;
		}
		if(EPP->fifo_counter == 1) {
			result = 1;
		}
	}
	else
#endif
	if(EPP->num_fifos == 1) {
		if((USBD_STM32_REG->EP_REG[ep_number] & USBDEP_STM32_STAT_RX) == USBDEP_STM32_RX_NAK) {
			result = 1;
		}
	}
	if(EPP->partial_read) {
		result = 1;
	}

	return(result);
}








#define USBDEP_get_fifo_OUT_packet_size(ep_number)		(USBDEP_private[(ep_number)].fifo_data_size[1])








static uint16_t USBDEP_get_fifo_OUT_stored_data_size(uint8_t ep_number) {
	USBDEP_private_params *EPP = &USBDEP_private[ep_number];

	if(!EPP->partial_read) {
#if((USBDEP_ISOCHRONOUS_SUPPORT == USBD_FEATURE_ON) && (USBD_MAX_NUM_ENDPOINTS > 1))
		if(EPP->EP_type == USBDEP_DESC_TRANSFER_TYPE_ISOCHRONOUS) {
			EPP->fifo_data_size[0] = USBDEP_ISO_get_rx_count(ep_number);
			EPP->fifo_OUT_data_position = (uint32_t*)USBDEP_ISO_get_rx_buffer_addr(ep_number);
		}
		else
#endif
#if((USBDEP_BULK_SUPPORT == USBD_FEATURE_ON) && (USBD_MAX_NUM_ENDPOINTS > 1))
		if(EPP->num_fifos == 2) {
			if(EPP->fifo_counter == 2) {
				EPP->fifo_data_size[0] = USBDEP_DB_get_rx_count_HB(ep_number);
				EPP->fifo_OUT_data_position = (uint32_t*)USBDEP_DB_get_rx_buffer_addr_HB(ep_number);
			}
			else {
				EPP->fifo_data_size[0] = USBDEP_DB_get_rx_count(ep_number);
				EPP->fifo_OUT_data_position = (uint32_t*)USBDEP_DB_get_rx_buffer_addr(ep_number);
			}
		}
		else 
#endif
		if(EPP->num_fifos == 1) {
			EPP->fifo_data_size[0] = USBDEP_SB_get_rx_count(ep_number);
			EPP->fifo_OUT_data_position = (uint32_t*)USBDEP_SB_get_rx_buffer_addr(ep_number);
		}

		EPP->fifo_data_size[1] = EPP->fifo_data_size[0];
		EPP->partial_read_additional_byte = 0;
	}

	return(EPP->fifo_data_size[0]);
}








static void USBDEP_read_from_fifo_OUT(uint8_t *data, uint16_t length, uint8_t ep_number) {
	USBDEP_private_params *EPP = &USBDEP_private[ep_number];

	EPP->fifo_data_size[0] -= length;
	if(EPP->partial_read_additional_byte && length) {
		EPP->partial_read_additional_byte = 0;
		// odczytanie drogiej polowy slowa (drugi bajt)
		*data = ((uint8_t*)(EPP->fifo_OUT_data_position))[1];
		EPP->fifo_OUT_data_position++;
		data++;
		length--;
	}
	if(length) {
		EPP->fifo_OUT_data_position = USBDEP_load_data_from_buffer (data, EPP->fifo_OUT_data_position, length);
		if(length & 0x1) {
			EPP->partial_read_additional_byte = 1;
		}
	}

	EPP->partial_read = 1;
}








static void USBDEP_release_fifo_OUT(uint8_t *data, uint16_t length, uint8_t ep_number) {
	uint32_t ep_reg_mirror;
	USBDEP_private_params *EPP = &USBDEP_private[ep_number];

#if((USBDEP_ISOCHRONOUS_SUPPORT == USBD_FEATURE_ON) && (USBD_MAX_NUM_ENDPOINTS > 1))
	if(EPP->EP_type == USBDEP_DESC_TRANSFER_TYPE_ISOCHRONOUS) {
		EPP->fifo_counter = 0;
	}
#endif
#if((USBDEP_BULK_SUPPORT == USBD_FEATURE_ON) && (USBD_MAX_NUM_ENDPOINTS > 1))
	if(EPP->num_fifos == 2) {
		if(EPP->fifo_counter == 2) {
			// zezwolenie na odbior do oczekujacego bufora
			USBDEP_reg_modify(ep_number,
					ep_reg_mirror,
					USBDEP_STM32_RX_VALID, USBDEP_STM32_TYPE | USBDEP_STM32_ADDR | USBDEP_STM32_KIND | USBDEP_STM32_STAT_RX,
					USBDEP_STM32_DTOG_TX/* | USBDEP_STM32_CTR_RX*/);
		}

		EPP->fifo_counter = 2;
	}
#endif

	// odczyt danych z bufora jezeli jeszcze tam sa (bo mogly byc wczesniej
	// odczytane funkcja USBDEP_read_from_fifo_OUT)
	if(EPP->fifo_data_size[0] && length) {
		if(!EPP->partial_read) {
			USBDEP_load_data_from_buffer (data, EPP->fifo_OUT_data_position, length);
		}
		else {
			if(EPP->partial_read_additional_byte) {
				EPP->partial_read_additional_byte = 0;
				// odczytanie drogiej polowy slowa (drugi bajt)
				*data = ((uint8_t*)(EPP->fifo_OUT_data_position))[1];
				EPP->fifo_OUT_data_position++;
				data++;
				length--;
			}
			if(length) {
				USBDEP_load_data_from_buffer (data, EPP->fifo_OUT_data_position, length);
			}
		}
	}

#if((USBDEP_ISOCHRONOUS_SUPPORT == USBD_FEATURE_ON) && (USBD_MAX_NUM_ENDPOINTS > 1))
	if((EPP->num_fifos == 1) && (EPP->EP_type != USBDEP_DESC_TRANSFER_TYPE_ISOCHRONOUS)) {
#else
	if(EPP->num_fifos == 1) {
#endif
		// odblokowanie Endpointa do odbierania kolejnych danych
		USBDEP_reg_modify(ep_number,
				ep_reg_mirror,
				USBDEP_STM32_RX_VALID, USBDEP_STM32_TYPE | USBDEP_STM32_ADDR | USBDEP_STM32_KIND | USBDEP_STM32_STAT_RX,
				USBDEP_STM32_CTR_TX);
	}

	EPP->partial_read = 0;
	EPP->fifo_data_size[0] = 0;
	EPP->fifo_data_size[1] = 0;
}








void USBD_send_data_in (uint8_t* data, uint32_t length, uint8_t ep_number, USBDC_params *usbdc) {
	USBDEP_params *EP;
	uint16_t MPS;

	// sprawdzenie czy konfiguracja ktora wywolala ta funkcje jest aktywna
	// oraz czy zdeklarowalismy dane o dlugosci wiekszej od zera (w przypadku
	// transmisji nie-izochronicznej) i czy numer punktu koncowego jest mniejszy
	// od maksymalnego dozwolonego
	if(!((usbdc == pUSBDC_active)
			&& (ep_number < USBD_MAX_NUM_ENDPOINTS))) {
		return;
	}
	if(!(length || ((USBD_STM32_REG->EP_REG[ep_number] & USBDEP_STM32_TYPE) == USBDEP_STM32_ISOCHRONOUS))) {
		return;
	}

	EP = &USBD_EP_params[ep_number];

	USBD_ENTER_CRITICAL();

	MPS = EP->max_packet_size;

	if(!USBDEP_get_num_busy_IN_fifos(ep_number)) {
		EP->transfer_params.data_buffered_length = 0;
	}

	EP->transfer_params.transfer_state = USBD_IDLE_OR_BACKGROUND;
	EP->transfer_params.data_transferred_length = 0;
	EP->transfer_params.data_left_length = EP->transfer_params.data_buffered_length;


	if(USBDEP_get_num_free_IN_fifos(ep_number)) {
		if(length > MPS) {
			USBDEP_get_fifo_IN(data, MPS, ep_number);
			data += MPS;
			EP->transfer_params.data_buffered_length += MPS;

#if((USBDEP_BULK_SUPPORT == USBD_FEATURE_ON) && (USBD_MAX_NUM_ENDPOINTS > 1))
			if(USBDEP_get_num_free_IN_fifos(ep_number)) {
				// dlugosc danych jest wieksza niz pojemnosc 2 buforow
				// w tej sytuacji reszte danych wysle USBD w kolejnych krokach
				if(length > (MPS << 1)) {
					USBDEP_get_fifo_IN(data, MPS, ep_number);
					EP->transfer_params.data = data + MPS;
					EP->transfer_params.data_buffered_length += MPS;
				}
				else {
					USBDEP_get_fifo_IN(data, length - MPS, ep_number);
					EP->transfer_params.data_buffered_length += length - MPS;

					// mamy zadeklarowane dane ktorych dlugosc jest wieksza
					// niz pojemnosc 1 bufora ale mniejsza niz 2 buforow -
					// w tej sytuacji zaladowalismy wszystko do buforow
					// i parametry transmisji ustawiamy na zakonczenie
					if(length == (MPS << 1)) {
						// ustawiamy dodatkowo status mowiacy, ze nie ma juz
						// danych na kolejny pakiet
						EP->transfer_params.transfer_state = USBD_TOO_LESS_SPACE_FOR_NEXT_PACKET;
					}
					EP->transfer_params.data = 0;
				}
			}
			else {
				EP->transfer_params.data = data;
			}
#else
			EP->transfer_params.data = data;
#endif
		}
		else {
			USBDEP_get_fifo_IN(data, length, ep_number);
			EP->transfer_params.data_buffered_length += length;

			// podsumowanie
			// mamy zadeklarowane dane ktorych dlugosc jest rowna
			// pojemnosci 1 bufora - w tej sytuacji zaladowalismy
			// wszystko do bufora i parametry transmisji ustawiamy
			// na zakonczenie
			if(length == MPS) {
				// ustawiamy dodatkowo status mowiacy, ze nie ma juz
				// danych na kolejny pakiet
				EP->transfer_params.transfer_state = USBD_TOO_LESS_SPACE_FOR_NEXT_PACKET;
			}

			EP->transfer_params.data = 0;
		}
	}
	else {
		EP->transfer_params.data = data;
	}
	// zaznaczenie czy wysylamy dane kontrolne czy zwykle
	if(EP->transfer_params.transfer_type & USBD_CONTROL) {
		EP->transfer_params.transfer_type = USBD_CONTROL_IN | USBD_TRANSFER_PRESENT;
	}
	else {
		EP->transfer_params.transfer_type = USBD_DATA_IN | USBD_TRANSFER_PRESENT;
	}
	EP->transfer_params.data_left_length += length;

	USBD_EXIT_CRITICAL();
}








void USBD_recv_data_out (uint8_t* data, uint32_t length, uint8_t ep_number, USBDC_params *usbdc) {
	USBDEP_params *EP;
	uint16_t PS;	// packet size
	uint16_t SDS;	// stored data size

	// sprawdzenie czy konfiguracja ktora wywolala ta funkcje ma prawo do jej uzycia
	// czyli czy dana konfiguracja jest aktywna oraz czy numer punktu koncowego
	// jest mniejszy od maksymalnego dozwolonego
	if(!((usbdc == pUSBDC_active)
			&& (ep_number < USBD_MAX_NUM_ENDPOINTS))) {
		return;
	}

	EP = &USBD_EP_params[ep_number];

	USBD_ENTER_CRITICAL();

	if(USBDEP_is_fifo_OUT_data_available(ep_number)) {
		SDS = USBDEP_get_fifo_OUT_stored_data_size(ep_number);
		PS = USBDEP_get_fifo_OUT_packet_size(ep_number);

		// sprawdzenie czy dlugosc czytanych danych zmiesci sie
		// w zadeklarowanej przestrzeni lub czy mamy zablokowac transmisje
		// 1. Jezeli wspieramy blokowanie transmisji, wowczas warunek ten
		// spelniony jest gdy:
		//    a. (punkt koncowy jest izochroniczny lub nie uzywamy blokowania transmisji) oraz
		//       pojemnosc zadeklarowanego bufora jest wieksza lub rowna wielkosci
		//       przechowywanych w buforze danych
		//    b. uzywamy blokowania transmisji i dlugosc zadeklarowanego bufora
		//       jest wieksza od wielkosci przechowywanych w buforze danych i rozmiar paczki
		//       jest rowny maksymalnej pojemnosci bufora (Host nie zakonczyl jeszcze transmisji)
		// 2. Jezeli nie wspieramy blokowanie transmisji, wowczas warunek ten
		// spelniony jest gdy pojemnosc zadeklarowanego bufora
		// jest wieksza lub rowna wielkosci przechowywanych w buforze danych
#if (USBDEP_DATA_OUT_WAIT_FOR_READY_SUPPORT == USBD_FEATURE_ON)
		if(((length >= SDS) && (!(pUSBDC_active->ep_data_out_wait_for_ready_mask & (0x1 << ep_number))
				|| (USBDEP_private[ep_number].EP_type == USBDEP_DESC_TRANSFER_TYPE_ISOCHRONOUS)))
				|| ((length > SDS) && (EP->max_packet_size == PS)
						&& (pUSBDC_active->ep_data_out_wait_for_ready_mask & (0x1 << ep_number))))
#else
		if(length >= SDS)
#endif
		{
			USBDEP_release_fifo_OUT(data, SDS, ep_number);
			data += SDS;

			// uaktualnienie parametrow transmisji
			EP->transfer_params.data_left_length = length - SDS;
			EP->transfer_params.data_transferred_length = SDS;
			EP->transfer_params.data_buffered_length = 0;

			// sprawdzenie czy odebrane dane mialy dlugosc rowna EP->max_packet_size
			if(PS == EP->max_packet_size) {
				// sprawdzenie czy w zadeklarowanym buforze jest jeszcze miejsce
				if(EP->transfer_params.data_left_length) {
					// uaktualnienie wskaznika do zadeklarowanego bufora
					EP->transfer_params.data = data;

					// zaznaczenie czy odbieramy dane kontrolne czy zwykle
					if(EP->transfer_params.transfer_type & USBD_CONTROL) {
						EP->transfer_params.transfer_type = USBD_CONTROL_OUT | USBD_TRANSFER_PRESENT;
					}
					else {
						EP->transfer_params.transfer_type = USBD_DATA_OUT | USBD_TRANSFER_PRESENT;
					}
					EP->transfer_params.transfer_state = USBD_IDLE_OR_BACKGROUND;
				}
				// przestrzen zadeklarowana na odebrane dane zostala juz wyczerpana
				else {
					// wyczyszczenie wskaznika do zadeklarowanego bufora
					EP->transfer_params.data = 0;

					// zaznaczenie czy odbieramy dane kontrolne czy zwykle
					if(EP->transfer_params.transfer_type & USBD_CONTROL) {
						EP->transfer_params.transfer_type = USBD_CONTROL_OUT;
					}
					else {
						EP->transfer_params.transfer_type = USBD_DATA_OUT;
					}
					EP->transfer_params.transfer_state = USBD_TOO_LESS_SPACE_FOR_NEXT_PACKET;
				}
			}
			// odebrane dane mialy dlugosc mniejsza od EP->max_packet_size
			// oznacza to, ze byla to ostatnia paczka danych jaka wysylal Host
			else {
				// wyczyszczenie wskaznika do zadeklarowanego bufora
				EP->transfer_params.data = 0;

				// zaznaczenie czy odbieramy dane kontrolne czy zwykle
				if(EP->transfer_params.transfer_type & USBD_CONTROL) {
					EP->transfer_params.transfer_type = USBD_CONTROL_OUT;
				}
				else {
					EP->transfer_params.transfer_type = USBD_DATA_OUT;
				}
				EP->transfer_params.transfer_state = USBD_END_OF_TRANSFER;
			}
		}
		// przestrzen zadeklarowana na odebrane dane jest zbyt mala
		// w tej sytuacji odbieramy pod zadany bufor tylko tyle danych ile mamy miejsca
		else {
#if (USBDEP_DATA_OUT_WAIT_FOR_READY_SUPPORT == USBD_FEATURE_ON)
			if(length >= SDS) {
				length = SDS;
			}
#endif
			// zaznaczenie ile danych jest aktualnie buforowanych
			EP->transfer_params.data_buffered_length = SDS - length;

			// odczyt z bufora tyle danych ile mamy miejsca
			USBDEP_read_from_fifo_OUT(data, length, ep_number);

			// uaktualnienie parametrow transmisji
			EP->transfer_params.data_transferred_length = length;
			EP->transfer_params.data_left_length = 0;
			EP->transfer_params.data = 0;

			// zaznaczenie czy odbieramy dane kontrolne czy zwykle
			if(EP->transfer_params.transfer_type & USBD_CONTROL) {
				EP->transfer_params.transfer_type = USBD_CONTROL_OUT;
			}
			else {
				EP->transfer_params.transfer_type = USBD_DATA_OUT;
			}
#if (USBDEP_DATA_OUT_WAIT_FOR_READY_SUPPORT == USBD_FEATURE_ON)
			if(pUSBDC_active->ep_data_out_wait_for_ready_mask & (0x1 << ep_number)) {
#if((USBDEP_ISOCHRONOUS_SUPPORT == USBD_FEATURE_ON) && (USBD_MAX_NUM_ENDPOINTS > 1))
				if(USBDEP_private[ep_number].EP_type != USBDEP_DESC_TRANSFER_TYPE_ISOCHRONOUS) {
					EP->transfer_params.transfer_type |= USBD_TRANSFER_IS_LOCKED;
				}
#else
				EP->transfer_params.transfer_type |= USBD_TRANSFER_IS_LOCKED;
#endif
			}
#endif
			if(EP->transfer_params.data_buffered_length) {
				EP->transfer_params.transfer_state = USBD_TOO_LESS_SPACE;
			}
			else {
				if(PS == EP->max_packet_size) {
					EP->transfer_params.transfer_state = USBD_TOO_LESS_SPACE_FOR_NEXT_PACKET;
				}
				else {
					EP->transfer_params.transfer_state = USBD_END_OF_TRANSFER;
				}
			}
		}
	}
	// w buforze chwilowo brak danych
	else {
		// uaktualnienie parametrow transmisji
		EP->transfer_params.data = data;
		EP->transfer_params.data_left_length = length;
		EP->transfer_params.data_buffered_length = 0;
		EP->transfer_params.data_transferred_length = 0;

		// zaznaczenie czy odbieramy dane kontrolne czy zwykle
		if(EP->transfer_params.transfer_type & USBD_CONTROL) {
			EP->transfer_params.transfer_type = USBD_CONTROL_OUT | USBD_TRANSFER_PRESENT;
		}
		else {
			EP->transfer_params.transfer_type = USBD_DATA_OUT | USBD_TRANSFER_PRESENT;
		}
		EP->transfer_params.transfer_state = USBD_IDLE_OR_BACKGROUND;
	}

	USBD_EXIT_CRITICAL();
}








void USBDEP_data_out_ready(uint8_t ep_number, USBDC_params *usbdc) {
#if (USBDEP_DATA_OUT_WAIT_FOR_READY_SUPPORT == USBD_FEATURE_ON)
	USBDEP_params *EP;

	// sprawdzenie czy konfiguracja ktora wywolala ta funkcje ma prawo do jej uzycia
	// czyli czy dana konfiguracja jest aktywna oraz czy numer punktu koncowego
	// jest mniejszy od maksymalnego dozwolonego
	if(!((usbdc == pUSBDC_active)
			&& (ep_number < USBD_MAX_NUM_ENDPOINTS))) {
		return;
	}
	EP = &USBD_EP_params[ep_number];

	USBD_ENTER_CRITICAL();

	// sprawdzamy, czy na danym endpoincie mamy wlaczona opcje czekania
	// na gotowosc wyzszej warstwy oraz czy na danym endpoincie
	// zostala zablokowana transmisja typu USBD_CONTROL_OUT
	// lub USBD_DATA_OUT
	if((pUSBDC_active->ep_data_out_wait_for_ready_mask & (0x1 << ep_number))
			&& ((EP->transfer_params.transfer_type  ==  (USBD_CONTROL_OUT | USBD_TRANSFER_IS_LOCKED))
					|| (EP->transfer_params.transfer_type  ==  (USBD_DATA_OUT | USBD_TRANSFER_IS_LOCKED)))) {

		// skasowanie bitu USBD_TRANSFER_IS_LOCKED w statusie endpointa
		EP->transfer_params.transfer_type &= ~((uint8_t)USBD_TRANSFER_IS_LOCKED);

		USBDEP_release_fifo_OUT(0, 0, ep_number);

#if((USBDEP_BULK_SUPPORT == USBD_FEATURE_ON) && (USBD_MAX_NUM_ENDPOINTS > 1))
		if(USBDEP_hc_irq[ep_number] == USBDEP_perform_bulk_out_start_first_step) {
			if(!USBDEP_HP_state) {
				if(!(USBD_STM32_REG->EP_REG[ep_number] & USBDEP_STM32_CTR_RX)) {
					if((USBD_STM32_REG->EP_REG[ep_number] & USBDEP_STM32_DTOG_TX) == ((USBD_STM32_REG->EP_REG[ep_number] & USBDEP_STM32_DTOG_RX) >> 8)) {
						EP->transfer_params.transfer_state = USBD_IDLE_OR_BACKGROUND;
						USBDEP_perform_bulk_out_start_second_step(ep_number, EP);
					}
				}
			}
		}
#endif

	}

	USBD_EXIT_CRITICAL();
#endif
}








static void USBDEP_read_next_OUT_packet (USBDEP_params *EP) {
	uint16_t PS;	// packet size
	uint16_t SDS;	// stored data size
	uint8_t ep_number = EP->number;

	SDS = USBDEP_get_fifo_OUT_stored_data_size(ep_number);
	PS = USBDEP_get_fifo_OUT_packet_size(ep_number);

	// sprawdzenie czy dlugosc czytanych danych zmiesci sie
	// w zadeklarowanej przestrzeni lub czy mamy zablokowac transmisje
	// 1. Jezeli wspieramy blokowanie transmisji, wowczas warunek ten
	// spelniony jest gdy:
	//    a. pojemnosc zadeklarowanego bufora jest wieksza lub rowna wielkosci
	//       przechowywanych w buforze danych
	//    b. dlugosc zadeklarowanego bufora jest wieksza od wielkosci
	//       przechowywanych w buforze danych i rozmiar paczki jest rowny
	//       maksymalnej pojemnosci bufora (Host nie zakonczyl jeszcze transmisji)
	// 2. Jezeli nie wspieramy blokowania transmisji, wowczas warunek ten
	// spelniony jest gdy pojemnosc zadeklarowanego bufora
	// jest wieksza lub rowna wielkosci przechowywanych w buforze danych
#if (USBDEP_DATA_OUT_WAIT_FOR_READY_SUPPORT == USBD_FEATURE_ON)
	if(((EP->transfer_params.data_left_length >= SDS) && !(pUSBDC_active->ep_data_out_wait_for_ready_mask & (0x1 << ep_number)))
			|| ((EP->transfer_params.data_left_length > SDS) && (EP->max_packet_size == PS)
					&& (pUSBDC_active->ep_data_out_wait_for_ready_mask & (0x1 << ep_number))))
#else
	if(EP->transfer_params.data_left_length >= SDS)
#endif
	{
		USBDEP_release_fifo_OUT(EP->transfer_params.data, SDS, ep_number);

		// uaktualnienie parametrow transmisji
		EP->transfer_params.data_left_length -= SDS;
		EP->transfer_params.data_transferred_length += SDS;
		EP->transfer_params.data_buffered_length = 0;

		// sprawdzenie czy odebrane dane mialy dlugosc rowna EP->max_packet_size
		if(PS == EP->max_packet_size) {
			// sprawdzenie czy w zadeklarowanym buforze przypadkiem
			// nie skonczylo sie miejsce
			if(!EP->transfer_params.data_left_length) {
				// wyczyszczenie parametrow transmisji
				EP->transfer_params.data = 0;
				
				// ustawienie statusu transmisji
//				if(EP->transfer_params.transfer_type == (USBD_CONTROL_OUT | USBD_TRANSFER_PRESENT)) {
					EP->transfer_params.transfer_type &= ~((uint8_t)USBD_TRANSFER_PRESENT);
//				}
				EP->transfer_params.transfer_state = USBD_TOO_LESS_SPACE_FOR_NEXT_PACKET;

				// jezeli to mozliwe wywolanie wyzszej warstwy API
				if(pUSBDC_active) {
					if(pUSBDC_active->ep_irq) {
						pUSBDC_active->ep_irq[ep_number] (pUSBDC_active, EP);
					}
				}
//				if((EP->transfer_params.transfer_state == USBD_TOO_LESS_SPACE_FOR_NEXT_PACKET)
//						&& (EP->transfer_params.transfer_type != (USBD_CONTROL_OUT | USBD_TRANSFER_PRESENT))) {
				if(EP->transfer_params.transfer_state == USBD_TOO_LESS_SPACE_FOR_NEXT_PACKET) {
					EP->transfer_params.transfer_type = USBD_IDLE;
					EP->transfer_params.transfer_state = USBD_IDLE_OR_BACKGROUND;
					EP->transfer_params.data_left_length = 0;
					EP->transfer_params.data_transferred_length = 0;
				}
			}
			else {
				EP->transfer_params.data += SDS;
			}
		}
		// odebrane dane mialy dlugosc mniejsza od EP->max_packet_size
		// oznacza to, ze byla to ostatnia paczka danych jaka wysylal Host
		else {
			// podsumowanie zakonczonej transmisji
			EP->transfer_params.data = 0;
			EP->transfer_params.transfer_type &= ~((uint8_t)USBD_TRANSFER_PRESENT);
			EP->transfer_params.transfer_state = USBD_END_OF_TRANSFER;
			if(pUSBDC_active) {
				if(pUSBDC_active->ep_irq[ep_number]) {
					pUSBDC_active->ep_irq[ep_number] (pUSBDC_active, EP);
				}
			}
			if(EP->transfer_params.transfer_state == USBD_END_OF_TRANSFER) {
				EP->transfer_params.transfer_type = USBD_IDLE;
				EP->transfer_params.transfer_state = USBD_IDLE_OR_BACKGROUND;
				EP->transfer_params.data = 0;
				EP->transfer_params.data_left_length = 0;
				EP->transfer_params.data_transferred_length = 0;
				EP->transfer_params.data_buffered_length = 0;
			}
		}
	}
	// przestrzen zadeklarowana na odebrane dane jest zbyt mala
	// w tej sytuacji odbieramy pod zadany bufor tylko tyle danych ile mamy miejsca
	else {
#if (USBDEP_DATA_OUT_WAIT_FOR_READY_SUPPORT == USBD_FEATURE_ON)
		if(EP->transfer_params.data_left_length >= SDS) {
			EP->transfer_params.data_left_length = SDS;
		}
#endif
		// wyznaczenie dlugosci danych jakie pozostaja nieczytane
		EP->transfer_params.data_buffered_length = SDS - EP->transfer_params.data_left_length;

		// odczyt z bufora tyle danych ile mamy miejsca
		USBDEP_read_from_fifo_OUT(EP->transfer_params.data, EP->transfer_params.data_left_length, ep_number);

		// uaktualnienie parametrow transmisji
		EP->transfer_params.data_transferred_length += EP->transfer_params.data_left_length;
		EP->transfer_params.data_left_length = 0;
		EP->transfer_params.data = 0;

		// ustawienie statusu transmisji
		EP->transfer_params.transfer_type &= ~((uint8_t)USBD_TRANSFER_PRESENT);
#if (USBDEP_DATA_OUT_WAIT_FOR_READY_SUPPORT == USBD_FEATURE_ON)
		if(pUSBDC_active->ep_data_out_wait_for_ready_mask & (0x1 << ep_number)) {
			EP->transfer_params.transfer_type |= USBD_TRANSFER_IS_LOCKED;
		}
#endif
		if(EP->transfer_params.data_buffered_length) {
			EP->transfer_params.transfer_state = USBD_TOO_LESS_SPACE;
		}
		else {
			if(PS == EP->max_packet_size) {
				EP->transfer_params.transfer_state = USBD_TOO_LESS_SPACE_FOR_NEXT_PACKET;
			}
			else {
				EP->transfer_params.transfer_state = USBD_END_OF_TRANSFER;
			}
		}

		// jezeli to mozliwe wywolanie wyzszej warstwy API
		if(pUSBDC_active) {
			if(pUSBDC_active->ep_irq[ep_number]) {
				pUSBDC_active->ep_irq[ep_number] (pUSBDC_active, EP);
			}
		}
	}
}








static void USBD_write_next_IN_packet_single_buffer (USBDEP_params *EP) {
	uint32_t length;
	uint32_t ep_reg_mirror;
	uint8_t ep_number = EP->number;
	
	// zwolnienie bufora
	length = (uint32_t)USBDEP_release_fifo_IN(ep_number);

	// uaktualnienie danych odnosnie transmisji
	EP->transfer_params.data_left_length -= length;
	EP->transfer_params.data_transferred_length += length;

	length = EP->transfer_params.data_left_length;
	
	// sprawdzenie, czy mamy jakies dane do zaladowania do bufora
	if(length) {
		// sprawdzenie, czy dlugosc danych jest wieksza
		// od maksymalnej pojemnosci bufora
		if(length > EP->max_packet_size) {
			USBDEP_get_fifo_IN(EP->transfer_params.data, EP->max_packet_size, ep_number);
			EP->transfer_params.data_buffered_length = EP->max_packet_size;
			EP->transfer_params.data += EP->max_packet_size;

		}
		else {
			USBDEP_get_fifo_IN(EP->transfer_params.data, length, ep_number);

			EP->transfer_params.data_buffered_length = length;
			EP->transfer_params.data = 0;

			// zaznaczenie stanu transmisji
			if(length == EP->max_packet_size) {
				// mamy zadeklarowane dane ktorych dlugosc jest rowna
				// pojemnosci 1 bufora - w tej sytuacji zaladowalismy
				// wszystko do bufora i parametry transmisji ustawiamy
				// na zakonczenie, ktore zostanie zrealizowane jezeli
				// nie dolaczymy kolejnych danych
				EP->transfer_params.transfer_state = USBD_TOO_LESS_SPACE_FOR_NEXT_PACKET;

				// jezeli to mozliwe wywolanie wyzszej warstwy API
				if(pUSBDC_active) {
					if(pUSBDC_active->ep_irq[ep_number]) {
						pUSBDC_active->ep_irq[ep_number] (pUSBDC_active, EP);
					}
				}
			}
		}
	}
	// nie mamy danych do zaladowania - zakonczenie transmisji
	// lub wyslanie pakietu zerowego
	else {
		// sprawdzenie, czy dane byly calkowita wielokrotnoscia EP->max_packet_size
		// i czy mamy do czynienia z transmisja kontrolna - jezeli tak, wyslanie zerowego pakietu
		if(!(EP->transfer_params.data_buffered_length < EP->max_packet_size)
				&& (EP->transfer_params.transfer_type == (USBD_CONTROL_IN | USBD_TRANSFER_PRESENT))) {
			// ustawienie dlugosci danych do wyslania
			USBDEP_SB_set_tx_count(ep_number, 0);

			// modyfikacja rejestru sterujacego USBD
			USBDEP_reg_modify(ep_number,
					ep_reg_mirror,
					USBDEP_STM32_TX_VALID,
					USBDEP_STM32_TYPE | USBDEP_STM32_ADDR | USBDEP_STM32_KIND | USBDEP_STM32_STAT_TX,
					USBDEP_STM32_CTR_RX);

			// zaznaczenie ile danych jest aktualnie buforowanych
			EP->transfer_params.data_buffered_length = 0;
		}
		else {
			// modyfikacja rejestru sterujacego USBD
			USBDEP_reg_modify(ep_number,
					ep_reg_mirror,
					USBDEP_STM32_TX_NAK,
					USBDEP_STM32_TYPE | USBDEP_STM32_ADDR | USBDEP_STM32_KIND | USBDEP_STM32_STAT_TX,
					USBDEP_STM32_CTR_RX);

			// podsumowanie zakonczonej transmisji
			EP->transfer_params.data = 0;
			EP->transfer_params.data_buffered_length = 0;
			EP->transfer_params.transfer_type &= ~((uint8_t)USBD_TRANSFER_PRESENT);
			EP->transfer_params.transfer_state = USBD_END_OF_TRANSFER;
			if(pUSBDC_active) {
				if(pUSBDC_active->ep_irq[ep_number]) {
					pUSBDC_active->ep_irq[ep_number] (pUSBDC_active, EP);
				}
			}
			if(EP->transfer_params.transfer_state == USBD_END_OF_TRANSFER) {
				EP->transfer_params.transfer_type = USBD_IDLE;
				EP->transfer_params.transfer_state = USBD_IDLE_OR_BACKGROUND;
				EP->transfer_params.data_left_length = 0;
				EP->transfer_params.data_transferred_length = 0;
			}
		}
	}
}








#if((USBDEP_BULK_SUPPORT == USBD_FEATURE_ON) && (USBD_MAX_NUM_ENDPOINTS > 1))
static void USBD_write_next_IN_packet_dual_buffer (USBDEP_params *EP) {
	uint32_t length;
	uint32_t ep_reg_mirror;
	uint8_t ep_number = EP->number;
	
	// zwolnienie bufora
	length = (uint32_t)USBDEP_release_fifo_IN(ep_number);

	EP->transfer_params.data_left_length -= length;
	EP->transfer_params.data_transferred_length += length;
	EP->transfer_params.data_buffered_length -= length;

	length = EP->transfer_params.data_left_length - EP->transfer_params.data_buffered_length;

	// sprawdzenie, czy mamy jakies dane do zaladowania do bufora
	if(length) {
		if(length > EP->max_packet_size) {
			USBDEP_get_fifo_IN(EP->transfer_params.data, EP->max_packet_size, ep_number);
			EP->transfer_params.data_buffered_length += EP->max_packet_size;
			EP->transfer_params.data += EP->max_packet_size;

			// modyfikacja rejestru sterujacego USBD
			USBDEP_reg_modify(ep_number,
					ep_reg_mirror,
					USBDEP_STM32_TX_VALID,
					USBDEP_STM32_TYPE | USBDEP_STM32_ADDR | USBDEP_STM32_KIND | USBDEP_STM32_STAT_TX,
					0);

		}
		else {
			USBDEP_get_fifo_IN(EP->transfer_params.data, length, ep_number);

			// modyfikacja rejestru sterujacego USBD
			USBDEP_reg_modify(ep_number,
					ep_reg_mirror,
					USBDEP_STM32_TX_VALID,
					USBDEP_STM32_TYPE | USBDEP_STM32_ADDR | USBDEP_STM32_KIND | USBDEP_STM32_STAT_TX,
					0);

			EP->transfer_params.data_buffered_length += length;
			EP->transfer_params.data = 0;

			if(length == EP->max_packet_size) {
				// mamy zadeklarowane dane ktorych dlugosc jest rowna
				// pojemnosci 1 bufora - w tej sytuacji zaladowalismy
				// wszystko do bufora i parametry transmisji ustawiamy
				// na zakonczenie, ktore zostanie zrealizowane jezeli
				// nie dolaczymy kolejnych danych
				EP->transfer_params.transfer_state = USBD_TOO_LESS_SPACE_FOR_NEXT_PACKET;

				// jezeli to mozliwe wywolanie wyzszej warstwy API
				if(pUSBDC_active) {
					if(pUSBDC_active->ep_irq[ep_number]) {
						pUSBDC_active->ep_irq[ep_number] (pUSBDC_active, EP);
					}
				}
			}
		}
	}
	// nie ma danych do zaladowania - przelaczenie na ostatni bufor lub zakonczenie transmisji
	else {
		// sprawdzenie czy mamy jeszcze oczekujacy na wyslanie ostatni bufor z danymi
		if(USBDEP_get_num_busy_IN_fifos(ep_number)) {
			// modyfikacja rejestru sterujacego USBD - ustawienie STAT_TX_VALID (powinno tak byc
			// ale czasami ulega to przestawieniu - nie wiem z jakiego powodu)
			USBDEP_reg_modify(ep_number,
					ep_reg_mirror,
					USBDEP_STM32_TX_VALID,
					USBDEP_STM32_TYPE | USBDEP_STM32_ADDR | USBDEP_STM32_KIND | USBDEP_STM32_STAT_TX,
					0);
			USBDEP_reg_modify(ep_number,
					ep_reg_mirror,
					USBDEP_STM32_TX_VALID,
					USBDEP_STM32_TYPE | USBDEP_STM32_ADDR | USBDEP_STM32_KIND | USBDEP_STM32_STAT_TX,
					0);
			
		}
		else {
			// zakonczenie transmisji
			USBDEP_reg_modify(ep_number,
					ep_reg_mirror,
					USBDEP_STM32_TX_NAK,
					USBDEP_STM32_TYPE | USBDEP_STM32_ADDR | USBDEP_STM32_KIND | USBDEP_STM32_STAT_TX,
					0);

			// podsumowanie zakonczonej transmisji
			EP->transfer_params.data = 0;
			EP->transfer_params.data_buffered_length = 0;
			EP->transfer_params.transfer_type &= ~((uint8_t)USBD_TRANSFER_PRESENT);
			EP->transfer_params.transfer_state = USBD_END_OF_TRANSFER;
			if(pUSBDC_active) {
				if(pUSBDC_active->ep_irq[ep_number]) {
					pUSBDC_active->ep_irq[ep_number] (pUSBDC_active, EP);
				}
			}
			if(EP->transfer_params.transfer_state == USBD_END_OF_TRANSFER) {
				EP->transfer_params.transfer_type = USBD_IDLE;
				EP->transfer_params.transfer_state = USBD_IDLE_OR_BACKGROUND;
				EP->transfer_params.data_left_length = 0;
				EP->transfer_params.data_transferred_length = 0;
			}
		}
	}
}
#endif








#if((USBDEP_ISOCHRONOUS_SUPPORT == USBD_FEATURE_ON) && (USBD_MAX_NUM_ENDPOINTS > 1))
static void USBD_write_next_IN_ISO_packet (USBDEP_params *EP) {
	uint32_t length;
	uint8_t ep_number = EP->number;
	
	// zwolnienie bufora
	length = (uint32_t)USBDEP_release_fifo_IN(ep_number);

	// uaktualnienie danych odnosnie transmisji
	EP->transfer_params.data_left_length -= length;
	EP->transfer_params.data_transferred_length += length;

	length = EP->transfer_params.data_left_length;

	if(length) {
		// sprawdzenie, czy dlugosc danych jest wieksza
		// od maksymalnej pojemnosci bufora
		if(length > EP->max_packet_size) {
			USBDEP_get_fifo_IN(EP->transfer_params.data, EP->max_packet_size, ep_number);
			EP->transfer_params.data_buffered_length = EP->max_packet_size;
			EP->transfer_params.data += EP->max_packet_size;

		}
		else {
			USBDEP_get_fifo_IN(EP->transfer_params.data, length, ep_number);

			EP->transfer_params.data_buffered_length = length;
			EP->transfer_params.data = 0;

			// zaznaczenie stanu transmisji
			if(length == EP->max_packet_size) {
				// mamy zadeklarowane dane ktorych dlugosc jest rowna
				// pojemnosci 1 bufora - w tej sytuacji zaladowalismy
				// wszystko do bufora i parametry transmisji ustawiamy
				// na zakonczenie, ktore zostanie zrealizowane jezeli
				// nie dolaczymy kolejnych danych
				EP->transfer_params.transfer_state = USBD_TOO_LESS_SPACE_FOR_NEXT_PACKET;

				// jezeli to mozliwe wywolanie wyzszej warstwy API
				if(pUSBDC_active) {
					if(pUSBDC_active->ep_irq[ep_number]) {
						pUSBDC_active->ep_irq[ep_number] (pUSBDC_active, EP);
					}
				}
			}
		}
	}
	else {
		// wyzerowanie dlugosci danych w polu tablicy buforow
		// danego endpointa
		USBDEP_ISO_set_tx_count(ep_number, length);

		// podsumowanie zakonczonej transmisji
		EP->transfer_params.transfer_type &= ~((uint8_t)USBD_TRANSFER_PRESENT);
		EP->transfer_params.transfer_state = USBD_END_OF_TRANSFER;

		// jezeli to mozliwe wywolanie wyzszej warstwy API
		if(pUSBDC_active) {
			if(pUSBDC_active->ep_irq[ep_number]) {
				pUSBDC_active->ep_irq[ep_number] (pUSBDC_active, EP);
			}
		}

		if(EP->transfer_params.transfer_state == USBD_END_OF_TRANSFER) {
			EP->transfer_params.transfer_type = USBD_IDLE;
			EP->transfer_params.transfer_state = USBD_IDLE_OR_BACKGROUND;
			EP->transfer_params.data_left_length = 0;
			EP->transfer_params.data_transferred_length = 0;
		}
	}
}
#endif








static void USBD_set_address_ready (USBDC_params *usbdc, USBDEP_params *EP) {
#if ((USBDE_ADDRESSED_SUPPORT == USBD_FEATURE_ON) && (USBD_MAX_NUM_EVENTS > 0))
	uint8_t counter;
	uint8_t events_counter;
	USBDE_params *USBDE;
#endif
	
	// ustawienie nowego adresu USBD
	USBD_STM32_REG->DADDR = USBD_DADDR_EF | USBD_received_address;

	USBD_status_masked = USBDST_ATTACHED | USBDST_POWERED | USBDST_DEFAULT | USBDST_ADDRESSED;
	USBD_status        = USBDST_ATTACHED | USBDST_POWERED | USBDST_DEFAULT | USBDST_ADDRESSED;
		
#if ((USBDE_ADDRESSED_SUPPORT == USBD_FEATURE_ON) && (USBD_MAX_NUM_EVENTS > 0))

#if(USBD_ZERO_CONFIG_SUPPORT == USBD_FEATURE_ON)
	if(USBD_configurations[0]) {
		counter=0;
	}
	else {
		counter=1;
	}
	for(; counter <= USBD_Device_Descriptor.bNumConfigurations; counter++) {
#else
	for(counter=1; counter <= USBD_Device_Descriptor.bNumConfigurations; counter++) {
#endif
		USBDE = USBD_configurations[counter]->events;
		for(events_counter = 0; events_counter < USBD_MAX_NUM_EVENTS; events_counter++) {
			if(USBDE[events_counter].active_events & USBDEM_ADDRESSED) {
				if(USBDE[events_counter].event) {
					(*USBDE[events_counter].event) (USBD_configurations[counter], USBDEF_ADDRESSED, &USBDE[events_counter]);
				}
			}
		}
	}
	
#endif
}








static void USBD_EP0_ENUMERATE (void) {
	uint8_t number;
	USBDR_params REQ;
	uint32_t ep_reg_mirror;
	uint8_t *descriptor;
#if (((USBDE_CONFIGURED_SUPPORT == USBD_FEATURE_ON) || (USBDE_UNCONFIGURED_SUPPORT == USBD_FEATURE_ON)) && (USBD_MAX_NUM_EVENTS > 0))
	uint8_t events_counter;
	USBDE_params *USBDE;
#endif
	USBDEP_params *EP = &USBD_EP_params[0];
	uint32_t *data = (uint32_t*)USBDEP_SB_get_rx_buffer_addr(0);


	// w tej i kilku kolejnych liniach pobierane sa
	// z bufora czesci skladowe rzadania (REQUEST)
	REQ.Request.WholeRequestValue = (uint16_t)(*data);
	data++;
	REQ.wValue = (uint16_t)(*data);
	data++;
	REQ.wIndex = (uint16_t)(*data);
	data++;
	REQ.wLength = (uint16_t)(*data);

	USBDEP_reg_modify(0, ep_reg_mirror, USBDEP_STM32_RX_VALID, USBDEP_STM32_TYPE | USBDEP_STM32_ADDR | USBDEP_STM32_KIND | USBDEP_STM32_STAT_RX, USBDEP_STM32_CTR_TX);

	// sprawdzamy, czy Request, ktory otrzymalismy jest typu Standard
	if(!(REQ.Request.RequestFields.bmRequestType & USBD_BMREQUESTTYPE_TYPE)) {

		switch (REQ.Request.WholeRequestValue) {
			//--------------------
			case USBD_SET_ADDRESS_ZERO:
			//--------------------
				
				USBD_received_address = REQ.wValue;
				
				USBD_send_status(USBD_set_address_ready, 0, pUSBDC_active);
				break;



			//--------------------------
			case USBD_SET_CONFIGURATION_ZERO:
			//--------------------------

				// sprawdzenie, czy otrzymany numer konfiguracji jest rozny od zera
				if(!(REQ.wValue & 0xFF)) {
					USBDC_active_number = 0;
					pUSBDC_active = 0;

					USBD_status_masked = USBDST_ATTACHED | USBDST_POWERED | USBDST_DEFAULT | USBDST_ADDRESSED;
					USBD_status        = USBDST_ATTACHED | USBDST_POWERED | USBDST_DEFAULT | USBDST_ADDRESSED;
				}
				else {
					// sprawdzenie, czy otrzymany numer konfiguracji jest mniejszy
					// od numeru konfiguracji o najwyzszym numerze
					if(USBD_Device_Descriptor.bNumConfigurations >= (REQ.wValue & 0xFF)) {
						// ustawienie nowego numeru konfiguracji
						USBDC_active_number = REQ.wValue & 0xFF;						

						// jezeli USBD byl juz w stanie Configure i prowadzil transmisje
						// to w tym momencie transmisja jest zatrzymywana
						if(pUSBDC_active) {
							for (number = 0; number < USBD_MAX_NUM_ENDPOINTS; number++) {
								if(USBD_EP_params[number].transfer_params.transfer_type & USBD_TRANSFER_PRESENT) {
									USBD_EP_params[number].transfer_params.transfer_state = USBD_TRANSFER_ABORTED;
									USBD_EP_params[number].transfer_params.transfer_type &= ~((uint8_t)USBD_TRANSFER_PRESENT);
									if(pUSBDC_active->ep_irq[number]) {
										pUSBDC_active->ep_irq[number](pUSBDC_active, &USBD_EP_params[number]);
									}
								}
							}
#if ((USBDE_UNCONFIGURED_SUPPORT == USBD_FEATURE_ON) && (USBD_MAX_NUM_EVENTS > 0))
							events_counter = 0;
							do {
								USBDE = &(pUSBDC_active->events[events_counter]);
								if(USBDE->active_events & USBDEM_UNCONFIGURED) {
									if(USBDE->event) {
										(*USBDE->event) (pUSBDC_active, USBDEF_UNCONFIGURED, USBDE);
									}
								}
								events_counter++;
							} while(events_counter < USBD_MAX_NUM_EVENTS);
#endif
							
							pUSBDC_active->usbd_status = &USBD_status_masked;
						}

						// przestawienie wskaznika USBD na nowa konfiguracje
						pUSBDC_active = USBD_configurations[USBDC_active_number];
						
						// i przestawienie wskaznika do statusu
						pUSBDC_active->usbd_status = &USBD_status;

						// ustawienie nowego statusu USBD
						USBD_status_masked = USBDST_ATTACHED | USBDST_POWERED | USBDST_DEFAULT | USBDST_ADDRESSED;
						USBD_status        = USBDST_ATTACHED | USBDST_POWERED | USBDST_DEFAULT | USBDST_ADDRESSED | USBDST_CONFIGURED;

						// przekonfigurowanie USBD
						USBD_configure();

#if ((USBDE_CONFIGURED_SUPPORT == USBD_FEATURE_ON) && (USBD_MAX_NUM_EVENTS > 0))
						events_counter = 0;
						do {
							USBDE = &(pUSBDC_active->events[events_counter]);
							if(USBDE->active_events & USBDEM_CONFIGURED) {
								if(USBDE->event) {
									(*USBDE->event) (pUSBDC_active, USBDEF_CONFIGURED, USBDE);
								}
							}
							events_counter++;
						} while(events_counter < USBD_MAX_NUM_EVENTS);
#endif
					}
				}
				USBD_send_status(0, 0, pUSBDC_active);

				break;

			//--------------------------
			case USBD_GET_CONFIGURATION_ZERO:
			//--------------------------
				USBD_send_data_in (&USBDC_active_number, 1, 0, pUSBDC_active);
				break;



			//---------------------------
			case USBD_SET_INTERFACE_INTERFACE:
			//---------------------------
#if(USBD_ALTERNATE_SETTING_SUPPORT == USBD_FEATURE_ON)
				// sprawdzenie, czy Device jest w stanie Configured
				if(pUSBDC_active) {
					descriptor = pUSBDC_active->configuration_descriptor;
					// sprawdzenie, czy numer interfejsu odebrany od Hosta
					// jest mniejszy niz liczba interfejsow w danej konfiguracji
					if(descriptor[4] > REQ.wIndex) {
						// sprawdzanie, czy alternatywny interfejs odeslany
						// przez Host jest mniejszy od maksymalnego
						// alternatywnego ustawienia dla danego interfejsu
						if(pUSBDC_active->interfaces[REQ.wIndex].max_alternate_setting >= REQ.wValue) {

							USBD_reconfigure (REQ.wIndex, pUSBDC_active->interfaces[REQ.wIndex].descriptor->bAlternateSetting, REQ.wValue);

							USBD_send_status(0, 0, pUSBDC_active);	//wyslanie potwierdzenia

							if(pUSBDC_active) {
								number = USBD_EP_params[REQ.wIndex & 0xFF].interface;
								if(pUSBDC_active->ep0_setup_interface_irq[number]) {
									pUSBDC_active->ep0_setup_interface_irq[number] (pUSBDC_active, EP, REQ.wIndex, &REQ);
								}
							}
						}
					}
				}
#else
				USBD_send_stall(0, pUSBDC_active);
#endif
				break;

			//---------------------------
			case USBD_GET_INTERFACE_INTERFACE:
			//---------------------------
				descriptor = pUSBDC_active->configuration_descriptor;
				// sprawdzenie, czy numer interfejsu nie
				// jest wiekszy niz liczba interfejsow w danej konfiguracji
				if(descriptor[4] > REQ.wIndex) {
#if(USBD_ALTERNATE_SETTING_SUPPORT == USBD_FEATURE_ON)
					// wyslanie numeru alternatywnego ustawienia danego interfejsu
					USBD_send_data_in (&pUSBDC_active->interfaces[REQ.wIndex].descriptor->bAlternateSetting, 1, 0, pUSBDC_active);
#else
					USBD_send_data_in ((uint8_t*) &USBD_zero_const[0], 1, 0, pUSBDC_active);
#endif
				}
				break;



			//-------------------
			case USBD_GET_STATUS_ZERO:
			//-------------------
				USBD_send_data_in ((uint8_t*) &USBD_device_status, 2, 0, pUSBDC_active);
				break;

			//------------------------
			case USBD_GET_STATUS_INTERFACE:
			//------------------------
				USBD_send_data_in ((uint8_t*) &USBD_interface_status[0], 2, 0, pUSBDC_active);
				break;

			//-----------------------
			case USBD_GET_STATUS_ENDPOINT:
			//-----------------------
				number = REQ.wIndex & 0xF;
				if(number < USBD_MAX_NUM_ENDPOINTS) {
					USBD_send_data_in ((uint8_t*) &USBD_EP_params[number].status, 2, 0, pUSBDC_active);
				}
				break;



			//--------------------
			case USBD_SET_FEATURE_ZERO:
			//--------------------
				// brak wsparcia sprzetowego dla SET_FEATURE_ZERO REQUEST
				USBD_send_stall(0, pUSBDC_active);
				break;

			//-------------------------
			case USBD_SET_FEATURE_INTERFACE:
			//-------------------------
				USBD_send_status(0, 0, pUSBDC_active);
				break;

			//-------------------------
			case USBD_SET_FEATURE_ENDPOINT :
			//-------------------------
				number = REQ.wIndex & 0xF;
				if(number < USBD_MAX_NUM_ENDPOINTS) {
					// sprawdzamy, czy wyzsza warstwa sama chce
					// obsluzyc to zadanie, czy zostawia to nam
					if(!(pUSBDC_active->ep_halt_mask & (uint16_t)(0x1 << number))) {
						USBD_halt(number, pUSBDC_active);
						USBD_send_status(0, 0, pUSBDC_active);
					}
					else {
						number = USBD_EP_params[number].interface;

						// wywolanie rozszerzonej funkcji enumeracji
						// przypisanej danemu interfejsowi
						if(number < USBD_MAX_NUM_INTERFACES) {
							if(pUSBDC_active->ep0_setup_interface_irq[number]) {
								pUSBDC_active->ep0_setup_interface_irq[number] (pUSBDC_active, EP, number, &REQ);
							}
						}
					}
				}
				break;



			//----------------------
			case USBD_CLEAR_FEATURE_ZERO:
			//----------------------
				// brak wsparcia sprzetowego dla CLEAR_FEATURE_ZERO REQUEST
				USBD_send_stall(0, pUSBDC_active);
				break;

			//---------------------------
			case USBD_CLEAR_FEATURE_INTERFACE:
			//---------------------------
				USBD_send_status(0, 0, pUSBDC_active);
				break;

			//---------------------------
			case USBD_CLEAR_FEATURE_ENDPOINT :
			//---------------------------
				number = REQ.wIndex & 0xF;
				if(number < USBD_MAX_NUM_ENDPOINTS) {
					// sprawdzamy, czy wyzsza warstwa sama chce
					// obsluzyc to zadanie, czy zostawia to nam
					if(!(pUSBDC_active->ep_halt_mask & (uint16_t)(0x1 << number))) {
						USBD_unhalt(number, pUSBDC_active);
						USBD_send_status(0, 0, pUSBDC_active);
					}
					else {
						number = USBD_EP_params[number].interface;

						// wywolanie rozszerzonej funkcji enumeracji
						// przypisanej danemu interfejsowi
						if(number < USBD_MAX_NUM_INTERFACES) {
							if(pUSBDC_active->ep0_setup_interface_irq[number]) {
								pUSBDC_active->ep0_setup_interface_irq[number] (pUSBDC_active, EP, number, &REQ);
							}
						}
					}
				}
				break;



			//-------------------------------------------------------------------------------------------------------
			//USBD_SYNCH_FRAME_ENDPOINT w tej wersji usbd.c obslugiwane w rozszerzonej enumeracji jezeli jest taka potrzeba
			//-------------------------------------------------------------------------------------------------------






			//-----------------------
			case USBD_GET_DESCRIPTOR_ZERO:
			//-----------------------

				switch (((REQ.wValue & 0xFF00) >> 8)) {
					//----------
					case USBD_DESC_TYPE_DEVICE:
					//----------
						if(REQ.wLength >= USBD_Device_Descriptor.bLength) {
							USBD_send_data_in((uint8_t*) &USBD_Device_Descriptor, sizeof(USBD_device_descriptor), 0, pUSBDC_active);
						}
						else {
							USBD_send_data_in((uint8_t*) &USBD_Device_Descriptor, REQ.wLength, 0, pUSBDC_active);
						}
						break;

					//-----------------
					case USBD_DESC_TYPE_CONFIGURATION:
					//-----------------
						if((REQ.wValue & 0x0FF) <= USBD_Device_Descriptor.bNumConfigurations) {
							descriptor = (USBD_configurations[(REQ.wValue & 0x0FF) + 1])->configuration_descriptor;
							if(REQ.wLength < (((uint16_t)descriptor[2]) | (((uint16_t)descriptor[3]) << 8))) {
								USBD_send_data_in((uint8_t*) descriptor, REQ.wLength, 0, pUSBDC_active);
							}
							else {
								USBD_send_data_in((uint8_t*) descriptor, descriptor[2], 0, pUSBDC_active);
							}
						}
						break;

					//----------
					case USBD_DESC_TYPE_STRING:
					//----------
						number = (uint8_t) (REQ.wValue & 0x0FF);
						if(USBD_private_data.installed_strings_count > number) {
							if(REQ.wLength < USBD_private_data.strings[number].string[0]) {
								USBD_send_data_in((uint8_t*) USBD_private_data.strings[number].string, REQ.wLength, 0, pUSBDC_active);
							}
							else {
								USBD_send_data_in((uint8_t*) USBD_private_data.strings[number].string, USBD_private_data.strings[number].string[0], 0, pUSBDC_active);
							}
						}
						else {
							USBD_send_stall(0, pUSBDC_active);
						}
						break;

					//-------
					default :
					//-------
						if(pUSBDC_active) {
							if((REQ.Request.RequestFields.bmRequestType & USBD_BMREQUESTTYPE_DESTINATION) == USBD_BMREQUESTTYPE_INTERFACE) {
								number = REQ.wIndex & 0xFF;
								if(number < USBD_MAX_NUM_INTERFACES) {
									if(pUSBDC_active->ep0_setup_interface_irq[number]) {
										pUSBDC_active->ep0_setup_interface_irq[number] (pUSBDC_active, EP, number, &REQ);
									}
								}
							}
							else {
								if((REQ.Request.RequestFields.bmRequestType & USBD_BMREQUESTTYPE_DESTINATION) == USBD_BMREQUESTTYPE_ENDPOINT) {
									if((REQ.wIndex & 0xF) < USBD_MAX_NUM_ENDPOINTS) {
										number = USBD_EP_params[REQ.wIndex & 0xF].interface;
										if(number < USBD_MAX_NUM_INTERFACES) {
											if(pUSBDC_active->ep0_setup_interface_irq[number]) {
												pUSBDC_active->ep0_setup_interface_irq[number] (pUSBDC_active, EP, number, &REQ);
											}
										}
									}
								}
							}
						}
						break;
				}
				break;




			//-------
			default:
			//-------
				if(pUSBDC_active) {
					if((REQ.Request.RequestFields.bmRequestType & USBD_BMREQUESTTYPE_DESTINATION) == USBD_BMREQUESTTYPE_INTERFACE) {
						number = REQ.wIndex & 0xFF;
						if(number < USBD_MAX_NUM_INTERFACES) {
							if(pUSBDC_active->ep0_setup_interface_irq[number]) {
								pUSBDC_active->ep0_setup_interface_irq[number] (pUSBDC_active, EP, number, &REQ);
							}
						}
					}
					else {
						if((REQ.Request.RequestFields.bmRequestType & USBD_BMREQUESTTYPE_DESTINATION) == USBD_BMREQUESTTYPE_ENDPOINT) {
							if((REQ.wIndex & 0xF) < USBD_MAX_NUM_ENDPOINTS) {
								number = USBD_EP_params[REQ.wIndex & 0xF].interface;
								if(number < USBD_MAX_NUM_INTERFACES) {
									if(pUSBDC_active->ep0_setup_interface_irq[number]) {
										pUSBDC_active->ep0_setup_interface_irq[number] (pUSBDC_active, EP, number, &REQ);
									}
								}
							}
						}
					}
				}
				break;
		}
	}
	else {
		if(pUSBDC_active) {
			// sprawdzamy, czy Request, ktory przyszedl jest typu Class
			if((REQ.Request.RequestFields.bmRequestType & USBD_BMREQUESTTYPE_TYPE) == USBD_BMREQUESTTYPE_CLASS) {
				number = REQ.wIndex & 0xFF;
				if(number < USBD_MAX_NUM_INTERFACES) {
					if(pUSBDC_active->ep0_setup_interface_irq[number]) {
						pUSBDC_active->ep0_setup_interface_irq[number] (pUSBDC_active, EP, number, &REQ);
					}
				}
			}
#if(USBD_VENDOR_SUPPORT == USBD_FEATURE_ON)
			// odebrany Request jest typu Vendor
			else {
				if(pUSBDC_active->ep0_setup_vendor_irq) {
					pUSBDC_active->ep0_setup_vendor_irq (pUSBDC_active, EP, &REQ);
				}
			}
#endif
		}
	}
}








static void USBD_EP_OUT_SB_irq (uint8_t ep_number) {
	uint32_t ep_reg_mirror;
	USBDEP_params *EP = &USBD_EP_params[ep_number];
	
	EP->transfer_params.transfer_state = USBD_IDLE_OR_BACKGROUND;

	// sprawdzenie czy nadszedl SETUP czy zwykle dane
	if(!(USBD_STM32_REG->EP_REG[ep_number] & USBDEP_STM32_SETUP)) {
		
		// skasowanie flagi przerwania RX
		USBDEP_reg_modify(ep_number, ep_reg_mirror, 0, USBDEP_STM32_TYPE | USBDEP_STM32_ADDR | USBDEP_STM32_KIND, USBDEP_STM32_CTR_TX);

		// sprawdzenie czy prowadzimy transmisje typu CONTROL_OUT lub DATA_OUT
		if((EP->transfer_params.transfer_type == (USBD_CONTROL_OUT | USBD_TRANSFER_PRESENT))
				|| (EP->transfer_params.transfer_type == (USBD_DATA_OUT | USBD_TRANSFER_PRESENT))){
			USBDEP_read_next_OUT_packet(EP);
		}
		else {
			// sprawdzenie czy przypadkiem odebrany pakiet danych to nie STATUS_OUT
			if(((USBD_STM32_REG->EP_REG[ep_number] & USBDEP_STM32_TYPE) == USBDEP_STM32_CONTROL)
					&& !(USBDEP_SB_get_rx_count(ep_number))) {
				// zakonczenie wszelkiej transmisji na tym Endpoincie
				USBDEP_reg_modify(ep_number, ep_reg_mirror, USBDEP_STM32_RX_VALID | USBDEP_STM32_TX_NAK, USBDEP_STM32_ADDR | USBDEP_STM32_KIND | USBDEP_STM32_TYPE | USBDEP_STM32_STAT_RX | USBDEP_STM32_STAT_TX, 0);

				USBDEP_release_fifo_IN(ep_number);

				EP->transfer_params.data = 0;
				EP->transfer_params.transfer_state = USBD_STATUS_STAGE;
				EP->transfer_params.data_buffered_length = 0;
				EP->transfer_params.data_left_length = 0;

				// jezeli podczas transmisji kontrolnej odebrany zostanie STATUS_OUT
				// nie wazne, czy byla to transmisja typu IN, czy OUT wowczas
				// oznacza to, ze Host przerwal transmisje
				// dlatego handler endpointu wywolujemy tylko w celu poinformowania
				// uzytkownika o tym fakcie. Nie powinien on zlecac zadnej innej transmisji
				// na tym Endpoincie bo Host nie chce tego. Z tego powodu
				// po wywolaniu handlera transmisja wszystkie parametry transmisji
				// sa kasowane bez sprawdzania niczego. Sytuacja zas taka, ze
				// otrzymano zerowy pakiet transmisji CONTROL_OUT jako zakonczenie
				// pakietu danych nie wystepuje tu, tylko w warunku wczesniejszym.
				if(pUSBDC_active) {
					if(pUSBDC_active->ep_irq[ep_number]) {
						pUSBDC_active->ep_irq[ep_number] (pUSBDC_active, EP);
					}
				}
				EP->transfer_params.transfer_type = USBD_IDLE;
				EP->transfer_params.transfer_state = USBD_IDLE_OR_BACKGROUND;
				EP->transfer_params.data = 0;
				EP->transfer_params.data_left_length = 0;
				EP->transfer_params.data_buffered_length = 0;
				EP->transfer_params.data_transferred_length = 0;
			}
			// zaden z wymienionych wyzej przypadkow - wywolujemy funkcje wyzszej warstwy API - niech sie martwi:D
			else {
				if(pUSBDC_active) {
					if(pUSBDC_active->ep_irq[ep_number]) {
						// uaktualnienie parametrow transmisji
						EP->transfer_params.data = 0;
						EP->transfer_params.data_left_length = 0;
						EP->transfer_params.data_buffered_length = USBDEP_SB_get_rx_count(ep_number);
						EP->transfer_params.data_transferred_length = 0;
						EP->transfer_params.transfer_state = USBD_TOO_LESS_SPACE;
						EP->transfer_params.transfer_type = USBD_DATA_OUT | USBD_TRANSFER_IS_LOCKED;

						pUSBDC_active->ep_irq[ep_number] (pUSBDC_active, EP);
					}
					else {
						// ustawienie nowego stanu transmisji
						EP->transfer_params.transfer_state = USBD_CLEAR_ALL_FLAGS;
					}
				}
				else {
					// ustawienie nowego stanu transmisji
					EP->transfer_params.transfer_state = USBD_CLEAR_ALL_FLAGS;
				}
			}
		}
	}
	// nadszedl SETUP
	else {
		// skasowanie flag przerwania
		USBDEP_reg_modify(ep_number, ep_reg_mirror, USBDEP_STM32_RX_NAK | USBDEP_STM32_TX_NAK, USBDEP_STM32_ADDR | USBDEP_STM32_KIND | USBDEP_STM32_TYPE | USBDEP_STM32_STAT_TX | USBDEP_STM32_STAT_RX, 0);

		// ustawienie stanu transmisji
		EP->transfer_params.transfer_type = USBD_CONTROL_OUT;
		
		if(!ep_number) {
			// wyzeworanie wskaznika do funkcji obslugi Endpointa 0
			if(pUSBDC_active) {
				pUSBDC_active->ep_irq[0] = 0;
			}
			USBD_EP0_ENUMERATE();
		}
		else {
			if(pUSBDC_active) {
				if(pUSBDC_active->ep_irq[ep_number]) {
					pUSBDC_active->ep_irq[ep_number] (pUSBDC_active, EP);
				}
				else {
					// ustawienie stanu transmisji
					EP->transfer_params.transfer_state = USBD_CLEAR_ALL_FLAGS;
				}
			}
			else {
				// ustawienie statusu transmisji
				EP->transfer_params.transfer_state = USBD_CLEAR_ALL_FLAGS;
			}
		}

	}
	// sprawdzenie czy nie zostal ustawiony status nakazujacy skasowanie wszystkich flag
	if(EP->transfer_params.transfer_state == USBD_CLEAR_ALL_FLAGS) {
		if(USBDEP_is_fifo_OUT_data_available(ep_number)) {
			USBDEP_release_fifo_OUT(0, 0, ep_number);
		}

		//USBDEP_reg_modify(ep_number, ep_reg_mirror, USBDEP_STM32_RX_VALID, USBDEP_STM32_TYPE | USBDEP_STM32_ADDR | USBDEP_STM32_KIND | USBDEP_STM32_STAT_RX, USBDEP_STM32_CTR_TX);

		// uaktualnienie parametrow transmisji
		EP->transfer_params.data = 0;
		EP->transfer_params.data_left_length = 0;
		EP->transfer_params.data_buffered_length = 0;
		EP->transfer_params.data_transferred_length = 0;
		EP->transfer_params.transfer_type = USBD_IDLE;
		EP->transfer_params.transfer_state = USBD_IDLE_OR_BACKGROUND;

		if(pUSBDC_active) {
			if(pUSBDC_active->ep_irq[ep_number]) {
				pUSBDC_active->ep_irq[ep_number] (pUSBDC_active, EP);
			}
		}
	}
	else if(EP->transfer_params.transfer_state != USBD_STATUS_STAGE) {
		EP->transfer_params.transfer_state = USBD_IDLE_OR_BACKGROUND;
	}
}








static void USBD_EP_IN_SB_irq (uint8_t ep_number) {
	uint32_t ep_reg_mirror;
	USBDEP_params *EP = &USBD_EP_params[ep_number];

	// skasowanie flagi przerwania TX
	USBDEP_reg_modify(ep_number, ep_reg_mirror, 0, USBDEP_STM32_TYPE | USBDEP_STM32_ADDR | USBDEP_STM32_KIND, USBDEP_STM32_CTR_RX);

	// sprawdzenie, prowadzimy transmisje typu DATA_IN
	if(EP->transfer_params.transfer_type == (USBD_DATA_IN | USBD_TRANSFER_PRESENT)) {
		EP->transfer_params.transfer_state = USBD_IDLE_OR_BACKGROUND;
		// wywolanie funkcji odpowiedzialnej za wyslanie danych
		USBD_write_next_IN_packet_single_buffer(EP);
	}
	else {
		// sprawdzenie, prowadzimy transmisje typu DATA_IN
		if(EP->transfer_params.transfer_type == (USBD_CONTROL_IN | USBD_TRANSFER_PRESENT)) {
			if(EP->transfer_params.transfer_state != USBD_STATUS_STAGE) {
				EP->transfer_params.transfer_state = USBD_IDLE_OR_BACKGROUND;
				// wywolanie funkcji odpowiedzialnej za wyslanie danych
				USBD_write_next_IN_packet_single_buffer(EP);
			}
			else {
				EP->transfer_params.data = 0;
				EP->transfer_params.data_buffered_length = 0;
				EP->transfer_params.data_left_length = 0;

				if(USBD_status_in_complete_irq) {
					(*USBD_status_in_complete_irq) (pUSBDC_active, EP);
					USBD_status_in_complete_irq = 0;
				}
				EP->transfer_params.transfer_type = USBD_IDLE;
				EP->transfer_params.transfer_state = USBD_IDLE_OR_BACKGROUND;
			}
		}
		else {
			if(pUSBDC_active) {
				if(pUSBDC_active->ep_irq[ep_number]) {
					pUSBDC_active->ep_irq[ep_number] (pUSBDC_active, EP);
				}
				else {
					// ustawienie nowego stanu transmisji
					EP->transfer_params.transfer_state = USBD_CLEAR_ALL_FLAGS;
				}
			}
			else {
				// ustawienie nowego stanu transmisji
				EP->transfer_params.transfer_state = USBD_CLEAR_ALL_FLAGS;
			}
		}
	}
	
	// sprawdzenie, czy mamy skasowac flagi przerwan
	if(EP->transfer_params.transfer_state  == USBD_CLEAR_ALL_FLAGS) {
		// skasowania flagi przerwania TX tu nie dokonujemy
		// gdyz zostalo to przeprowadzone na poczatku tej funkcji

		// uaktualnienie parametrow transmisji
		EP->transfer_params.data = 0;
		EP->transfer_params.data_left_length = 0;
		EP->transfer_params.data_buffered_length = 0;
		EP->transfer_params.data_transferred_length = 0;
		EP->transfer_params.transfer_type = USBD_IDLE;
		EP->transfer_params.transfer_state = USBD_IDLE_OR_BACKGROUND;

		if(pUSBDC_active) {
			if(pUSBDC_active->ep_irq[ep_number]) {
				pUSBDC_active->ep_irq[ep_number] (pUSBDC_active, EP);
			}
		}
	}
	else if(EP->transfer_params.transfer_state != USBD_STATUS_STAGE) {
		EP->transfer_params.transfer_state = USBD_IDLE_OR_BACKGROUND;
	}
}








#if((USBDEP_ISOCHRONOUS_SUPPORT == USBD_FEATURE_ON) && (USBD_MAX_NUM_ENDPOINTS > 1))
static void USBDEP_perform_isochronous_out(uint8_t ep_number) {
	uint32_t ep_reg_mirror;
	USBDEP_params *EP = &USBD_EP_params[ep_number];

	EP->transfer_params.transfer_state = USBD_IDLE_OR_BACKGROUND;

	// skasowanie flagi przerwania RX
	USBDEP_reg_modify(ep_number, ep_reg_mirror, 0, USBDEP_STM32_TYPE | USBDEP_STM32_ADDR | USBDEP_STM32_KIND, 0);

	// zaznaczenie, ze mamy dane oderane w buforze
	USBDEP_private[ep_number].fifo_counter = 1;

	// sprawdzenie czy mamy zadeklarowana przestrzen na dane i czy prowadzimy transmisje typu OUT
	if(EP->transfer_params.transfer_type == (USBD_DATA_OUT | USBD_TRANSFER_PRESENT)) {
		// programowy odbior danych
		if(!(USBD_STM32_REG->ISTR & (USBD_ISTR_ERR | USBD_ISTR_PMAOVR))) {
			USBDEP_read_next_OUT_packet(EP);
		}
		else {
			USBD_STM32_REG->ISTR &= ~((uint32_t)(USBD_ISTR_PMAOVR | USBD_ISTR_ERR));

			if(pUSBDC_active) {
				if(pUSBDC_active->ep_irq[ep_number]) {
					EP->transfer_params.transfer_state = USBD_PACKET_ERROR;
					EP->transfer_params.transfer_type |= USBD_TRANSFER_DATA_CORRUPTION;
					pUSBDC_active->ep_irq[ep_number] (pUSBDC_active, EP);
				}
			}
		}
	}
	else {
		if(pUSBDC_active) {
			if(pUSBDC_active->ep_irq[ep_number]) {
				// uaktualnienie parametrow transmisji
				EP->transfer_params.data = 0;
				EP->transfer_params.data_left_length = 0;
				EP->transfer_params.data_buffered_length = USBDEP_ISO_get_rx_count(ep_number);
				EP->transfer_params.data_transferred_length = 0;
				EP->transfer_params.transfer_type = USBD_DATA_OUT;
				if(USBD_STM32_REG->ISTR & (USBD_ISTR_ERR | USBD_ISTR_PMAOVR)) {
					USBD_STM32_REG->ISTR &= ~((uint32_t)(USBD_ISTR_PMAOVR | USBD_ISTR_ERR));
					EP->transfer_params.transfer_state = USBD_PACKET_ERROR;
				}

				pUSBDC_active->ep_irq[ep_number] (pUSBDC_active, EP);
			}
			else {
				// ustawienie nowego stanu transmisji
				EP->transfer_params.transfer_state = USBD_CLEAR_ALL_FLAGS;
			}
		}
		else {
			// ustawienie nowego stanu transmisji
			EP->transfer_params.transfer_state = USBD_CLEAR_ALL_FLAGS;
		}
	}
	// sprawdzenie czy nie zostal ustawiony status nakazujacy skasowanie wszystkich flag
	if(EP->transfer_params.transfer_state  == USBD_CLEAR_ALL_FLAGS) {

		// uaktualnienie parametrow transmisji
		EP->transfer_params.data = 0;
		EP->transfer_params.data_left_length = 0;
		EP->transfer_params.data_buffered_length = 0;
		EP->transfer_params.data_transferred_length = 0;
		EP->transfer_params.transfer_type = USBD_IDLE;

		if(pUSBDC_active) {
			if(pUSBDC_active->ep_irq[ep_number]) {
				pUSBDC_active->ep_irq[ep_number] (pUSBDC_active, EP);
			}
		}
	}
}



static void USBDEP_perform_isochronous_in(uint8_t ep_number) {
	uint32_t ep_reg_mirror;
	USBDEP_params *EP = &USBD_EP_params[ep_number];

	EP->transfer_params.transfer_state = USBD_IDLE_OR_BACKGROUND;

	// skasowanie flagi przerwania TX
	USBDEP_reg_modify(ep_number, ep_reg_mirror, 0, USBDEP_STM32_TYPE | USBDEP_STM32_ADDR | USBDEP_STM32_KIND, 0);

	// sprawdzenie, czy mamy zdeklarowane dane w buforze i czy sa to dane do wyslania
	if(EP->transfer_params.transfer_type == (USBD_DATA_IN | USBD_TRANSFER_PRESENT)) {
		// wywolanie funkcji odpowiedzialnej za wyslanie danych
		USBD_write_next_IN_ISO_packet(EP);
	}
	else {
		// informujemy sprzet o tym, ze wysylamy dane o dlugosci 0
		// jezeli wyzsza warstwa nie zadeklaruje danych, wowczas wyslany
		// zostanie pakiet o dlugosci 0
		USBDEP_ISO_set_tx_count(ep_number, 0);

		if(pUSBDC_active) {
			if(pUSBDC_active->ep_irq[ep_number]) {

				// wyczyszczenie parametrow transmisji
				EP->transfer_params.data = 0;
				EP->transfer_params.data_left_length = 0;
				EP->transfer_params.data_buffered_length = 0;
				EP->transfer_params.data_transferred_length = 0;
				EP->transfer_params.transfer_type = USBD_DATA_IN;

				pUSBDC_active->ep_irq[ep_number] (pUSBDC_active, EP);
			}
			else {
				// ustawienie nowego stanu transmisji
				EP->transfer_params.transfer_state = USBD_CLEAR_ALL_FLAGS;
			}
		}
		else {
			// ustawienie nowego stanu transmisji
			EP->transfer_params.transfer_state = USBD_CLEAR_ALL_FLAGS;
		}
	}

	// sprawdzenie, czy mamy skasowac flagi przerwan
	if(EP->transfer_params.transfer_state == USBD_CLEAR_ALL_FLAGS) {

		// uaktualnienie parametrow transmisji
		EP->transfer_params.data = 0;
		EP->transfer_params.data_left_length = 0;
		EP->transfer_params.data_buffered_length = 0;
		EP->transfer_params.data_transferred_length = 0;
		EP->transfer_params.transfer_type = USBD_IDLE;

		if(pUSBDC_active) {
			if(pUSBDC_active->ep_irq[ep_number]) {
				pUSBDC_active->ep_irq[ep_number] (pUSBDC_active, EP);
			}
		}
	}
}
#endif







#if((USBDEP_BULK_SUPPORT == USBD_FEATURE_ON) && (USBD_MAX_NUM_ENDPOINTS > 1))
static void USBDEP_perform_bulk_out(uint8_t ep_number) {
	uint32_t ep_reg_mirror;
	USBDEP_params *EP = &USBD_EP_params[ep_number];

	EP->transfer_params.transfer_state = USBD_IDLE_OR_BACKGROUND;

	// skasowanie flagi przerwania RX
	USBDEP_reg_modify(ep_number, ep_reg_mirror, 0, USBDEP_STM32_TYPE | USBDEP_STM32_ADDR | USBDEP_STM32_KIND, 0);

	// sprawdzenie czy mamy zadeklarowana przestrzen na dane i czy prowadzimy transmisje typu OUT
	if(EP->transfer_params.transfer_type == (USBD_DATA_OUT | USBD_TRANSFER_PRESENT)) {
		// programowy odbior danych
// TODO: 		if(!wystapily bledy transmisji)
		USBDEP_read_next_OUT_packet(EP);
// TODO:		else {wywolaj ep_irq z parametrem "transfer error" i jezeli nie zostanie zmieniony na USBD_CLEAR_ALL_FLAGS to odbierz mimo wszystko dane}
	}
	else {
		if(pUSBDC_active) {
			if(pUSBDC_active->ep_irq[ep_number]) {
				// uaktualnienie parametrow transmisji
				EP->transfer_params.data = 0;
				EP->transfer_params.data_left_length = 0;
				EP->transfer_params.data_buffered_length = USBDEP_DB_get_rx_count_HB(ep_number);
				EP->transfer_params.data_transferred_length = 0;
				EP->transfer_params.transfer_type = USBD_DATA_OUT;
// TODO: 				if(wystapily bledy transmisji) ustaw status na "transfer error"

				pUSBDC_active->ep_irq[ep_number] (pUSBDC_active, EP);
			}
			else {
				// ustawienie nowego stanu transmisji
				EP->transfer_params.transfer_state = USBD_CLEAR_ALL_FLAGS;
			}
		}
		else {
			// ustawienie nowego stanu transmisji
			EP->transfer_params.transfer_state = USBD_CLEAR_ALL_FLAGS;
		}
	}
	// sprawdzenie czy nie zostal ustawiony status nakazujacy skasowanie wszystkich flag
	if(EP->transfer_params.transfer_state  == USBD_CLEAR_ALL_FLAGS) {
		if(USBDEP_is_fifo_OUT_data_available(ep_number)) {
			USBDEP_release_fifo_OUT(0, 0, ep_number);
		}

		// zezwolenie na odbior do oczekujacego bufora
//				USBDEP_reg_modify(ep_number, ep_reg_mirror, USBDEP_STM32_RX_VALID, USBDEP_STM32_TYPE | USBDEP_STM32_ADDR | USBDEP_STM32_KIND | USBDEP_STM32_STAT_RX, USBDEP_STM32_DTOG_TX);

		// uaktualnienie parametrow transmisji
		EP->transfer_params.data = 0;
		EP->transfer_params.data_left_length = 0;
		EP->transfer_params.data_buffered_length = 0;
		EP->transfer_params.data_transferred_length = 0;
		EP->transfer_params.transfer_type = USBD_IDLE;

		if(pUSBDC_active) {
			if(pUSBDC_active->ep_irq[ep_number]) {
				pUSBDC_active->ep_irq[ep_number] (pUSBDC_active, EP);
			}
		}
	}
}


static void USBDEP_perform_bulk_in(uint8_t ep_number) {
	uint32_t ep_reg_mirror;
	USBDEP_params *EP = &USBD_EP_params[ep_number];

	EP->transfer_params.transfer_state = USBD_IDLE_OR_BACKGROUND;

	// skasowanie flagi przerwania TX
	USBDEP_reg_modify(ep_number, ep_reg_mirror, 0, USBDEP_STM32_TYPE | USBDEP_STM32_ADDR | USBDEP_STM32_KIND, 0);

	// sprawdzenie, czy mamy zdeklarowane dane w buforze i czy sa to dane do wyslania
	if(EP->transfer_params.transfer_type == (USBD_DATA_IN | USBD_TRANSFER_PRESENT)) {
		// wywolanie funkcji odpowiedzialnej za wyslanie danych
		USBD_write_next_IN_packet_dual_buffer(EP);
	}
	else {
		if(pUSBDC_active) {
			if(pUSBDC_active->ep_irq[ep_number]) {

				// wyczyszczenie parametrow transmisji
				EP->transfer_params.data = 0;
				EP->transfer_params.data_left_length = 0;
				EP->transfer_params.data_buffered_length = 0;
				EP->transfer_params.data_transferred_length = 0;
				EP->transfer_params.transfer_type = USBD_DATA_IN;

				pUSBDC_active->ep_irq[ep_number] (pUSBDC_active, EP);
			}
			else {
				// ustawienie nowego stanu transmisji
				EP->transfer_params.transfer_state = USBD_CLEAR_ALL_FLAGS;
			}
		}
		else {
			// ustawienie nowego stanu transmisji
			EP->transfer_params.transfer_state = USBD_CLEAR_ALL_FLAGS;
		}
	}

	// sprawdzenie, czy mamy skasowac flagi przerwan
	if(EP->transfer_params.transfer_state == USBD_CLEAR_ALL_FLAGS) {
		// jezeli jest nadal kolizja buforow wowczas zwalniamy bufor
		if((USBD_STM32_REG->EP_REG[ep_number] & USBDEP_STM32_DTOG_TX) == ((USBD_STM32_REG->EP_REG[ep_number] & USBDEP_STM32_DTOG_RX) >> 8)) {
			// zezwolenie na wyslanie oczekujacego bufora
			USBDEP_reg_modify(ep_number, ep_reg_mirror, USBDEP_STM32_TX_VALID, USBDEP_STM32_TYPE | USBDEP_STM32_ADDR | USBDEP_STM32_KIND | USBDEP_STM32_STAT_TX, USBDEP_STM32_DTOG_RX);
		}


		// uaktualnienie parametrow transmisji
		EP->transfer_params.data = 0;
		EP->transfer_params.data_left_length = 0;
		EP->transfer_params.data_buffered_length = 0;
		EP->transfer_params.data_transferred_length = 0;
		EP->transfer_params.transfer_type = USBD_IDLE;

		if(pUSBDC_active) {
			if(pUSBDC_active->ep_irq[ep_number]) {
				pUSBDC_active->ep_irq[ep_number] (pUSBDC_active, EP);
			}
		}
	}
}


static void USBDEP_perform_bulk_out_start_second_step(uint8_t ep_number, USBDEP_params *EP) {
	uint32_t ep_reg_mirror;

	// skasowanie flagi przerwania RX
	USBDEP_reg_modify(ep_number, ep_reg_mirror, 0, USBDEP_STM32_TYPE | USBDEP_STM32_ADDR | USBDEP_STM32_KIND, 0);

	if(USBDEP_private[ep_number].fifo_counter != 2) {
		USBDEP_private[ep_number].fifo_counter = 1;
		if(EP->transfer_params.transfer_type & USBD_TRANSFER_IS_LOCKED) {
			return;
		}
	}
	else {
		USBDEP_hc_irq[ep_number] = USBDEP_perform_bulk_out;
	}

	// sprawdzenie czy mamy zadeklarowana przestrzen na dane i czy prowadzimy transmisje typu OUT
	if(EP->transfer_params.transfer_type == (USBD_DATA_OUT | USBD_TRANSFER_PRESENT)) {
		// programowy odbior danych
// TODO: 		if(!wystapily bledy transmisji)
		USBDEP_read_next_OUT_packet(EP);
// TODO:		else {wywolaj ep_irq z parametrem "transfer error" i jezeli nie zostanie zmieniony na USBD_CLEAR_ALL_FLAGS to odbierz mimo wszystko dane}
	}
	else {
		if(pUSBDC_active) {
			if(pUSBDC_active->ep_irq[ep_number]) {
				// uaktualnienie parametrow transmisji
				EP->transfer_params.data = 0;
				EP->transfer_params.data_left_length = 0;
				if(USBDEP_private[ep_number].fifo_counter == 2) {
					EP->transfer_params.data_buffered_length = USBDEP_DB_get_rx_count_HB(ep_number);
				}
				else {
					EP->transfer_params.data_buffered_length = USBDEP_DB_get_rx_count(ep_number);
				}
				EP->transfer_params.data_transferred_length = 0;
				EP->transfer_params.transfer_type = USBD_DATA_OUT;
// TODO: 				if(wystapily bledy transmisji) ustaw status na "transfer error"
				
				pUSBDC_active->ep_irq[ep_number] (pUSBDC_active, EP);
			}
			else {
				// ustawienie nowego stanu transmisji
				EP->transfer_params.transfer_state = USBD_CLEAR_ALL_FLAGS;
			}
		}
		else {
			// ustawienie nowego stanu transmisji
			EP->transfer_params.transfer_state = USBD_CLEAR_ALL_FLAGS;
		}
	}
	// sprawdzenie czy nie zostal ustawiony status nakazujacy skasowanie wszystkich flag
	if(EP->transfer_params.transfer_state  == USBD_CLEAR_ALL_FLAGS) {
		if(USBDEP_is_fifo_OUT_data_available(ep_number)) {
			USBDEP_release_fifo_OUT(0, 0, ep_number);
		}

		// zezwolenie na odbior do oczekujacego bufora
//				USBDEP_reg_modify(ep_number, ep_reg_mirror, USBDEP_STM32_RX_VALID, USBDEP_STM32_TYPE | USBDEP_STM32_ADDR | USBDEP_STM32_KIND | USBDEP_STM32_STAT_RX, USBDEP_STM32_DTOG_TX);

		// uaktualnienie parametrow transmisji
		EP->transfer_params.data = 0;
		EP->transfer_params.data_left_length = 0;
		EP->transfer_params.data_buffered_length = 0;
		EP->transfer_params.data_transferred_length = 0;
		EP->transfer_params.transfer_type = USBD_IDLE;

		if(pUSBDC_active) {
			if(pUSBDC_active->ep_irq[ep_number]) {
				pUSBDC_active->ep_irq[ep_number] (pUSBDC_active, EP);
			}
		}
	}
}





static void USBDEP_perform_bulk_out_start_first_step(uint8_t ep_number) {
	uint32_t ep_reg_mirror;
	USBDEP_params *EP = &USBD_EP_params[ep_number];

	USBDEP_HP_state = 1;

	// ustawienie nowego stanu transmisji
	EP->transfer_params.transfer_state = USBD_IDLE_OR_BACKGROUND;

	USBDEP_perform_bulk_out_start_second_step(ep_number, EP);
	if(!(EP->transfer_params.transfer_type & USBD_TRANSFER_IS_LOCKED)) {
		if(!(USBD_STM32_REG->EP_REG[ep_number] & USBDEP_STM32_CTR_RX)) {
			if((USBD_STM32_REG->EP_REG[ep_number] & USBDEP_STM32_DTOG_TX) == ((USBD_STM32_REG->EP_REG[ep_number] & USBDEP_STM32_DTOG_RX) >> 8)) {
				EP->transfer_params.transfer_state = USBD_IDLE_OR_BACKGROUND;

				USBDEP_perform_bulk_out_start_second_step(ep_number, EP);
			}
		}
		else if(USBDEP_private[ep_number].fifo_counter == 1) {
			// skasowanie flagi przerwania RX
			USBDEP_reg_modify(ep_number, ep_reg_mirror, 0, USBDEP_STM32_TYPE | USBDEP_STM32_ADDR | USBDEP_STM32_KIND, 0);
		}
	}


	EP->transfer_params.transfer_state = USBD_IDLE_OR_BACKGROUND;

	USBDEP_HP_state = 0;

}
#endif








void USB_HP_CAN1_TX_IRQHandler(void) {
#if(((USBDEP_BULK_SUPPORT == USBD_FEATURE_ON) || (USBDEP_ISOCHRONOUS_SUPPORT == USBD_FEATURE_ON)) && (USBD_MAX_NUM_ENDPOINTS > 1))
	uint8_t ep_number;
#endif


	USBD_ENTER_IRQ(USBD_STM32_IRQ_CHANELL_HP);


#if(((USBDEP_BULK_SUPPORT == USBD_FEATURE_ON) || (USBDEP_ISOCHRONOUS_SUPPORT == USBD_FEATURE_ON)) && (USBD_MAX_NUM_ENDPOINTS > 1))
	ep_number = 1;
	do {
		// sprawdzenie, czy przerwanie dotyczny transmisji danych i czy odnosi sie do danego endpointa
		if(USBD_STM32_REG->EP_REG[ep_number] & (USBDEP_STM32_CTR_RX | USBDEP_STM32_CTR_TX)) {
			if(USBDEP_hc_irq[ep_number]) {
				USBDEP_hc_irq[ep_number](ep_number);
			}
		}

		ep_number++;
	}while((USBD_STM32_REG->ISTR & USBD_ISTR_CTR) && (ep_number < USBD_MAX_NUM_ENDPOINTS));
#endif


	USBD_EXIT_IRQ(USBD_STM32_IRQ_CHANELL_HP);
}



void USB_LP_CAN1_RX0_IRQHandler(void) {
	USBDEP_params *EP;
	uint8_t counter;
	uint32_t ep_reg_mirror;
#if(USBD_MAX_NUM_ENDPOINTS > 1)
	uint8_t ep_number;
#endif

#if (((USBDE_RESET_SUPPORT == USBD_FEATURE_ON) || (USBDE_SUSPENDED_SUPPORT == USBD_FEATURE_ON) || (USBDE_SOF_SUPPORT == USBD_FEATURE_ON)) &&  (USBD_MAX_NUM_EVENTS > 0))
	uint8_t events_counter;
	USBDE_params *USBDE;
#endif


	USBD_ENTER_IRQ(USBD_STM32_IRQ_CHANELL_LP);


	// sprawdzenie, czy przerwanie zostalo spowodowane resetem
	if(USBD_STM32_REG->ISTR & (uint32_t)USBD_ISTR_RESET) {
		USBD_STM32_REG->ISTR &= ~((uint32_t)USBD_ISTR_RESET);
		USBD_STM32_REG->DADDR = USBD_DADDR_EF;

#if((USBDEP_BULK_SUPPORT == USBD_FEATURE_ON) && (USBD_MAX_NUM_ENDPOINTS > 1))
		USBDEP_HP_state = 0;
#endif

		USBD_STM32_REG->EP_REG[0] &= USBDEP_STM32_DTOG_RX | USBDEP_STM32_STAT_RX | USBDEP_STM32_DTOG_TX | USBDEP_STM32_STAT_TX;
		USBD_STM32_REG->EP_REG[1] &= USBDEP_STM32_DTOG_RX | USBDEP_STM32_STAT_RX | USBDEP_STM32_DTOG_TX | USBDEP_STM32_STAT_TX;
		USBD_STM32_REG->EP_REG[2] &= USBDEP_STM32_DTOG_RX | USBDEP_STM32_STAT_RX | USBDEP_STM32_DTOG_TX | USBDEP_STM32_STAT_TX;
		USBD_STM32_REG->EP_REG[3] &= USBDEP_STM32_DTOG_RX | USBDEP_STM32_STAT_RX | USBDEP_STM32_DTOG_TX | USBDEP_STM32_STAT_TX;
		USBD_STM32_REG->EP_REG[4] &= USBDEP_STM32_DTOG_RX | USBDEP_STM32_STAT_RX | USBDEP_STM32_DTOG_TX | USBDEP_STM32_STAT_TX;
		USBD_STM32_REG->EP_REG[5] &= USBDEP_STM32_DTOG_RX | USBDEP_STM32_STAT_RX | USBDEP_STM32_DTOG_TX | USBDEP_STM32_STAT_TX;
		USBD_STM32_REG->EP_REG[6] &= USBDEP_STM32_DTOG_RX | USBDEP_STM32_STAT_RX | USBDEP_STM32_DTOG_TX | USBDEP_STM32_STAT_TX;
		USBD_STM32_REG->EP_REG[7] &= USBDEP_STM32_DTOG_RX | USBDEP_STM32_STAT_RX | USBDEP_STM32_DTOG_TX | USBDEP_STM32_STAT_TX;

		// wyczyszczenie parametrow transmisji
		for (counter = 0; counter < USBD_MAX_NUM_ENDPOINTS; counter++) {
			EP = &USBD_EP_params[counter];
			// wypelnienie struktury opisujacej endpoint
			EP->number = counter;
			EP->max_packet_size = 0;
			EP->status = 0;
			EP->interface = 0;
			EP->endpoint_descriptor = 0;
			EP->transfer_params.data = 0;
			EP->transfer_params.data_left_length = 0;
			EP->transfer_params.data_buffered_length = 0;
			EP->transfer_params.data_transferred_length = 0;
			EP->transfer_params.transfer_type = USBD_IDLE;
			EP->transfer_params.transfer_state = USBD_IDLE_OR_BACKGROUND;
			USBDEP_hc_irq[counter] = 0;

			memset(&USBDEP_private[counter], 0, sizeof(USBDEP_private_params));
		}

		// ustawienie rozmiaru bufora punktu zerowego
		USBD_EP_params[0].max_packet_size = USBDEP_0_MAX_POSSIBLE_PACKET_SIZE;
		USBDEP_private[0].num_fifos = 1;
		USBDEP_private[0].fifo_counter = 0;
		USBDEP_private[0].fifo_data_size[0] = 0;
		USBDEP_private[0].fifo_data_size[1] = 0;
		USBDEP_private[0].fifo_OUT_data_position = 0;
		USBDEP_private[0].partial_read_additional_byte = 0;
		USBDEP_private[0].partial_read = 0;

		// ustawienie nowego statusu USBD
		USBD_status_masked = USBDST_ATTACHED | USBDST_POWERED | USBDST_DEFAULT;
		USBD_status        = USBDST_ATTACHED | USBDST_POWERED | USBDST_DEFAULT;

		if(pUSBDC_active) {
			pUSBDC_active->usbd_status = &USBD_status_masked;
			pUSBDC_active = 0;
		}
		USBD_STM32_REG->BTABLE = 0;

		// ustawienie adresow buforow w przestrzeni
		// adresowej tuz za tablica buforow
		USBDEP_set_tx_buf_addr(0, 64);
		USBDEP_set_rx_buf_addr(0, (USBD_STM32_EPMEM[0].SB.tx_addr + USBDEP_0_MAX_POSSIBLE_PACKET_SIZE));
		USBDEP_set_tx_buf_addr(1, 0);
		USBDEP_set_rx_buf_addr(1, 0);
		USBDEP_set_tx_buf_addr(2, 0);
		USBDEP_set_rx_buf_addr(2, 0);
		USBDEP_set_tx_buf_addr(3, 0);
		USBDEP_set_rx_buf_addr(3, 0);
		USBDEP_set_tx_buf_addr(4, 0);
		USBDEP_set_rx_buf_addr(4, 0);
		USBDEP_set_tx_buf_addr(5, 0);
		USBDEP_set_rx_buf_addr(5, 0);
		USBDEP_set_tx_buf_addr(6, 0);
		USBDEP_set_rx_buf_addr(6, 0);
		USBDEP_set_tx_buf_addr(7, 0);
		USBDEP_set_rx_buf_addr(7, 0);
		ep_reg_mirror = USBDEP_0_MAX_POSSIBLE_PACKET_SIZE;
		USBDEP_SB_set_rx_max_possible_count(0, ep_reg_mirror);


		// ustawienia:
		// EP_TYPE = USBDEP_STM32_CONTROL
		// EP_KIND = 0
		// STAT_TX = USBDEP_STM32_TX_NAK
		// STAT_RX = USBDEP_STM32_RX_VALID
		// EPADDR  = 0
		USBDEP_reg_modify(0,
				ep_reg_mirror,
				USBDEP_STM32_RX_VALID | USBDEP_STM32_TX_NAK,
				USBDEP_STM32_STAT_RX | USBDEP_STM32_STAT_TX,
				USBDEP_STM32_CONTROL);


#if(USBDE_SOF_SUPPORT == USBD_FEATURE_OFF)
		USBD_STM32_REG->CNTR = USBD_CNTR_CTRM | USBD_CNTR_RESETM | USBD_CNTR_SUSPM;
#else
		USBD_STM32_REG->CNTR = USBD_CNTR_CTRM | USBD_CNTR_RESETM | USBD_CNTR_SUSPM | USBD_CNTR_SOFM | USBD_CNTR_ESOFM;
#endif

#if ((USBDE_RESET_SUPPORT == USBD_FEATURE_ON) &&  (USBD_MAX_NUM_EVENTS > 0))
		USBD_private_data.session_activity &= 0xF;

#if(USBD_MAX_NUM_STRINGS > 0)
		// wyzerowanie numerow wszystkich zainstalowanych stringow
		for(counter = 0; counter < USBD_private_data.installed_strings_count; counter++) {
			if(USBD_private_data.strings[counter].index_string) {
				*(USBD_private_data.strings[counter].index_string) = 0;
			}
		}
#endif

#if(USBD_ZERO_CONFIG_SUPPORT == USBD_FEATURE_ON)
			if(USBD_configurations[0]) {
				counter=0;
			}
			else {
				counter=1;
			}
			for(; counter <= USBD_Device_Descriptor.bNumConfigurations; counter++) {
#else
			for(counter=1; counter <= USBD_Device_Descriptor.bNumConfigurations; counter++) {
#endif
			USBDE = USBD_configurations[counter]->events;
			for(events_counter = 0; events_counter < USBD_MAX_NUM_EVENTS; events_counter++) {
				if(USBDE[events_counter].active_events & USBDEM_RESET) {
					if(USBDE[events_counter].event) {
						(*USBDE[events_counter].event) (USBD_configurations[counter], USBDEF_RESET, &USBDE[events_counter]);
					}
				}
			}
		}
#endif


#if(USBD_MAX_NUM_STRINGS > 0)
		// ponumerowanie wszystkich zainstalowanych stringow
		for(counter = 0; counter < USBD_private_data.installed_strings_count; counter++) {
			if(USBD_private_data.strings[counter].index_string) {
				*(USBD_private_data.strings[counter].index_string) = counter;
			}
		}
#endif

		USBD_private_data.session_activity |= 0xF0;

	}
	

	else {
		// sprawdzenie czy przerwanie nie dotyczy transmisji danych
		if(USBD_STM32_REG->ISTR & USBD_ISTR_CTR) {

#if(USBD_MAX_NUM_ENDPOINTS > 1)
			ep_number = 0;
			do {
				// sprawdzenie, czy przerwanie dotyczny transmisji danych i czy odnosi sie do danego endpointa
				if(USBD_STM32_REG->EP_REG[ep_number] & (USBDEP_STM32_CTR_RX | USBDEP_STM32_CTR_TX)) {

#if(USBDEP_INTERRUPT_SUPPORT == USBD_FEATURE_ON)
					if((USBDEP_private[ep_number].EP_type == USBDEP_DESC_TRANSFER_TYPE_CONTROL)
							|| (USBDEP_private[ep_number].EP_type == USBDEP_DESC_TRANSFER_TYPE_INTERRUPT)) {
						// OUT
						if(USBD_STM32_REG->EP_REG[ep_number] & USBDEP_STM32_CTR_RX) {
							USBD_EP_OUT_SB_irq(ep_number);
						}
						// IN
						else {
							USBD_EP_IN_SB_irq(ep_number);
						}
					}
#else
					if(USBDEP_private[ep_number].EP_type == USBDEP_DESC_TRANSFER_TYPE_CONTROL) {
						// OUT
						if(USBD_STM32_REG->EP_REG[ep_number] & USBDEP_STM32_CTR_RX) {
							USBD_EP_OUT_SB_irq(ep_number);
						}
						// IN
						else {
							USBD_EP_IN_SB_irq(ep_number);
						}
					}
#endif
				}

				ep_number++;
			}while((USBD_STM32_REG->ISTR & USBD_ISTR_CTR) && (ep_number < USBD_MAX_NUM_ENDPOINTS));


#else


			// OUT
			if(USBD_STM32_REG->ISTR & USBD_ISTR_DIR) {
				USBD_EP_OUT_SB_irq(0);
			}
			// IN
			else {
				USBD_EP_IN_SB_irq(0);
			}


#endif
		}
		// sprawdzenie czy przerwanie nie dotyczy innych rzeczy
		if(USBD_STM32_REG->ISTR & (USBD_ISTR_SUSP | USBD_ISTR_SOF | USBD_ISTR_ESOF | USBD_ISTR_PMAOVR | USBD_ISTR_ERR)) {
			// sprawdzenie, czy przerwanie nie dotyczy wejscia w tryb uspienia
			if(USBD_STM32_REG->ISTR & USBD_ISTR_SUSP) {
				USBD_STM32_REG->ISTR &= ~((uint32_t)USBD_ISTR_SUSP);
				USBD_STM32_REG->CNTR &= ~((uint32_t)USBD_CNTR_SUSPM);
				USBD_STM32_REG->CNTR |= USBD_CNTR_WKUPM | USBD_CNTR_FSUSP;
				USBD_STM32_REG->CNTR |= USBD_CNTR_LPMODE;

				// ustawienie nowego statusu USBD
				USBD_status_masked |= USBDST_SUSPENDED;
				USBD_status        |= USBDST_SUSPENDED;

#if ((USBDE_SUSPENDED_SUPPORT == USBD_FEATURE_ON) && (USBD_MAX_NUM_EVENTS > 0))
#if(USBD_ZERO_CONFIG_SUPPORT == USBD_FEATURE_ON)
					if(USBD_configurations[0]) {
						counter=0;
					}
					else {
						counter=1;
					}
					for(; counter <= USBD_Device_Descriptor.bNumConfigurations; counter++) {
#else
					for(counter=1; counter <= USBD_Device_Descriptor.bNumConfigurations; counter++) {
#endif
					USBDE = USBD_configurations[counter]->events;
					for(events_counter = 0; events_counter < USBD_MAX_NUM_EVENTS; events_counter++) {
						if(USBDE[events_counter].active_events & USBDEM_SUSPENDED) {
							if(USBDE[events_counter].event) {
								(*USBDE[events_counter].event) (USBD_configurations[counter], USBDEF_SUSPENDED, &USBDE[events_counter]);
							}
						}
					}
				}
#endif

			}
			if(USBD_STM32_REG->ISTR & (USBD_ISTR_SOF | USBD_ISTR_ESOF)) {
				USBD_STM32_REG->ISTR &= ~((uint32_t)(USBD_ISTR_SOF | USBD_ISTR_ESOF));

#if ((USBDE_SOF_SUPPORT == USBD_FEATURE_ON) && (USBD_MAX_NUM_EVENTS > 0))
				if(pUSBDC_active) {
					events_counter = 0;
					do {
						USBDE = &(pUSBDC_active->events[events_counter]);
						if(USBDE->active_events & USBDEM_SOF) {
							if(USBDE->event) {
								(*USBDE->event) (pUSBDC_active, USBDEF_SOF, USBDE);
							}
						}
						events_counter++;
					} while(events_counter < USBD_MAX_NUM_EVENTS);
				}
#endif
			}
			if(USBD_STM32_REG->ISTR & (USBD_ISTR_PMAOVR | USBD_ISTR_ERR)) {
				USBD_STM32_REG->ISTR &= ~((uint32_t)(USBD_ISTR_PMAOVR | USBD_ISTR_ERR));
// TODO:
			}
		}
	}


	USBD_EXIT_IRQ(USBD_STM32_IRQ_CHANELL_LP);
}







void USBWakeUp_IRQHandler(void) {
#if ((USBDE_WAKEUP_SUPPORT == USBD_FEATURE_ON) &&  (USBD_MAX_NUM_EVENTS > 0))
	uint8_t counter;
	uint8_t events_counter;
	USBDE_params *USBDE;
#endif


	USBD_ENTER_IRQ(USBD_STM32_IRQ_CHANELL_WAKE_UP);


#if (((USBDE_WAKEUP_SUPPORT == USBD_FEATURE_ON) || (USBDE_SUSPENDED_SUPPORT == USBD_FEATURE_ON)) &&  (USBD_MAX_NUM_EVENTS > 0))
	USBD_clear_wakeup_exti_status();
#endif

	// sprawdzenie, czy przerwanie nie dotyczy wybudzenia z trybu uspienia
	if(USBD_STM32_REG->ISTR & USBD_ISTR_WKUP) {
		USBD_STM32_REG->ISTR &= ~((uint32_t)USBD_ISTR_WKUP);
		USBD_STM32_REG->CNTR &= ~((uint32_t)USBD_ISTR_WKUP);
		USBD_STM32_REG->CNTR |= USBD_CNTR_SUSPM;

		// ustawienie nowego statusu USBD
		USBD_status_masked &= ~((uint8_t)USBDST_SUSPENDED);
		USBD_status        &= ~((uint8_t)USBDST_SUSPENDED);

#if ((USBDE_WAKEUP_SUPPORT == USBD_FEATURE_ON) && (USBD_MAX_NUM_EVENTS > 0))
#if(USBD_ZERO_CONFIG_SUPPORT == USBD_FEATURE_ON)
			if(USBD_configurations[0]) {
				counter=0;
			}
			else {
				counter=1;
			}
			for(; counter <= USBD_Device_Descriptor.bNumConfigurations; counter++) {
#else
			for(counter=1; counter <= USBD_Device_Descriptor.bNumConfigurations; counter++) {
#endif
			USBDE = USBD_configurations[counter]->events;
			for(events_counter = 0; events_counter < USBD_MAX_NUM_EVENTS; events_counter++) {
				if(USBDE[events_counter].active_events & USBDEM_WAKEUP) {
					if(USBDE[events_counter].event) {
						(*USBDE[events_counter].event) (USBD_configurations[counter], USBDEF_WAKEUP, &USBDE[events_counter]);
					}
				}
			}
		}
#endif
	}


	USBD_EXIT_IRQ(USBD_STM32_IRQ_CHANELL_WAKE_UP);
}








#if(USBD_VBUS_DETECT_SUPPORT == USBD_FEATURE_ON)

void USBD_Attach_irq_handler(void) {
	USBDEP_params *EP;
	uint8_t counter;
#if (((USBDE_ATTACHED_SUPPORT == USBD_FEATURE_ON) || (USBDE_DETACHED_SUPPORT == USBD_FEATURE_ON)) && (USBD_MAX_NUM_EVENTS > 0))
		uint8_t events_counter;
		USBDE_params *USBDE;
#endif

	// sprawdzenie, czy na linii VBUS jest 5V czy 0V
	if(USBD_device_is_powered()) {
		// wyczyszczenie parametrow transmisji
		for (counter = 0; counter < USBD_MAX_NUM_ENDPOINTS; counter++) {
			EP = &USBD_EP_params[counter];
			// wypelnienie struktury opisujacej endpoint
			EP->number = counter;
			EP->max_packet_size = 0;
			EP->status = 0;
			EP->interface = 0;
			EP->endpoint_descriptor = 0;
			EP->transfer_params.data = 0;
			EP->transfer_params.data_left_length = 0;
			EP->transfer_params.data_buffered_length = 0;
			EP->transfer_params.data_transferred_length = 0;
			EP->transfer_params.transfer_type = USBD_IDLE;
			EP->transfer_params.transfer_state = USBD_IDLE_OR_BACKGROUND;
		}

		// ustawienie rozmiaru bufora punktu zerowego
		USBD_EP_params[0].max_packet_size = USBDEP_0_MAX_POSSIBLE_PACKET_SIZE;

#if ((USBDE_ATTACHED_SUPPORT == USBD_FEATURE_ON) && (USBD_MAX_NUM_EVENTS > 0))

		USBD_status_masked = USBDST_ATTACHED;
		USBD_status        = USBDST_ATTACHED;

		// wywolanie funkcji event z flaga USBDEF_ATTACHED
#if(USBD_ZERO_CONFIG_SUPPORT == USBD_FEATURE_ON)
		if(USBD_configurations[0]) {
			counter=0;
		}
		else {
			counter=1;
		}
		for(; counter <= USBD_Device_Descriptor.bNumConfigurations; counter++) {
#else
		for(counter=1; counter <= USBD_Device_Descriptor.bNumConfigurations; counter++) {
#endif
			USBDE = USBD_configurations[counter]->events;
			for(events_counter = 0; events_counter < USBD_MAX_NUM_EVENTS; events_counter++) {
				if(USBDE[events_counter].active_events & USBDEM_ATTACHED) {
					if(USBDE[events_counter].event) {
						(*USBDE[events_counter].event) (USBD_configurations[counter], USBDEF_ATTACHED, &USBDE[events_counter]);
					}
				}
			}
		}
#endif

		USBD_status_masked = USBDST_ATTACHED | USBDST_POWERED;
		USBD_status        = USBDST_ATTACHED | USBDST_POWERED;

#if ((USBDE_POWERED_SUPPORT == USBD_FEATURE_ON) && (USBD_MAX_NUM_EVENTS > 0))
		// wywolanie funkcji event z flaga USBDEF_POWERED
#if(USBD_ZERO_CONFIG_SUPPORT == USBD_FEATURE_ON)
		if(USBD_configurations[0]) {
			counter=0;
		}
		else {
			counter=1;
		}
		for(; counter <= USBD_Device_Descriptor.bNumConfigurations; counter++) {
#else
		for(counter=1; counter <= USBD_Device_Descriptor.bNumConfigurations; counter++) {
#endif
			USBDE = USBD_configurations[counter]->events;
			for(events_counter = 0; events_counter < USBD_MAX_NUM_EVENTS; events_counter++) {
				if(USBDE[events_counter].active_events & USBDEM_POWERED) {
					if(USBDE[events_counter].event) {
						(*USBDE[events_counter].event) (USBD_configurations[counter], USBDEF_POWERED, &USBDE[events_counter]);
					}
				}
			}
		}
#endif

		USBD_power_up();


#if(USBD_DP_PULL_UP_SUPPORT == USBD_FEATURE_ON)
		// wlaczenie rezystora na linii D+
		USBD_DP_PULL_UP_enable();
#endif


	}
	else {


#if(USBD_DP_PULL_UP_SUPPORT == USBD_FEATURE_ON)
		// wylaczenie rezystora na linii D+
		USBD_DP_PULL_UP_disable();
#endif

		if(pUSBDC_active) {
			pUSBDC_active->usbd_status = &USBD_status_masked;
			pUSBDC_active = 0;
		}
		USBDC_active_number = 0;
		
		USBD_power_down();

		// wyczyszczenie parametrow transmisji
		for (counter = 0; counter < USBD_MAX_NUM_ENDPOINTS; counter++) {
			EP = &USBD_EP_params[counter];
			// wypelnienie struktury opisujacej endpoint
			EP->number = counter;
			EP->max_packet_size = 0;
			EP->status = 0;
			EP->interface = 0;
			EP->endpoint_descriptor = 0;
			EP->transfer_params.data = 0;
			EP->transfer_params.data_left_length = 0;
			EP->transfer_params.data_buffered_length = 0;
			EP->transfer_params.data_transferred_length = 0;
			EP->transfer_params.transfer_type = USBD_IDLE;
			EP->transfer_params.transfer_state = USBD_IDLE_OR_BACKGROUND;
		}

		// ustawienie rozmiaru bufora punktu zerowego
		USBD_EP_params[0].max_packet_size = USBDEP_0_MAX_POSSIBLE_PACKET_SIZE;

		USBD_private_data.session_activity &= 0xF;

#if(USBD_MAX_NUM_STRINGS > 0)
		// wyzerowanie numerow wszystkich zainstalowanych stringow
		for(counter = 0; counter < USBD_private_data.installed_strings_count; counter++) {
			if(USBD_private_data.strings[counter].index_string) {
				*(USBD_private_data.strings[counter].index_string) = 0;
			}
		}
#endif

#if ((USBDE_UNPOWERED_SUPPORT == USBD_FEATURE_ON) && (USBD_MAX_NUM_EVENTS > 0))

		USBD_status_masked = USBDST_ATTACHED;
		USBD_status        = USBDST_ATTACHED;

		// wywolanie funkcji event z flaga USBDEF_UNPOWERED
#if(USBD_ZERO_CONFIG_SUPPORT == USBD_FEATURE_ON)
		if(USBD_configurations[0]) {
			counter=0;
		}
		else {
			counter=1;
		}
		for(; counter <= USBD_Device_Descriptor.bNumConfigurations; counter++) {
#else
		for(counter=1; counter <= USBD_Device_Descriptor.bNumConfigurations; counter++) {
#endif
			USBDE = USBD_configurations[counter]->events;
			for(events_counter = 0; events_counter < USBD_MAX_NUM_EVENTS; events_counter++) {
				if(USBDE[events_counter].active_events & USBDEM_UNPOWERED) {
					if(USBDE[events_counter].event) {
						(*USBDE[events_counter].event) (USBD_configurations[counter], USBDEF_UNPOWERED, &USBDE[events_counter]);
					}
				}
			}
		}
#endif

		USBD_status_masked = 0;
		USBD_status        = 0;

#if ((USBDE_DETACHED_SUPPORT == USBD_FEATURE_ON) && (USBD_MAX_NUM_EVENTS > 0))

		// wywolanie funkcji event z flaga USBDEF_DETACHED
#if(USBD_ZERO_CONFIG_SUPPORT == USBD_FEATURE_ON)
		if(USBD_configurations[0]) {
			counter=0;
		}
		else {
			counter=1;
		}
		for(; counter <= USBD_Device_Descriptor.bNumConfigurations; counter++) {
#else
		for(counter=1; counter <= USBD_Device_Descriptor.bNumConfigurations; counter++) {
#endif
			USBDE = USBD_configurations[counter]->events;
			for(events_counter = 0; events_counter < USBD_MAX_NUM_EVENTS; events_counter++) {
				if(USBDE[events_counter].active_events & USBDEM_DETACHED) {
					if(USBDE[events_counter].event) {
						(*USBDE[events_counter].event) (USBD_configurations[counter], USBDEF_DETACHED, &USBDE[events_counter]);
					}
				}
			}
		}
#endif
	}
}

#endif







void USBD_activate (USBDC_params *usbdc) {
	USBDEP_params *EP;
	uint8_t counter;
#if ((USBDE_ATTACHED_SUPPORT == USBD_FEATURE_ON) && (USBD_MAX_NUM_EVENTS > 0) && (USBD_VBUS_DETECT_SUPPORT == USBD_FEATURE_OFF))
		uint8_t events_counter;
		USBDE_params *USBDE;
#endif

	// wyczyszczenie parametrow transmisji
	for (counter = 0; counter < USBD_MAX_NUM_ENDPOINTS; counter++) {
		EP = &USBD_EP_params[counter];
		// wypelnienie struktury opisujacej endpoint
		EP->number = counter;
		EP->max_packet_size = 0;
		EP->status = 0;
		EP->interface = 0;
		EP->endpoint_descriptor = 0;
		EP->transfer_params.data = 0;
		EP->transfer_params.data_left_length = 0;
		EP->transfer_params.data_buffered_length = 0;
		EP->transfer_params.data_transferred_length = 0;
		EP->transfer_params.transfer_type = USBD_IDLE;
		EP->transfer_params.transfer_state = USBD_IDLE_OR_BACKGROUND;
	}

	// ustawienie rozmiaru bufora punktu zerowego
	USBD_EP_params[0].max_packet_size = USBDEP_0_MAX_POSSIBLE_PACKET_SIZE;

#if(USBD_VBUS_DETECT_SUPPORT == USBD_FEATURE_ON)
	//wlaczenie przerwania wykrywajacego stan linii VBUS
	USBD_VBUS_DETECT_reconfigure_irq(USBD_FEATURE_ON);

	// sprawdzenie polaczenia z Hostem
	USBD_Attach_irq_handler();
    
#else

#if ((USBDE_ATTACHED_SUPPORT == USBD_FEATURE_ON) && (USBD_MAX_NUM_EVENTS > 0))

	USBD_status_masked = USBDST_ATTACHED;
	USBD_status        = USBDST_ATTACHED;

		// wywolanie funkcji event z flaga USBDEF_ATTACHED
#if(USBD_ZERO_CONFIG_SUPPORT == USBD_FEATURE_ON)
		if(USBD_configurations[0]) {
			counter=0;
		}
		else {
			counter=1;
		}
		for(; counter <= USBD_Device_Descriptor.bNumConfigurations; counter++) {
#else
		for(counter=1; counter <= USBD_Device_Descriptor.bNumConfigurations; counter++) {
#endif
			USBDE = USBD_configurations[counter]->events;
			for(events_counter = 0; events_counter < USBD_MAX_NUM_EVENTS; events_counter++) {
				if(USBDE[events_counter].active_events & USBDEM_ATTACHED) {
					if(USBDE[events_counter].event) {
						(*USBDE[events_counter].event) (USBD_configurations[counter], USBDEF_ATTACHED, &USBDE[events_counter]);
					}
				}
			}
		}
#endif

		USBD_status_masked = USBDST_ATTACHED | USBDST_POWERED;
		USBD_status        = USBDST_ATTACHED | USBDST_POWERED;

#if ((USBDE_POWERED_SUPPORT == USBD_FEATURE_ON) && (USBD_MAX_NUM_EVENTS > 0))
		// wywolanie funkcji event z flaga USBDEF_POWERED
#if(USBD_ZERO_CONFIG_SUPPORT == USBD_FEATURE_ON)
		if(USBD_configurations[0]) {
			counter=0;
		}
		else {
			counter=1;
		}
		for(; counter <= USBD_Device_Descriptor.bNumConfigurations; counter++) {
#else
		for(counter=1; counter <= USBD_Device_Descriptor.bNumConfigurations; counter++) {
#endif
			USBDE = USBD_configurations[counter]->events;
			for(events_counter = 0; events_counter < USBD_MAX_NUM_EVENTS; events_counter++) {
				if(USBDE[events_counter].active_events & USBDEM_POWERED) {
					if(USBDE[events_counter].event) {
						(*USBDE[events_counter].event) (USBD_configurations[counter], USBDEF_POWERED, &USBDE[events_counter]);
					}
				}
			}
		}
#endif

	USBD_power_up();
	

#if(USBD_DP_PULL_UP_SUPPORT == USBD_FEATURE_ON)
	// wlaczenie rezystora na linii D+
	USBD_DP_PULL_UP_enable();
#endif


#endif
}







#if(USBD_MULTI_SESSION_SUPPORT == USBD_FEATURE_ON)
void USBD_deactivate (USBDC_params *usbdc) {
	USBDEP_params *EP;
	uint8_t counter;
#if ((USBDE_DETACHED_SUPPORT == USBD_FEATURE_ON) && (USBD_MAX_NUM_EVENTS > 0))
	uint8_t events_counter;
	USBDE_params *USBDE;
#endif

#if(USBD_VBUS_DETECT_SUPPORT == USBD_FEATURE_ON)
	//wylaczenie przerwania wykrywajacego stan linii VBUS
	USBD_VBUS_DETECT_reconfigure_irq(USBD_FEATURE_OFF);
#endif


#if(USBD_DP_PULL_UP_SUPPORT == USBD_FEATURE_ON)
	// wylaczenie rezystora na linii D+
	USBD_DP_PULL_UP_disable();
#endif

	USBD_status_masked = 0;
	USBD_status        = 0;

	if(pUSBDC_active) {
		pUSBDC_active->usbd_status = &USBD_status_masked;
		pUSBDC_active = 0;
	}
	USBDC_active_number = 0;
	
	USBD_power_down();

	// wyczyszczenie parametrow transmisji
	for (counter = 0; counter < USBD_MAX_NUM_ENDPOINTS; counter++) {
		EP = &USBD_EP_params[counter];
		// wypelnienie struktury opisujacej endpoint
		EP->number = counter;
		EP->max_packet_size = 0;
		EP->status = 0;
		EP->interface = 0;
		EP->endpoint_descriptor = 0;
		EP->transfer_params.data = 0;
		EP->transfer_params.data_left_length = 0;
		EP->transfer_params.data_buffered_length = 0;
		EP->transfer_params.data_transferred_length = 0;
		EP->transfer_params.transfer_type = USBD_IDLE;
		EP->transfer_params.transfer_state = USBD_IDLE_OR_BACKGROUND;
	}

	// ustawienie rozmiaru bufora punktu zerowego
	USBD_EP_params[0].max_packet_size = USBDEP_0_MAX_POSSIBLE_PACKET_SIZE;

	USBD_private_data.session_activity &= 0xF;

#if(USBD_MAX_NUM_STRINGS > 0)
	// wyzerowanie numerow wszystkich zainstalowanych stringow
	for(counter = 0; counter < USBD_private_data.installed_strings_count; counter++) {
		if(USBD_private_data.strings[counter].index_string) {
			*(USBD_private_data.strings[counter].index_string) = 0;
		}
	}
#endif

#if ((USBDE_DETACHED_SUPPORT == USBD_FEATURE_ON) && (USBD_MAX_NUM_EVENTS > 0))
	// wywolanie funkcji event z flaga USBDEF_DETACHED
#if(USBD_ZERO_CONFIG_SUPPORT == USBD_FEATURE_ON)
	if(USBD_configurations[0]) {
		counter=0;
	}
	else {
		counter=1;
	}
	for(; counter <= USBD_Device_Descriptor.bNumConfigurations; counter++) {
#else
	for(counter=1; counter <= USBD_Device_Descriptor.bNumConfigurations; counter++) {
#endif
		USBDE = USBD_configurations[counter]->events;
		for(events_counter = 0; events_counter < USBD_MAX_NUM_EVENTS; events_counter++) {
			if(USBDE[events_counter].active_events & USBDEM_DETACHED) {
				if(USBDE[events_counter].event) {
					(*USBDE[events_counter].event) (USBD_configurations[counter], USBDEF_DETACHED, &USBDE[events_counter]);
				}
			}
		}
	}
#endif

}
#endif






uint16_t USBD_get_frame_number(USBDC_params *usbdc) {
	return(USBD_STM32_REG->FNR & 0x7FF);
}








static void USBD_port_initialize(void) {
	USBDEP_params *EP;
	uint8_t counter;

	USBD_first_installation_processed = 0x55;

	// wyczyszczenie parametrow transmisji
	for (counter = 0; counter < USBD_MAX_NUM_ENDPOINTS; counter++) {
		EP = &USBD_EP_params[counter];
		// wypelnienie struktury opisujacej endpoint
		EP->number = counter;
		EP->max_packet_size = 0;
		EP->status = 0;
		EP->interface = 0;
		EP->endpoint_descriptor = 0;
		EP->transfer_params.data = 0;
		EP->transfer_params.data_left_length = 0;
		EP->transfer_params.data_buffered_length = 0;
		EP->transfer_params.data_transferred_length = 0;
		EP->transfer_params.transfer_type = USBD_IDLE;
		EP->transfer_params.transfer_state = USBD_IDLE_OR_BACKGROUND;
	}

	// ustawienie rozmiaru bufora punktu zerowego
	USBD_EP_params[0].max_packet_size = USBDEP_0_MAX_POSSIBLE_PACKET_SIZE;

	pUSBDC_active = 0;
	
	USBD_low_level_init();

	memset((void*)(&USBD_private_data), 0, sizeof(USBD_params));

	USBD_private_data.session_activity = 0x0F;
	USBD_private_data.installed_strings_count = 1;
	USBD_private_data.device_descriptor = &USBD_Device_Descriptor;
	USBD_private_data.strings[0].string = (uint8_t*)USBD_Language_String_Descriptor;
}








void USBDC_bind_to_port(void *usbd, USBDC_params *usbdc) {
	// inicjalizacja USBD
	if(!USBD_first_installation_processed) {
		USBD_port_initialize();

#if(USBD_ZERO_CONFIG_SUPPORT == USBD_FEATURE_ON)
		if(!USBDZC_state) {
			USBD_configurations[0] = 0;
		}
#endif
	}

	USBD_install_port(&USBD_private_data, usbdc);
}








uint8_t USBDC_install (USBDC_params *usbdc) {
	// inicjalizacja USBD
	if(!USBD_first_installation_processed) {
		USBD_port_initialize();

#if(USBD_ZERO_CONFIG_SUPPORT == USBD_FEATURE_ON)
		if(!USBDZC_state) {
			USBD_configurations[0] = 0;
		}
#endif
	}

	// instalacja konfiguracji
	if(!(USBD_private_data.session_activity & 0xF0)) {
		if(USBD_Device_Descriptor.bNumConfigurations < USBD_MAX_NUM_CONFIGURATIONS) {
			USBD_Device_Descriptor.bNumConfigurations++;
			USBD_configurations[(USBD_Device_Descriptor.bNumConfigurations)] = usbdc;
			usbdc->usbd_status = &USBD_status_masked;

			USBD_install_port(&USBD_private_data, usbdc);

			return (USBD_Device_Descriptor.bNumConfigurations);
		}
		else {
			return (0);
		}
	}
	return(0);
}






#if(USBD_ZERO_CONFIG_SUPPORT == USBD_FEATURE_ON)
void USBDZC_install (USBDC_params *usbdc) {
	// inicjalizacja USBD
	if(!USBD_first_installation_processed) {
		USBD_port_initialize();
	}

	USBD_install_port(&USBD_private_data, usbdc);

	USBDZC_state = 0xF;
	USBD_private_data.configurations[0] = usbdc;
	usbdc->usbd_status = &USBD_status_masked;
}
#endif
